@@ -5,10 +5,9 @@
 ;; DESCRIPTION : Common tools to import XML data.
 ;; COPYRIGHT   : (C) 2002-2003  David Allouche
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -135,7 +134,10 @@
   (cork-grave->backquote (utf8->cork s)))
 
 (tm-define (xmltm-url-text s)
-  (cork-grave->backquote (utf8->cork (url-decode s))))
+  ;; (cork-grave->backquote (utf8->cork (url-decode s)))
+  ;; NOTE: don't decode URL names, or also implement a corresponding
+  ;; routine for the encoding, when we click on a hyperlink
+  (string-encode s))
 
 ;; Conversion of Cork GRAVE ACCENT to LEFT SINGLE QUOTATION MARK
 
