@@ -5,10 +5,9 @@
 ;; DESCRIPTION : routines for switching, folding and layers
 ;; COPYRIGHT   : (C) 2002  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -66,7 +65,8 @@
 ;; Operations on toggle trees
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (toggle-toggle t)
+(tm-define (toggle-toggle t)
+  (:synopsis "Toggle a fold/unfold")
   (tree-assign-node! t (ahash-ref toggle-table (tree-label t))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -90,6 +90,10 @@
 (tm-define (fold)
   (:type (-> void))
   (:synopsis "Fold at the current cursor position")
+  (noop))
+
+(tm-define (fold)
+  (:context toggle-second-context?)
   (with-innermost t toggle-second-context?
     (toggle-toggle t)
     (tree-go-to t 0 :start)))
@@ -97,6 +101,10 @@
 (tm-define (unfold)
   (:type (-> void))
   (:synopsis "Unfold at the current cursor position")
+  (noop))
+
+(tm-define (unfold)
+  (:context toggle-first-context?)
   (with-innermost t toggle-first-context?
     (toggle-toggle t)
     (tree-go-to t 1 :start)))
@@ -223,7 +231,7 @@
     (when (and (>= i 0) (<= i (tree-arity t)))
       (let* ((empty (if (tree-in? t (big-switch-tag-list)) '(document "") ""))
 	     (v (switch-index :visible)))
-	(tree-insert! t i `(,(tree-label t) (shown ,empty)))
+	(tree-insert! t i `((shown ,empty)))
 	(if (tree-in? t (alternative-tag-list))
 	    (switch-select i)
 	    (switch-select (+ v 1)))
@@ -483,6 +491,29 @@
   (if (in? mode '(:first :var-first)) (tree-go-to (buffer-tree) :start))
   (if (in? mode '(:last :var-last)) (tree-go-to (buffer-tree) :end)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Transform presentation into slides
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (dynamic-make-slide t)
+  (when (or (tm-func? t 'shown 1) (tm-func? t 'hidden 1))
+    (tree-assign-node! t 'slide))
+  (dynamic-operate (tree-ref t 0) :var-expand))
+
+(tm-define (dynamic-make-slides)
+  (init-remove-package "presentation")
+  (init-add-package "slides")
+  (init-default "page-medium" "page-type" "page-width" "page-height"
+		"page-width-margin" "page-height-margin"
+		"page-odd" "page-even" "page-right"
+		"par-width" "page-odd-shift" "page-even-shift"
+		"page-top" "page-bot" "page-height-margin")
+  (with t (buffer-tree)
+    (when (and (tm-func? t 'document 1) (switch-context? (tree-ref t 0)))
+      (tree-assign-node (tree-ref t 0) 'document)
+      (tree-set! t (tree-ref t 0))
+      (for-each dynamic-make-slide (tree-children t)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Global navigation in recursive fold/switch structure
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
