@@ -5,10 +5,9 @@
 ;; DESCRIPTION : routines for generating menus
 ;; COPYRIGHT   : (C) 2002  Joris van der Hoeven, David Allouche
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; See menu-define.scm for the grammar of menus
@@ -316,30 +315,35 @@
   "Expand links and conditional menus in list of menus @l."
   (map menu-expand l))
 
+(define (replace-procedures x)
+  (cond ((procedure? x) (procedure-source x))
+	((list? x) (map replace-procedures x))
+	(else x)))
+
 (tm-define (menu-expand p)
   (:type (-> object object))
   (:synopsis "Expand links and conditional menus in menu @p.")
-  (cond ((npair? p) p)
+  (cond ((npair? p) (replace-procedures p))
 	((string? (car p)) p)
 	((symbol? (car p))
 	 (with result (ahash-ref menu-expand-table (car p))
 	   (if result ((car result) p) p)))
-	((match? (car p) ':menu-wide-label) p)
+	((match? (car p) ':menu-wide-label) (replace-procedures p))
 	(else (menu-expand-list p))))
 
 (define-table menu-expand-table
   (--- ,(lambda (p) `(--- ,@(menu-expand-list (cdr p)))))
   (| ,(lambda (p) `(| ,@(menu-expand-list (cdr p)))))
-  (group ,(lambda (p) p))
-  (symbol ,(lambda (p) p))
+  (group ,replace-procedures)
+  (symbol ,replace-procedures)
   (link ,menu-expand-link p)
   (horizontal ,(lambda (p) `(horizontal ,@(menu-expand-list (cdr p)))))
   (vertical ,(lambda (p) `(vertical ,@(menu-expand-list (cdr p)))))
-  (-> ,(lambda (p) p))
-  (=> ,(lambda (p) p))
-  (tile ,(lambda (p) p))
+  (-> ,replace-procedures)
+  (=> ,replace-procedures)
+  (tile ,replace-procedures)
   (if ,menu-expand-if)
-  (when ,(lambda (p) p))
+  (when ,replace-procedures)
   (promise ,menu-expand-promise))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
