@@ -5,10 +5,9 @@
 ;; DESCRIPTION : keyboard shortcuts for windows and kde look and feel
 ;; COPYRIGHT   : (C) 2004  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -181,7 +180,7 @@
   ("windows v" (clipboard-paste "primary"))
   ("windows w" (safely-kill-buffer))
   ("windows x" (clipboard-cut "primary"))
-  ("windows z" (undo)))
+  ("windows z" (undo 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Windows-ish commands added by TeXmacs
@@ -204,7 +203,7 @@
   ("windows:meta ;" (make 'new-line))
   ("windows:meta return" (make 'next-line))
   ("windows:meta /" (make 'no-break))
-  ("windows:meta !" (make 'label))
+  ("windows:meta !" (make-label))
   ("windows:meta ?" (make 'reference))
   ("windows:meta C-?" (make 'pageref))
   ("windows:meta $" (spell-start))
