@@ -1,4 +1,4 @@
-<TeXmacs|1.0.4.7>
+<TeXmacs|1.0.6.14>
 
 <style|source>
 
@@ -16,11 +16,11 @@
       </src-copyright>
 
       <\src-license>
-        This <TeXmacs> style file falls under the <hlink|GNU general public
-        license|$TEXMACS_PATH/LICENSE> and comes WITHOUT ANY WARRANTY
-        WHATSOEVER. If you do not have a copy of the license, then write to
-        the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-        Boston, MA 02111-1307, USA.
+        This software falls under the <hlink|GNU general public
+        license, version 3 or later|$TEXMACS_PATH/LICENSE>.
+        It comes WITHOUT ANY WARRANTY WHATSOEVER.
+        You should have received a copy of the license which the software.
+        If not, see <hlink|http://www.gnu.org/licenses/gpl-3.0.html|http://www.gnu.org/licenses/gpl-3.0.html>.
       </src-license>
     </src-title>
   </active*>
@@ -442,19 +442,25 @@
 
   \;
 
+  <assign|toc-sv|<macro|l|w|left|right|<tabular|<tformat|<twith|table-width|1par>|<cwith|1|1|1|1|cell-hpart|1>|<cwith|1|1|1|1|cell-hyphen|t>|<cwith|1|1|2|2|cell-valign|b>|<cwith|1|1|2|2|cell-halign|r>|<cwith|1|1|2|2|cell-width|1.8em>|<cwith|1|-1|1|-1|cell-lsep|0cm>|<cwith|1|-1|1|-1|cell-rsep|0cm>|<cwith|1|1|1|1|cell-halign|L>|<table|<row|<\cell>
+    <\with|par-left|<arg|l>|par-first|<minus|<arg|w>>|par-mode|left>
+      <yes-indent><arg|left><toc-dots>
+    </with>
+  </cell>|<cell|<arg|right>>>>>>>>
+
   <assign|toc-strong-1|<macro|left|right|<quasiquote|<surround|<vspace*|2em>|<vspace|5pt>|<wide-bothlined|1ln|1ln|3sep|3sep|<with|font-series|bold|math-font-series|bold|<style-with|src-compact|none|<unquote|<arg|left>>>>>>>>>
 
-  <assign|toc-strong-2|<macro|left|right|<style-with|src-compact|none|<vspace*|1em><with|font-series|bold|math-font-series|bold|<arg|left>><toc-dots><no-break><arg|right>>>>
+  <assign|toc-strong-2|<macro|left|right|<style-with|src-compact|none|<vspace*|1em><style-with|src-compact|none|<toc-sv|<value|section-toc-indent>|<value|chapter-toc-width>|<with|font-series|bold|math-font-series|bold|<arg|left>>|<arg|right>>>>>>
 
-  <assign|toc-1|<macro|left|right|<style-with|src-compact|none|<with|par-left|<value|subsection-toc-indent>|par-first|<minus|<value|section-toc-width>>|<arg|left><toc-dots><no-break><arg|right>>>>>
+  <assign|toc-1|<macro|left|right|<toc-sv|<value|subsection-toc-indent>|<value|section-toc-width>|<arg|left>|<arg|right>>>>
 
-  <assign|toc-2|<macro|left|right|<style-with|src-compact|none|<with|par-left|<value|subsubsection-toc-indent>|par-first|<minus|<value|subsection-toc-width>>|<arg|left><toc-dots><no-break><arg|right>>>>>
+  <assign|toc-2|<macro|left|right|<toc-sv|<value|subsubsection-toc-indent>|<value|subsection-toc-width>|<arg|left>|<arg|right>>>>
 
-  <assign|toc-3|<macro|left|right|<style-with|src-compact|none|<with|par-left|<value|paragraph-toc-indent>|par-first|<minus|<value|subsubsection-toc-width>>|<arg|left><toc-dots><no-break><arg|right>>>>>
+  <assign|toc-3|<macro|left|right|<toc-sv|<value|paragraph-toc-indent>|<value|subsubsection-toc-width>|<arg|left>|<arg|right>>>>
 
-  <assign|toc-4|<macro|left|right|<style-with|src-compact|none|<with|par-left|<value|subparagraph-toc-indent>|par-first|<minus|<value|paragraph-toc-width>>|<arg|left><toc-dots><no-break><arg|right>>>>>
+  <assign|toc-4|<macro|left|right|<toc-sv|<value|subparagraph-toc-indent>|<value|paragraph-toc-width>|<arg|left>|<arg|right>>>>
 
-  <assign|toc-5|<macro|left|right|<style-with|src-compact|none|<with|par-left|<value|subsubparagraph-toc-indent>|par-first|<minus|<value|subparagraph-toc-width>>|<arg|left><toc-dots><no-break><arg|right>>>>>
+  <assign|toc-5|<macro|left|right|<style-with|src-compact|none|<toc-sv|<value|subsubparagraph-toc-indent>|<value|subparagraph-toc-width>|<arg|left>|<arg|right>>>>>
 
   <\active*>
     <\src-comment>
@@ -518,6 +524,19 @@
 
   \;
 
+  <assign|glossary-sv|<macro|left|right|<tabular|<tformat|<twith|table-width|1par>|<cwith|1|1|1|1|cell-hpart|1>|<cwith|1|1|1|1|cell-hyphen|t>|<cwith|1|1|2|2|cell-valign|b>|<cwith|1|1|2|2|cell-halign|r>|<cwith|1|1|2|2|cell-width|1.8em>|<cwith|1|-1|1|-1|cell-lsep|0cm>|<cwith|1|-1|1|-1|cell-rsep|0cm>|<cwith|1|1|1|1|cell-halign|L>|<table|<row|<\cell>
+    <surround||<glossary-dots>|<with|par-mode|left|<arg|left>>>
+  </cell>|<cell|<arg|right>>>>>>>>
+
+  <assign|glossary-1|<macro|left|right|<glossary-sv|<arg|left>|<arg|right>>>>
+
+  <assign|glossary-2|<\macro|entry|explain|right>
+    <glossary-sv|<margin-first-other|0fn|10fn|<style-with|src-compact|none|<resize|<arg|entry>
+    |||r]10fn|><arg|explain>>>|<arg|right>>
+  </macro>>
+
+  \;
+
   <assign|render-index|<\macro|name|body>
     <\with|par-par-sep|-0.5fn>
       <principal-section*|<arg|name>>
