@@ -4,13 +4,13 @@
 * DESCRIPTION: conversion of tex/latex strings into logical tex/latex trees
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Tex/convert_tex.hpp"
+#include "converter.hpp"
 
 /******************************************************************************
 * The latex_parser structure
@@ -39,7 +39,8 @@
 ******************************************************************************/
 
 struct latex_parser {
-  latex_parser () {}
+  bool unicode;
+  latex_parser (bool unicode2): unicode (unicode2) {}
   void latex_error (string s, int i, string message);
 
   tree parse           (string s, int& i, string stop= "", bool change= false);
@@ -68,6 +69,14 @@ latex_parser::latex_error (string s, int i, string message) {
 * Main parsing routine
 ******************************************************************************/
 
+static bool
+is_regular (tree t) {
+  if (!is_tuple (t)) return true;
+  if (N(t) == 0 || !is_atomic (t[0])) return false;
+  string s= t[0]->label;
+  return !starts (s, "\\begin-") && !starts (s, "\\end-");
+}
+
 tree
 latex_parser::parse (string s, int& i, string stop, bool change) {
   bool no_error= true;
@@ -82,7 +91,11 @@ latex_parser::parse (string s, int& i, string stop, bool change) {
   while ((i<n) && no_error &&
 	 (s[i] != '\0' || N (stop) != 0) &&
 	 (N(stop) != 1 || s[i] != stop[0]) &&
-	 (s[i] != '$' || stop != "$$" || i+1>=n || s[i+1] != '$')) {
+	 (s[i] != '$' || stop != "$$" || i+1>=n || s[i+1] != '$') &&
+	 (stop != "denom" ||
+	  (s[i] != '$' && s[i] != '}' &&
+	   (i+2>n || s(i,i+2) != "\\]") &&
+	   (i+4>n || s(i,i+4) != "\\end")))) {
     switch (s[i]) {
     case '~':
       if (command_type ["!mode"] == "math") t << tuple ("\\sim");
@@ -105,25 +118,19 @@ latex_parser::parse (string s, int& i, string stop, bool change) {
       }
       break;
     }
-    case '%':
-      if (test (s, i, "%%%%%%%%%% Start TeXmacs macros\n")) {
-	while ((i<n) && (!test (s, i, "%%%%%%%%%% End TeXmacs macros\n")))
-	  i++;
-	i += 30;
-      }
-      else {
-	while ((i<n) && (s[i]!='\n')) i++;
-	if (i<n) i++;
-	int ln=0;
-	while ((i<n) && is_space (s[i]))
-	  if (s[i++]=='\n') ln++;
-	if (ln > 0) {
-	  if ((N(t)>0) && ((t[N(t)-1]==" ") || (t[N(t)-1]=="\n")))
-	    t[N(t)-1]= "\n";
-	  else t << "\n";
-	}
+    case '%': {
+      while ((i<n) && (s[i]!='\n')) i++;
+      if (i<n) i++;
+      int ln=0;
+      while ((i<n) && is_space (s[i]))
+	if (s[i++]=='\n') ln++;
+      if (ln > 0) {
+	if ((N(t)>0) && ((t[N(t)-1]==" ") || (t[N(t)-1]=="\n")))
+	  t[N(t)-1]= "\n";
+	else t << "\n";
       }
       break;
+    }
     case '#':
       i++;
       if (i==n) return t;
@@ -134,15 +141,22 @@ latex_parser::parse (string s, int& i, string stop, bool change) {
       else t << s (i-1, i);
       break;
     case '\\':
-      if (((i+7)<n) && (s(i,i+5)=="\\over") && (!is_alpha (s (i+5, i+7)))) {
-	i+=5;
-	tree arg= parse_command (s, i, "\\over");
-	while ((N(t)>=1) &&
-	       ((t[N(t)-1] == " ") || (t[N(t)-1] == tree (TUPLE, "\\ "))))
-	  t= t (0, N(t)-1);
-	if (is_tuple (arg, "\\over", 1) && (N(t)>=1))
-	  t[N(t)-1]= tree (TUPLE, "\\frac", t[N(t)-1], arg[1]);
-      }
+      if (((i+7)<n) && !is_alpha (s (i+5, i+7)) &&
+	  (s (i, i+5) == "\\over" || s (i, i+5) == "\\atop"))
+	{
+	  string fr_cmd= s(i,i+5);
+	  if (fr_cmd == "\\over") fr_cmd= "\\frac";
+	  if (fr_cmd == "\\atop") fr_cmd= "\\ontop";
+	  int j;
+	  for (j=N(t); j>0 && is_regular (t[j-1]); j--);
+	  tree num= t (j, N(t));
+	  if (N(num) == 0) num= "";
+	  t= t (0, j);
+	  i+=5;
+	  while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+	  tree den= parse (s, i, "denom");
+	  t << tree (TUPLE, fr_cmd, num, den);
+	}
       else if (((i+5)<n) && (s(i,i+3)=="\\sp") && (!is_alpha(s[i+3]))) {
 	i+=3;
 	t << parse_command (s, i, "\\<sup>");
@@ -244,7 +258,11 @@ latex_parser::parse (string s, int& i, string stop, bool change) {
       break;
     }
     default:
-      if (is_iso_alpha (s[i])) {
+      if (unicode && ((unsigned char) s[i]) >= 128) {
+	unsigned int code= decode_from_utf8 (s, i);
+	t << tree (TUPLE, "\\#" * as_hexadecimal (code));
+      }
+      else if (!unicode && is_iso_alpha (s[i])) {
 	// If we encounter too much text in math mode, then return
 	int start= i;
 	while ((i<n) && is_iso_alpha (s[i])) i++;
@@ -311,9 +329,32 @@ latex_parser::parse_backslash (string s, int& i) {
     i+=16;
     return parse_verbatim (s, i, "\\end{verbatim}");
   }
-  if (((i+6)<n) && (s(i,i+5)=="\\url{")) {
+  if (((i+5)<n) && (s(i,i+4)=="\\url")) {
+    i+=4;
+    while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+    string ss;
+    if (i<n && s[i] == '{') {
+      i++;
+      int start= i;
+      while ((i<n) && s[i] != '}') i++;
+      ss= s (start, i++);
+    }
+    return tree (TUPLE, "\\url", ss);
+  }
+  if (((i+6)<n) && (s(i,i+5)=="\\href")) {
     i+=5;
-    return parse_verbatim (s, i, "}");
+    while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+    string ss;
+    if (i<n && s[i] == '{') {
+      i++;
+      int start= i;
+      while ((i<n) && s[i] != '}') i++;
+      ss= s (start, i++);
+    }
+    tree u= "";
+    while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+    if (i<n && s[i] == '{') { i++; u= parse (s, i, "}"); i++; }
+    return tree (TUPLE, "\\href", ss, u);
   }
 
   /************************ special commands *********************************/
@@ -438,6 +479,7 @@ latex_parser::parse_command (string s, int& i, string cmd) {
       u << s (j, i);
       if ((i<n) && (s[i]=='}')) i++;
       arity--;
+      if (arity == 0) option= false;
     }
     else if (option && (s[j]=='#') && (cmd == "\\def")) {
       while ((j+3 <= n) && is_numeric (s[j+1]) && (s[j+2] == '#')) j+=2;
@@ -456,6 +498,7 @@ latex_parser::parse_command (string s, int& i, string cmd) {
 	t << st;
 	u << st;
 	arity--;
+	if (arity == 0) option= false;
       }
       else break;
     }
@@ -561,11 +604,11 @@ latex_parser::parse_verbatim (string s, int& i, string end) {
 ******************************************************************************/
 
 static char Cork_unaccented[128]= {
-  'A', ' ', 'C', 'C', 'D', 'E', ' ', 'G',
+  'A', 'A', 'C', 'C', 'D', 'E', 'E', 'G',
   'L', 'L', ' ', 'N', 'N', ' ', 'O', 'R',
   'R', 'S', 'S', 'S', 'T', 'T', 'U', 'U',
   'Y', 'Z', 'Z', 'Z', ' ', 'I', 'd', ' ',
-  'a', ' ', 'c', 'c', 'd', 'e', ' ', 'g',
+  'a', 'a', 'c', 'c', 'd', 'e', 'e', 'g',
   'l', 'l', ' ', 'n', 'n', ' ', 'o', 'r',
   'r', 's', 's', 's', 't', 't', 'u', 'u',
   'y', 'z', 'z', 'z', ' ', ' ', ' ', ' ',
@@ -580,13 +623,13 @@ static char Cork_unaccented[128]= {
 };
 
 static char Cork_accent[128]= {
-  'u' , ' ' , '\'', 'v' , 'v' , 'v' , ' ' , 'u' ,
-  '\'', ' ' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
-  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , ' ' ,
+  'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
+  '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
+  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
   '\"', '\'', 'v' , '.' , ' ' , '.' , '=' , ' ' , // "
-  'u' , ' ' , '\'', 'v' , 'v' , 'v' , ' ' , 'u' ,
-  '\'', ' ' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
-  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , ' ' ,
+  'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
+  '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
+  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
   '\"', '\'', 'v' , '.' , ' ' , ' ' , ' ' , ' ' , // "
   '`' , '\'', '^' , '~' , '\"', ' ' , ' ' , 'c' , // "
   '`' , '\'', '^' , '\"', '`' , '\'', '^' , '\"', // "
@@ -657,6 +700,14 @@ latex_parser::parse (string s) {
   for (i=0; i<n; i++)
     if (s[i]=='\n') {
       while ((i<n) && is_space (s[i])) i++;
+      if (test (s, i, "%%%%%%%%%% Start TeXmacs macros\n")) {
+	a << s (start, i);
+	while ((i<n) && (!test (s, i, "%%%%%%%%%% End TeXmacs macros\n")))
+	  i++;
+	i += 30;
+	start= i;
+	continue;
+      }
       if (test (s, i, "\\begin{document}") ||
 	  test (s, i, "\\begin{abstract}") ||
 	  test (s, i, "\\chapter") ||
@@ -697,11 +748,120 @@ latex_parser::parse (string s) {
   return t;
 }
 
+static bool
+japanese_tex (string& s) {
+  if (search_forwards ("\\documentclass{jarticle}", s) != -1) {
+    s= replace (s, "\\documentclass{jarticle}", "\\documentclass{article}");
+    s= convert (s, "ISO-2022-JP", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\documentclass{jbook}", s) != -1) {
+    s= replace (s, "\\documentclass{jbook}", "\\documentclass{book}");
+    s= convert (s, "ISO-2022-JP", "UTF-8");
+    return true;
+  }
+  return false;
+}
+
+static bool
+korean_tex (string& s) {
+  if (search_forwards ("\\usepackage{hangul}", s) != -1 ||
+      search_forwards ("\\usepackage{hfont}", s) != -1 ||
+      search_forwards ("]{hangul}", s) != -1 ||
+      search_forwards ("]{hfont}", s) != -1)
+    {
+      s= replace (s, "\\usepackage{hangul}", "");
+      s= replace (s, "\\usepackage{hfont}", "");
+      s= convert (s, "EUC-KR", "UTF-8");
+      return true;
+    }
+  if (search_forwards ("\\usepackage{dhucs}", s) != -1 ||
+      search_forwards ("\\usepackage{memhangul-ucs}", s) != -1 ||
+      search_forwards ("]{dhucs}", s) != -1 ||
+      search_forwards ("]{memhangul-ucs}", s) != -1)
+    {
+      s= replace (s, "\\usepackage{dhucs}", "");
+      s= replace (s, "\\usepackage{memhangul-ucs}", "");
+      return true;
+    }
+  return false;
+}
+
+static bool
+chinese_tex (string& s) {
+  if (search_forwards ("\\kaishu", s) != -1)
+    s= replace (s, "\\kaishu", "");
+  if (search_forwards ("\\begin{CJK}{GBK}{kai}", s) != -1)
+    s= replace (s, "\\begin{CJK}{GBK}{kai}", "");
+  if (search_forwards ("\\begin{CJK*}{GBK}{kai}", s) != -1)
+    s= replace (s, "\\begin{CJK*}{GBK}{kai}", "");
+  if (search_forwards ("\\end{CJK}", s) != -1)
+    s= replace (s, "\\end{CJK}", "");
+  if (search_forwards ("\\end{CJK*}", s) != -1)
+    s= replace (s, "\\end{CJK*}", "");
+  if (search_forwards ("\\CJKindent", s) != -1)
+    s= replace (s, "\\CJKindent", "");
+  if (search_forwards ("\\CJKcaption{GBk}", s) != -1)
+    s= replace (s, "\\CJKcaption{GBK}", "");
+  if (search_forwards ("\\usepackage{CJK}", s) != -1) {
+    s= replace (s, "\\usepackage{CJK}", "");
+    s= convert (s, "cp936", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\documentclass{cctart}", s) != -1) {
+    s= replace (s, "\\documentclass{cctart}", "\\documentclass{article}");
+    s= convert (s, "cp936", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\documentclass[CJK]{cctart}", s) != -1) {
+    s= replace (s, "\\documentclass[CJK]{cctart}", "\\documentclass{article}");
+    s= convert (s, "cp936", "UTF-8");
+    return true;
+  }
+  return false;
+}
+
+static bool
+taiwanese_tex (string& s) {
+  if (search_forwards ("\\usepackage{CJKvert,type1cm}", s) != -1)
+    s= replace (s, "\\usepackage{CJKvert,type1cm}", "");
+  if (search_forwards ("\\begin{CJK}{Bg5}{aming}", s) != -1)
+    s= replace (s, "\\begin{CJK}{Bg5}{aming}", "");
+  if (search_forwards ("\\begin{CJK}{Bg5}{kai}", s) != -1)
+    s= replace (s, "\\begin{CJK}{Bg5}{kai}", "");
+  if (search_forwards ("\\end{CJK}", s) != -1)
+    s= replace (s, "\\end{CJK}", "");
+  if (search_forwards ("\\CJKcaption{Bg5}", s) != -1)
+    s= replace (s, "\\CJKcaption{Bg5}", "");
+  if (search_forwards ("\\CJKindent", s) != -1)
+    s= replace (s, "\\CJKindent", "");
+  if (search_forwards ("\\usepackage{CJK}", s) != -1) {
+    s= replace (s, "\\usepackage{CJK}", "");
+    s= convert (s, "cp950", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\usepackage{CJK*}", s) != -1) {
+    s= replace (s, "\\usepackage{CJK*}", "");
+    s= convert (s, "cp950", "UTF-8");
+    return true;
+  }
+  return false;
+}
+
 tree
 parse_latex (string s) {
   s= dos_to_better (s);
-  latex_parser ltx;
-  return accented_to_Cork (ltx.parse (s));
+  string lan= "";
+  if (japanese_tex (s)) lan= "japanese";
+  else if (korean_tex (s)) lan= "korean";
+  else if (taiwanese_tex (s)) lan= "taiwanese";
+  else if (chinese_tex (s)) lan= "chinese";
+  bool unicode= (lan == "chinese" || lan == "japanese" ||
+		 lan == "korean" || lan == "taiwanese");
+  latex_parser ltx (unicode);
+  tree r= accented_to_Cork (ltx.parse (s));
+  if (lan == "") return r;
+  return compound ("!language", r, lan);
 }
 
 tree
