@@ -4,10 +4,9 @@
 * DESCRIPTION: data relation descriptions
 * COPYRIGHT  : (C) 2003  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "vars.hpp"
@@ -26,9 +25,9 @@ drd_info_rep::drd_info_rep (string name2):
 drd_info_rep::drd_info_rep (string name2, drd_info base):
   name (name2), info (tag_info (), base->info) {}
 drd_info::drd_info (string name):
-  rep (new drd_info_rep (name)) {}
+  rep (tm_new<drd_info_rep> (name)) {}
 drd_info::drd_info (string name, drd_info base):
-  rep (new drd_info_rep (name, base)) {}
+  rep (tm_new<drd_info_rep> (name, base)) {}
 
 tree
 drd_info_rep::get_locals () {
@@ -465,7 +464,7 @@ drd_info_rep::heuristic_init_xmacro (string var, tree xmacro) {
 
 void
 drd_info_rep::heuristic_init (hashmap<string,tree> env) {
-  // int tt= texmacs_time ();
+  // time_t tt= texmacs_time ();
   bool flag= true;
   int round= 0;
   while (flag) {
