@@ -4,10 +4,9 @@
 * DESCRIPTION: DRD information about tags
 * COPYRIGHT  : (C) 2003  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "tag_info.hpp"
@@ -87,44 +86,52 @@ operator << (ostream& out, parent_info pi) {
 ******************************************************************************/
 
 child_info::child_info (bool frozen) {
-  accessible        = ACCESSIBLE_NEVER;
-  block             = 0;
-  mode              = MODE_PARENT;
-  freeze_accessible = frozen;
-  freeze_block      = frozen;
-  freeze_mode       = frozen;
+  accessible         = ACCESSIBLE_NEVER;
+  writability        = WRITABILITY_NORMAL;
+  block              = 0;
+  mode               = MODE_PARENT;
+  freeze_accessible  = frozen;
+  freeze_writability = frozen;
+  freeze_block       = frozen;
+  freeze_mode        = frozen;
 }
 
 child_info::child_info (string s) {
   int i= as_int (s);
-  get_bits (accessible       , 2);
-  get_bits (block            , 2);
-  get_bits (mode             , 3);
-  get_bits (freeze_accessible, 1);
-  get_bits (freeze_block     , 1);
-  get_bits (freeze_mode      , 1);
+  get_bits (accessible        , 2);
+  get_bits (writability       , 2);
+  get_bits (block             , 2);
+  get_bits (mode              , 3);
+  get_bits (freeze_accessible , 1);
+  get_bits (freeze_writability, 1);
+  get_bits (freeze_block      , 1);
+  get_bits (freeze_mode       , 1);
 }
 
 child_info::operator string () {
   int i=0, offset=0;
-  set_bits (accessible       , 2);
-  set_bits (block            , 2);
-  set_bits (mode             , 3);
-  set_bits (freeze_accessible, 1);
-  set_bits (freeze_block     , 1);
-  set_bits (freeze_mode      , 1);
+  set_bits (accessible        , 2);
+  set_bits (writability       , 2);
+  set_bits (block             , 2);
+  set_bits (mode              , 3);
+  set_bits (freeze_accessible , 1);
+  set_bits (freeze_writability, 1);
+  set_bits (freeze_block      , 1);
+  set_bits (freeze_mode       , 1);
   return as_string (i);
 }
 
 bool
 child_info::operator == (const child_info& ci) {
   return
-    (accessible        == ci.accessible       ) &&
-    (block             == ci.block            ) &&
-    (mode              == ci.mode             ) &&
-    (freeze_accessible == ci.freeze_accessible) &&
-    (freeze_block      == ci.freeze_block     ) &&
-    (freeze_mode       == ci.freeze_mode      );
+    (accessible         == ci.accessible        ) &&
+    (writability        == ci.writability       ) &&
+    (block              == ci.block             ) &&
+    (mode               == ci.mode              ) &&
+    (freeze_accessible  == ci.freeze_accessible ) &&
+    (freeze_writability == ci.freeze_writability) &&
+    (freeze_block       == ci.freeze_block      ) &&
+    (freeze_mode        == ci.freeze_mode       );
 }
 
 bool
@@ -156,24 +163,24 @@ tag_info_rep::tag_info_rep (int a, int x, int am, int cm, bool frozen):
 }
 
 tag_info::tag_info (parent_info pi, array<child_info> ci, tree extra) {
-  rep= new tag_info_rep (pi, ci, extra);
+  rep= tm_new<tag_info_rep> (pi, ci, extra);
 }
 
 tag_info::tag_info (int a, int x, int am, int cm, bool frozen) {
-  rep= new tag_info_rep (a, x, am, cm, frozen);
+  rep= tm_new<tag_info_rep> (a, x, am, cm, frozen);
 }
 
 tag_info::tag_info (tree t) {
   if ((!is_func (t, TUPLE)) || (N(t)<2) || (L(t[1]) != TUPLE)) {
     cerr << "\nt= " << t << "\n";
-    fatal_error ("Bad tag_info", "tag_info::tag_info (tree)");
+    FAILED ("bad tag_info");
   }
   parent_info pi (as_string (t[0]));
   int i, n= N(t[1]);
   array<child_info> ci (n);
   for (i=0; i<n; i++)
     ci[i]= as_string (t[1][i]);
-  rep= new tag_info_rep (pi, ci, N(t)==3? t[2]: tree (""));
+  rep= tm_new<tag_info_rep> (pi, ci, N(t)==3? t[2]: tree (""));
 }
 
 tag_info::operator tree () {
@@ -203,6 +210,18 @@ tag_info_rep::hidden (int i) {
   return tag_info (pi, ci, extra);
 }
 
+tag_info
+tag_info_rep::disable_writable (int i) {
+  ci[i].writability= WRITABILITY_DISABLE;
+  return tag_info (pi, ci, extra);
+}
+
+tag_info
+tag_info_rep::enable_writable (int i) {
+  ci[i].writability= WRITABILITY_ENABLE;
+  return tag_info (pi, ci, extra);
+}
+
 tag_info
 tag_info_rep::name (string s) {
   set_attribute ("name", s);
