@@ -4,10 +4,9 @@
 * DESCRIPTION: Properties of characters and strings
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "analyze.hpp"
@@ -694,6 +693,7 @@ tm_correct (string s) {
 
 void
 tm_char_forwards (string s, int& pos) {
+  ASSERT (pos >= 0 && pos <= N(s), "out of range");
   int n= N(s);
   if (pos == n);
   else if (s[pos] != '<') pos++;
@@ -705,6 +705,7 @@ tm_char_forwards (string s, int& pos) {
 
 void
 tm_char_backwards (string s, int& pos) {
+  ASSERT (pos >= 0 && pos <= N(s), "out of range");
   if (pos == 0);
   else if (s[pos-1] != '>') pos--;
   else {
@@ -776,6 +777,8 @@ escape_sh (string s) {
   string r;
   for (i=0; i<n; i++)
     switch (s[i]) {
+    case '?':
+    case '&':
     case '$':
     case '`':
     case '\"':
@@ -1026,7 +1029,7 @@ parse (string s, int& pos, SI& ret) {
 void
 parse (string s, int& pos, SI*& a, int len) {
   int i;
-  a= new int[len];
+  a= tm_new_array<int> (len);
   for (i=0; i<len; i++) parse (s, pos, a[i]);
 }
 
