@@ -4,10 +4,9 @@
 * DESCRIPTION: Applies dictionaries to strings in an efficient manner.
 * COPYRIGHT  : (C) 2002  Felix Breuer
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "converter.hpp"
@@ -53,7 +52,7 @@ load_converter (string from, string to) {
   string name= from * "-" * to;
   if (converter::instances -> contains (name))
     return converter (name);
-  converter conv = new converter_rep (from, to);
+  converter conv = tm_new<converter_rep> (from, to);
   return conv;
 }
 
@@ -214,7 +213,7 @@ template<class T> class auto_array {
   T* value;
 public:
   auto_array (T* x) : value (x) {}
-  ~auto_array () { delete[] value ; }
+  ~auto_array () { tm_delete_array (value ); }
   operator T* () const { return value; }
 };
 
@@ -271,7 +270,7 @@ string apply (iconv_converter &conv, string input) {
   size_t out_counter= 0;
   while (in_left > 0) {
     size_t out_left= max(int(in_left * expansion), 1024);
-    auto_array<char> out_cp= new char[out_left];
+    auto_array<char> out_cp= tm_new_array<char> (out_left);
     char* out_cursor= out_cp;
     size_t r = iconv_adaptor(iconv, conv.cd,
 			     &in_cursor, &in_left, &out_cursor, &out_left);
@@ -302,7 +301,7 @@ bool check_using_iconv (string input, string encoding) {
 #else
   (void) input;
   (void) encoding;
-  fatal_error ("iconv not enabled", "check_using_iconv", "converter.cpp");
+  FAILED ("iconv not enabled");
   return false;
 #endif
 }
@@ -316,7 +315,7 @@ convert_using_iconv (string input, string from, string to) {
   (void) input;
   (void) from;
   (void) to;
-  fatal_error ("iconv not enabled", "convert_using_iconv", "converter.cpp");
+  FAILED ("iconv not enabled");
   return "";
 #endif
 }
