@@ -4,50 +4,61 @@
 * DESCRIPTION: routines for the editor
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_main.hpp"
-#include "tm_widget.hpp"
 #include "tm_buffer.hpp"
 #include "file.hpp"
 #include "sys_utils.hpp"
-#include "PsDevice/printer.hpp"
+#include "printer.hpp"
 #include "convert.hpp"
 #include "connect.hpp"
 #include "typesetter.hpp"
 #include "drd_std.hpp"
-#include <dlfcn.h>
+#include "message.hpp"
 #include <setjmp.h>
+#ifdef EXPERIMENTAL
+#include "../../Style/Memorizer/clean_copy.hpp"
+#endif
 
 /******************************************************************************
 * Constructors and destructor
 ******************************************************************************/
 
 editor_rep::editor_rep ():
-  attribute_widget_rep (dis),
+  simple_widget_rep (),
   drd (buf->abbr, std_drd), et (the_et), rp (buf->rp) {}
 
-editor_rep::editor_rep (server_rep* sv2, display dis, tm_buffer buf2):
-  attribute_widget_rep (dis),
+editor_rep::editor_rep (server_rep* sv2, tm_buffer buf2):
+  simple_widget_rep (),
   sv (sv2), buf (buf2), drd (buf->abbr, std_drd),
   et (the_et), rp (buf2->rp) {}
 
-edit_main_rep::edit_main_rep (server_rep* sv, display dis, tm_buffer buf):
-  editor_rep (sv, dis, buf), props (UNKNOWN)
+edit_main_rep::edit_main_rep (server_rep* sv, tm_buffer buf):
+  editor_rep (sv, buf), props (UNKNOWN), ed_obs (edit_observer (this))
 {
+#ifdef EXPERIMENTAL
+  cct= copy (subtree (et, rp));
+  copy_ip (subtree (et, rp), cct);
+#endif
+  attach_observer (subtree (et, rp), ed_obs);
   notify_change (THE_TREE);
   tp= correct_cursor (et, rp * 0);
 }
 
-edit_main_rep::~edit_main_rep () {}
+edit_main_rep::~edit_main_rep () {
+  detach_observer (subtree (et, rp), ed_obs);
+#ifdef EXPERIMENTAL
+  mem= memorizer ();
+#endif
+}
 
 editor
 new_editor (server_rep* sv, tm_buffer buf) {
-  return new edit_main_rep (sv, current_display (), buf);
+  return tm_new<edit_main_rep> (sv, buf);
 }
 
 /******************************************************************************
@@ -127,9 +138,7 @@ edit_main_rep::focus_on_this_editor () {
 
 void
 edit_main_rep::notify_page_change () {
-  if (attached ()) this << emit_invalidate_all ();
-  if (get_init_string (PAGE_MEDIUM) == "automatic")
-    notify_change (THE_AUTOMATIC_SIZE);
+  if (is_attached (this)) send_invalidate_all (this);
 }
 
 /******************************************************************************
@@ -147,13 +156,8 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
   if (pdf) name= url_temp (".ps");
 
   string medium = env->get_string (PAGE_MEDIUM);
-  if (conform && (medium == "papyrus")) conform= false;
-  if ((!conform) && (medium == "automatic")) {
-    set_message (
-      "Error: you should switch to ``paper'' or ``papyrus'' page type",
-      "print");
-    return;
-  }
+  if (conform && (medium != "paper")) conform= false;
+  // FIXME: better command for conform printing
 
   // Set environment variables for printing
 
@@ -161,13 +165,16 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
   env->write (DPI, printing_dpi);
   env->write (PAGE_SHOW_HF, "true");
   env->write (PAGE_SCREEN_MARGIN, "false");
-  if (!conform) env->write (PAGE_MEDIUM, "paper");
+  if (!conform) {
+    env->write (PAGE_MEDIUM, "paper");
+    env->write (PAGE_PRINTED, "true");
+  }
 
   // Typeset pages for printing
 
   box the_box= typeset_as_document (env, subtree (et, rp), reverse (rp));
 
-  // Determine parameters for printer device
+  // Determine parameters for printer
 
   string page_type = env->get_string (PAGE_TYPE);
   double w         = env->page_width;
@@ -191,21 +198,21 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
   // Print pages
 
   int i;
-  ps_device dev=
-    printer (dis, name, dpi, pages, page_type, landsc, w/cm, h/cm);
+  renderer ren=
+    printer (name, dpi, pages, page_type, landsc, w/cm, h/cm);
   for (i=start; i<end; i++) {
-    string col_name= env->get_string (BG_COLOR);
-    dev->set_background (dis->get_color (col_name));
-    if (col_name != "white")
-      dev->clear (0, (SI) -h, (SI) w, 0);
+    tree bg= env->read (BG_COLOR);
+    ren->set_background_pattern (bg);
+    if (bg != "white")
+      ren->clear_pattern (0, (SI) -h, (SI) w, 0);
 
     rectangles rs;
     the_box[0]->sx(i)= 0;
     the_box[0]->sy(i)= 0;
-    the_box[0][i]->redraw (dev, path (0), rs);
-    if (i<end-1) dev->next_page ();
+    the_box[0][i]->redraw (ren, path (0), rs);
+    if (i<end-1) ren->next_page ();
   }
-  delete dev;
+  tm_delete (ren);
 
   if (pdf) {
     ps2pdf (name, orig);
@@ -236,13 +243,14 @@ edit_main_rep::print_snippet (url name, tree t) {
   bool ps= suffix (name) == "ps" || suffix (name) == "eps";
   typeset_prepare ();
   int dpi= as_int (printing_dpi);
-  if (!ps) t= tree (WITH, MAGNIFICATION, "2", PAGE_WIDTH, "40cm", t);
+  //if (!ps) t= tree (WITH, MAGNIFICATION, "2", PAGE_WIDTH, "40cm", t);
+  if (!ps) t= tree (WITH, MAGNIFICATION, "1.6", PAGE_WIDTH, "40cm", t);
   box b= typeset_as_box (env, t, path ());
   if (b->x4 - b->x3 >= 5*PIXEL && b->y4 - b->y3 >= 5*PIXEL) {
-    if (ps) make_eps (name, dis, b, dpi);
+    if (ps) make_eps (name, b, dpi);
     else {
       url temp= url_temp ("eps");
-      make_eps (temp, dis, b, dpi);
+      make_eps (temp, b, dpi);
       system ("convert", temp, name);
       ::remove (temp);
     }
@@ -338,7 +346,16 @@ void
 edit_main_rep::edit_special () {
 }
 
+#ifdef UNCOMMENTED
+void test_commute ();
+void test_invert ();
+#endif
+
 void
 edit_main_rep::edit_test () {
   cout << "Test !\n";
+#ifdef UNCOMMENTED
+  test_commute();
+  test_invert();
+#endif
 }
