@@ -4,10 +4,9 @@
 * DESCRIPTION: cursor handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_cursor.hpp"
@@ -15,6 +14,7 @@
 #include "tm_buffer.hpp"
 #include "tree_traverse.hpp"
 #include "drd_mode.hpp"
+#include "analyze.hpp"
 
 /******************************************************************************
 * Constructor and destructor
@@ -38,85 +38,91 @@ path
 edit_cursor_rep::make_cursor_accessible (path p, bool forwards) {
   path start_p= p;
   bool inverse= false;
+  int old_mode= get_access_mode ();
   if (get_init_string (MODE) == "src")
     set_access_mode (DRD_ACCESS_SOURCE);
-  else set_access_mode (DRD_ACCESS_NORMAL);
   while (!is_accessible_cursor (et, p) && !in_source ()) {
     path pp;
+    ASSERT (rp <= p, "path outside document");
+    p= rp * closest_inside (subtree (et, rp), p / rp);
     if (forwards ^ inverse)
-      pp= rp * next_valid (subtree (et, rp), p - rp);
+      pp= rp * next_valid (subtree (et, rp), p / rp);
     else
-      pp= rp * previous_valid (subtree (et, rp), p - rp);
+      pp= rp * previous_valid (subtree (et, rp), p / rp);
     if (pp == p) {
       if (inverse) break;
       else { p= start_p; inverse= true; }
     }
     else p= pp;
   }
+  set_access_mode (old_mode);
   return p;
 }
 
 path
-edit_cursor_rep::tree_path (SI x, SI y, SI delta) {
-  path p= correct_cursor (et, eb->find_tree_path (x, y, delta));
+edit_cursor_rep::tree_path (path sp, SI x, SI y, SI delta) {
+  path p= correct_cursor (et, find_scrolled_tree_path (eb, sp, x, y, delta));
   return make_cursor_accessible (p, searching_forwards);
 }
 
 bool
 edit_cursor_rep::cursor_move_sub (SI& x0, SI& y0, SI& d0, SI dx, SI dy) {
+  path sp= find_innermost_scroll (eb, tp);
   searching_forwards= dx == 1 || dy == -1;
+
   int i,d;
-  path ref_p= tree_path (x0, y0, d0);
+  path ref_p= tree_path (sp, x0, y0, d0);
   if (ref_p != tp) {
     tp= ref_p;
     return true;
   }
   
   // cout << "ref_p = " << ref_p << "\n";
-  if (ref_p == tree_path (x0, y0, d0+ dx*DELTA)) {
+  if (ref_p == tree_path (sp, x0, y0, d0+ dx*DELTA)) {
     for (i=1; i<DELTA; i=i<<1)
-      if (ref_p != tree_path (x0+ dx*i, y0+ dy*i, d0+ dx*DELTA)) break;
+      if (ref_p != tree_path (sp, x0+ dx*i, y0+ dy*i, d0+ dx*DELTA))
+	break;
     if (i>=DELTA) return false;
     for (d=i>>2; d>=1; d=d>>1)
-      if (ref_p != tree_path (x0+ dx*(i-d), y0+ dy*(i-d), d0+ dx*DELTA)) i-=d;
+      if (ref_p != tree_path (sp, x0+ dx*(i-d), y0+ dy*(i-d), d0+ dx*DELTA))
+	i-=d;
 
     x0 += dx*i;
     y0 += dy*i;
   }
   
-  // cout << "path  = " << tree_path (x0, y0, d0) << "\n";
+  // cout << "path  = " << tree_path (sp, x0, y0, d0) << "\n";
   if (dx!=0) {
-    if (ref_p == tree_path (x0, y0, d0)) {
+    if (ref_p == tree_path (sp, x0, y0, d0)) {
       for (i=1; i<DELTA; i=i<<1)
-	if (ref_p != tree_path (x0, y0, d0+ dx*i)) break;
+	if (ref_p != tree_path (sp, x0, y0, d0+ dx*i)) break;
       if (i>=DELTA)
-	fatal_error ("inconsistent cursor handling",
-		     "edit_cursor_rep::cursor_move_sub");
+	FAILED ("inconsistent cursor handling");
       for (d=i>>2; d>=1; d=d>>1)
-	if (ref_p != tree_path (x0, y0, d0+ dx*(i-d))) i-=d;
+	if (ref_p != tree_path (sp, x0, y0, d0+ dx*(i-d))) i-=d;
       d0 += dx*i;
     }
     else {
       for (i=1; i<DELTA; i=i<<1)
-	if (ref_p == tree_path (x0, y0, d0- dx*i)) break;
+	if (ref_p == tree_path (sp, x0, y0, d0- dx*i)) break;
       if (i<DELTA) {
 	for (d=i>>2; d>=1; d=d>>1)
-	  if (ref_p == tree_path (x0, y0, d0- dx*(i-d))) i-=d;
+	  if (ref_p == tree_path (sp, x0, y0, d0- dx*(i-d))) i-=d;
 	i--;
 	d0 -= dx*i;
       }
       else {  // exceptional case
-	ref_p= tree_path (x0, y0, d0- dx*DELTA);
+	ref_p= tree_path (sp, x0, y0, d0- dx*DELTA);
 	for (i=1; i<DELTA; i=i<<1)
-	  if (ref_p == tree_path (x0, y0, d0- dx*i)) break;
+	  if (ref_p == tree_path (sp, x0, y0, d0- dx*i)) break;
 	for (d=i>>2; d>=1; d=d>>1)
-	  if (ref_p == tree_path (x0, y0, d0- dx*(i-d))) i-=d;
+	  if (ref_p == tree_path (sp, x0, y0, d0- dx*(i-d))) i-=d;
 	d0 -= dx*i;
       }
     }
   }
 
-  tp= tree_path (x0, y0, d0);
+  tp= tree_path (sp, x0, y0, d0);
   return true;
 }
 
@@ -149,9 +155,9 @@ edit_cursor_rep::notify_cursor_moved (int status) {
 }
 
 void
-edit_cursor_rep::go_to (SI x, SI y) {
+edit_cursor_rep::go_to (SI x, SI y, bool absolute) {
   if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
-  tp= tree_path (x, y, 0);
+  tp= tree_path (absolute? path (): find_innermost_scroll (eb, tp), x, y, 0);
   notify_cursor_moved (CENTER);
   mv->ox   = x;
   mv->oy   = y;
@@ -197,16 +203,30 @@ edit_cursor_rep::go_down () {
 void
 edit_cursor_rep::go_page_up () {
   if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
-  if (mv->oy >= eb->y2) return;
-  go_to (mv->ox, mv->oy+ get_window_height ());
+  path sp= find_innermost_scroll (eb, tp);
+  if (is_nil (sp)) go_to (mv->ox, min (mv->oy + get_window_height (), eb->y2));
+  else {
+    SI x, y, sx, sy;
+    rectangle outer, inner;
+    box b= eb[path_up (sp)];
+    find_canvas_info (eb, sp, x, y, sx, sy, outer, inner);
+    go_to (mv->ox, min (mv->oy + b->h (), y + sy + inner->y2), false);
+  }
   select_from_cursor_if_active ();
 }
 
 void
 edit_cursor_rep::go_page_down () {
   if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
-  if (mv->oy < eb->y1) return;
-  go_to (mv->ox, mv->oy- get_window_height ());
+  path sp= find_innermost_scroll (eb, tp);
+  if (is_nil (sp)) go_to (mv->ox, max (mv->oy - get_window_height (), eb->y1));
+  else {
+    SI x, y, sx, sy;
+    rectangle outer, inner;
+    box b= eb[path_up (sp)];
+    find_canvas_info (eb, sp, x, y, sx, sy, outer, inner);
+    go_to (mv->ox, max (mv->oy - b->h (), y + sy + inner->y1), false);
+  }
   select_from_cursor_if_active ();
 }
 
@@ -280,27 +300,28 @@ edit_cursor_rep::go_end_line () {
 
 void
 edit_cursor_rep::adjust_cursor () {
+  path sp= find_innermost_scroll (eb, tp);
   cursor mv= copy (cu);
   SI dx= PIXEL << 8, ddelta= 0;
-  path p= tree_path (mv->ox, mv->oy, mv->delta);
+  path p= tree_path (sp, mv->ox, mv->oy, mv->delta);
   if (p != tp) {
     // cout << "Cursors don't match\n";
     while (dx != 0 || ddelta != 0) {
       // cout << "  " << tp << ", " << p << "\n";
-      p= tree_path (mv->ox, mv->oy, mv->delta);
+      p= tree_path (sp, mv->ox, mv->oy, mv->delta);
       int eps= (path_inf (p, tp)? 1: -1);
       if (p == tp) eps= (mv->ox < cu->ox? 1: -1);
       if (p == tp && mv->ox == cu->ox) eps= (mv->delta < cu->delta? 1: -1);
       if (dx > 0) {
 	if (p != tp ||
-	    tree_path (mv->ox + eps * dx, mv->oy, mv->delta) == tp)
+	    tree_path (sp, mv->ox + eps * dx, mv->oy, mv->delta) == tp)
 	  mv->ox += eps * dx;
 	dx >>= 1;
 	if (dx == 0) ddelta= DELTA;
       }
       else if (ddelta > 0) {
 	if (p != tp ||
-	    tree_path (mv->ox, mv->oy, mv->delta + eps * ddelta) == tp)
+	    tree_path (sp, mv->ox, mv->oy, mv->delta + eps * ddelta) == tp)
 	  mv->delta += eps * ddelta;
 	ddelta >>= 1;
       }
@@ -327,14 +348,16 @@ edit_cursor_rep::go_to_here () {
 
 void
 edit_cursor_rep::go_to (path p) {
-  tp= p;
-  mv_status= DIRECT;
-  if (!has_changed (THE_TREE+THE_ENVIRONMENT)) {
-    cu= eb->find_check_cursor (tp);
-    if (cu->valid) adjust_cursor ();
-    mv= copy (cu);
+  if (rp <= p) {
+    tp= p;
+    mv_status= DIRECT;
+    if (!has_changed (THE_TREE+THE_ENVIRONMENT)) {
+      cu= eb->find_check_cursor (tp);
+      if (cu->valid) adjust_cursor ();
+      mv= copy (cu);
+    }
+    notify_change (THE_CURSOR);
   }
-  notify_change (THE_CURSOR);
 }
 
 void
@@ -388,25 +411,25 @@ edit_cursor_rep::go_end_paragraph () {
 void
 edit_cursor_rep::go_start_of (tree_label what) {
   path p= search_upwards (what);
-  if (!nil (p)) go_to (start (et, p));
+  if (!is_nil (p)) go_to (start (et, p));
 }
 
 void
 edit_cursor_rep::go_end_of (tree_label what) {
   path p= search_upwards (what);
-  if (!nil (p)) go_to (end (et, p));
+  if (!is_nil (p)) go_to (end (et, p));
 }
 
 void
 edit_cursor_rep::go_start_with (string var, string val) {
   path p= search_upwards_with (var, val);
-  if (!nil (p)) go_to (start (et, p));
+  if (!is_nil (p)) go_to (start (et, p));
 }
 
 void
 edit_cursor_rep::go_end_with (string var, string val) {
   path p= search_upwards_with (var, val);
-  if (!nil (p)) go_to (end (et, p));
+  if (!is_nil (p)) go_to (end (et, p));
 }
 
 /******************************************************************************
@@ -430,14 +453,16 @@ edit_cursor_rep::get_labels () {
 }
 
 static path
-search_tree_in (tree t, tree what) {
+search_label (tree t, string which) {
   if (is_atomic (t)) return path ();
-  else if (t == what) return path (1);
+  else if (t == tree (LABEL, which)) return path (1);
+  else if (is_compound (t, "tag", 2) && t[0] == which)
+    return path (1, start (t[1]));
   else {
     int i, n=N(t);
     for (i=0; i<n; i++) {
-      path q= search_tree_in (t[i], what);
-      if (!nil (q)) return path (i, q);
+      path q= search_label (t[i], which);
+      if (!is_nil (q)) return path (i, q);
     }
     return path ();
   }
@@ -453,11 +478,39 @@ edit_cursor_rep::show_cursor_if_hidden () {
 
 void
 edit_cursor_rep::go_to_label (string s) {
-  path p= search_tree_in (et, tree (LABEL, s));
-  if (!nil (p)) go_to (p);
-  else if (!nil (eb)) {
+  path p= search_label (et, s);
+  if (!is_nil (p)) {
+    go_to (p);
+    show_cursor_if_hidden ();
+    return;
+  }
+  if (!is_nil (eb)) {
     p= eb->find_tag (s);
-    if (!nil (p)) go_to (p);
+    if (!is_nil (p)) {
+      go_to (p);
+      show_cursor_if_hidden ();
+      return;
+    }
+  }
+  tree val= (buf->prj==NULL? buf->ref[s]: buf->prj->ref[s]);
+  if (is_func (val, TUPLE, 3) && is_atomic (val[2])) {
+    string extra= val[2]->label;
+    if (starts (extra, "#")) {
+      string part= extra (1, N (extra));
+      int i= search_forwards (".", part);
+      if (i >= 0) part= part (0, i);
+      string show= "(show-hidden-part " * scm_quote (part) * ")";
+      string jump= "(go-to-label " * scm_quote (s) * ")";
+      exec_delayed (scheme_cmd ("(if " * show * " (delayed " * jump * "))"));
+    }
+    else {
+      url u= relative (buf->name, url (extra));
+      if (u != buf->name) {
+	string new_buf = scm_quote (as_string (u));
+	string load_buf= "(load-buffer (url-system " * new_buf * "))";
+	string jump_to = "(go-to-label " * scm_quote (s) * ")";
+	exec_delayed (scheme_cmd ("(begin " * load_buf * " " * jump_to * ")"));
+      }
+    }
   }
-  show_cursor_if_hidden ();
 }
