@@ -4,10 +4,9 @@
 * DESCRIPTION: interface between the editor and the window manager
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Interface/edit_interface.hpp"
@@ -20,7 +19,6 @@
 #include "drd_mode.hpp"
 #include "message.hpp"
 #ifdef EXPERIMENTAL
-#include "../../Style/Memorizer/clean_copy.hpp"
 #include "../../Style/Evaluate/evaluate_main.hpp"
 #endif
 
@@ -34,7 +32,7 @@ MODE_LANGUAGE (string mode) {
   else if (mode == "prog") return PROG_LANGUAGE;
   else if (mode == "src") return LANGUAGE;
   cerr << "Mode = " << mode << "\n";
-  fatal_error ("invalid mode", "MODE_LANGUAGE", "edit_interface.cpp");
+  FAILED ("invalid mode");
   return LANGUAGE;
 }
 
@@ -47,7 +45,7 @@ edit_interface_rep::edit_interface_rep ():
   last_change (texmacs_time()), last_update (last_change-1),
   do_animate (false), next_animate (last_change-1),
   full_screen (false), got_focus (false),
-  sh_s (""), sh_len (0),
+  sh_s (""), sh_mark (0),
   popup_win (),
   sfactor (sv->get_default_shrinking_factor ()),
   pixel (sfactor*PIXEL), copy_always (),
@@ -197,7 +195,7 @@ edit_interface_rep::cursor_visible () {
 	  if (new_xt != old_xt && is_accessible (obtain_ip (old_xt))) {
 	    object fun= symbol_object ("tree-set");
 	    object cmd= list_object (fun, old_xt, new_xt);
-	    eval_delayed (cmd);
+	    exec_delayed (scheme_cmd (cmd));
 	    temp_invalid_cursor= true;
 	  }
 	}
@@ -218,7 +216,7 @@ edit_interface_rep::cursor_visible () {
 	  if (new_yt != old_yt && is_accessible (obtain_ip (old_yt))) {
 	    object fun= symbol_object ("tree-set");
 	    object cmd= list_object (fun, old_yt, new_yt);
-	    eval_delayed (cmd);
+	    exec_delayed (scheme_cmd (cmd));
 	    temp_invalid_cursor= true;
 	  }
 	}
@@ -307,6 +305,7 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
 void
 edit_interface_rep::notify_change (int change) {
   env_change= env_change | change;
+  needs_update ();
 }
 
 bool
@@ -421,13 +420,9 @@ edit_interface_rep::apply_changes () {
   if (env_change & THE_ENVIRONMENT)
     environment_update ();
   if (env_change & THE_TREE) {
-    //global_trace_subtree (rp);
-    tree raw= global_get_subtree (rp);
     cout << HRULE;
-    mem= evaluate (ste, raw);
+    mem= evaluate (ste, cct);
     tree rew= mem->get_tree ();
-    //cout << HRULE;
-    //print_tree (mem);
     cout << HRULE;
     cout << tree_to_texmacs (rew) << LF;
     //print_tree (rew);
@@ -553,13 +548,14 @@ edit_interface_rep::full_screen_mode (bool flag) {
 
 void
 edit_interface_rep::before_menu_action () {
-  mark_undo_blocks ();
+  start_editing ();
   set_input_normal ();
 }
 
 void
 edit_interface_rep::after_menu_action () {
   notify_change (THE_DECORATIONS);
+  end_editing ();
 }
 
 /******************************************************************************
