@@ -4,10 +4,9 @@
 * DESCRIPTION: the interface for TeXmacs
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef EDIT_INTERFACE_H
@@ -15,9 +14,6 @@
 #include "editor.hpp"
 #include "timer.hpp"
 #include "widget.hpp"
-#ifndef AQUATEXMACS
-#include "Widkit/Event/attribute_event.hpp"
-#endif
 
 #define INPUT_NORMAL      0
 #define INPUT_SEARCH      1
@@ -37,7 +33,7 @@ protected:
   bool          full_screen;   // full screen mode ?
   bool          got_focus;     // do we have keyboard focus ?
   string        sh_s;          // current string for shorthands
-  int           sh_len;        // length of translation
+  double        sh_mark;       // 0 or mark for undoing shortcut
   widget        popup_win;     // the current popup window
   string        message_l;     // a left message to display
   string        message_r;     // a right message to display
@@ -130,6 +126,7 @@ public:
   bool in_replace_mode ();
   bool in_spell_mode ();
   bool kbd_get_command (string which, string& help, command& cmd);
+  void interrupt_shortcut ();
   bool try_shortcut (string comb);
   void key_press (string key);
   void emulate_keyboard (string keys, string action= "");
@@ -137,6 +134,7 @@ public:
   void complete_message ();
   void complete_start (string prefix, array<string> compls);
   bool complete_keypress (string key);
+  bool session_complete_try (tree t);
 
   /* mouse handling */
   void mouse_any (string s, SI x, SI y, int mods, time_t t);
