@@ -4,15 +4,15 @@
 * DESCRIPTION: Keyboard handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_interface.hpp"
 #include "analyze.hpp"
 #include "tm_buffer.hpp"
+#include "archiver.hpp"
 
 /******************************************************************************
 * Basic subroutines for keyboard handling
@@ -25,8 +25,8 @@ edit_interface_rep::get_input_mode () {
 
 void
 edit_interface_rep::set_input_mode (int mode) {
-  sh_s  = string ("");    // avoids keyboard shorthands when
-  sh_len= 0;              // using the menu between two keystrokes
+  interrupt_shortcut ();
+  // avoids keyboard shorthands when using the menu between two keystrokes
 
   if ((mode == INPUT_NORMAL) && (input_mode != INPUT_NORMAL)) {
     selection_cancel ();
@@ -69,6 +69,13 @@ edit_interface_rep::kbd_get_command (string which, string& help, command& c) {
 * Main keyboard routines
 ******************************************************************************/
 
+void
+edit_interface_rep::interrupt_shortcut () {
+  if (sh_mark != 0) mark_end (sh_mark);
+  sh_s= "";
+  sh_mark= 0;
+}
+
 bool
 edit_interface_rep::try_shortcut (string comb) {
   int     status;
@@ -77,29 +84,26 @@ edit_interface_rep::try_shortcut (string comb) {
   string  help;
 
   sv->get_keycomb (comb, status, cmd, shorth, help);
-  if ((status & 1) == 1) {
-    if (sh_len>0) {
-      tp= path_add (tp, -sh_len);
-      remove (tp, sh_len);
+  //cout << "Try " << comb << " -> " << shorth << ", " << help
+  //<< "; " << sh_mark << ", " << status << "\n";
+  if (status != 0) {
+    if (sh_mark != 0 && !mark_cancel (sh_mark)) {
+      sh_mark= 0;
+      return false;
     }
-    cmd ();
-    sh_s  = string ("");
-    sh_len= 0;
-    return true;
-  }
-  if ((status & 2) == 2) {
     sh_s= comb;
-    if (sh_len>0) {
-      tp= path_add (tp, -sh_len);
-      if (sh_len>0) remove (tp, sh_len);
-    }
-    if (N(shorth)>0) insert_tree (shorth);
-    sh_len= N (shorth);
+    sh_mark= new_marker ();
+    mark_start (sh_mark);
     string rew= sv->kbd_post_rewrite (sh_s);
     if (N(help)>0) set_message (help, rew);
+    else if (shorth == rew) set_message ("keyboard shorthand: " * rew);
     else set_message ("keyboard shorthand: " * rew, shorth);
-    return true;
+    if ((status & 1) == 1) cmd ();
+    else if (N(shorth) > 0) insert_tree (shorth);
+    //cout << "Mark= " << sh_mark << "\n";
+    return true;    
   }
+
   return false;
 }
 
@@ -142,18 +146,17 @@ edit_interface_rep::key_press (string key) {
   string new_sh= N(sh_s)==0? key: sh_s * " " * key;
   if (try_shortcut (new_sh)) return;
   if (new_sh != key) {
-    sh_s  = string ("");
-    sh_len= 0;
+    interrupt_shortcut ();
     if (try_shortcut (key)) return;
   }
 
   string rew= sv->kbd_post_rewrite (key);
   if (N(rew)==1) {
     int i ((unsigned char) rew[0]);
-    if ((((i>=32) && (i<=127)) || ((i>=128) && (i <=255)))
-     && !inside_active_graphics ()) insert_tree (rew);
-    sh_s  = string ("");
-    sh_len= 0;
+    if ((((i >= 32) && (i <= 127)) || ((i >= 128) && (i <= 255))) &&
+	!inside_active_graphics ())
+      insert_tree (rew);
+    interrupt_shortcut ();
   }
 }
 
@@ -179,9 +182,10 @@ edit_interface_rep::emulate_keyboard (string keys, string action) {
 
 void
 edit_interface_rep::handle_keypress (string key, time_t t) {
-  mark_undo_blocks ();
+  start_editing ();
   key_press (key); (void) t;
   notify_change (THE_DECORATIONS);
+  end_editing ();
 }
 
 void
