@@ -4,34 +4,38 @@
 * DESCRIPTION: Mouse handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_interface.hpp"
 #include "tm_buffer.hpp"
 #include "timer.hpp"
+#include "link.hpp"
+#include "analyze.hpp"
+#include "drd_mode.hpp"
+#include "message.hpp"
+#include "window.hpp"
 
 /******************************************************************************
 * dispatching
 ******************************************************************************/
 
 void
-edit_interface_rep::mouse_any (string type, SI x, SI y, time_t t) {
+edit_interface_rep::mouse_any (string type, SI x, SI y, int mods, time_t t) {
   last_x= x; last_y= y;
-  buf->mark_undo_block ();
+  if (type != "move" || (is_attached (this) && !check_event (MOTION_EVENT)))
+    update_active_loci ();
 
   if (type == "leave")
     set_pointer ("XC_top_left_arrow");
   if ((type != "move") && (type != "enter") && (type != "leave"))
     set_input_normal ();
-  if ((popup_win != NULL) && (type != "leave")) {
-    popup_win->map ();
-    delete popup_win;
-    popup_win= NULL;
-    this << emit_mouse_grab (false);
+  if (!is_nil (popup_win) && (type != "leave")) {
+    set_visibility (popup_win, false);
+    destroy_window_widget (popup_win);
+    popup_win= widget ();
   }
 
   if (inside_graphics (false)) {
@@ -51,7 +55,7 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, time_t t) {
       dragging= true;
     }
     else if (dragging && (type == "move"))
-      type2= "dragging"; 
+      type2= "dragging";
     if (dragging && (type == "release-left"))
       type2= "end-drag";
 
@@ -61,7 +65,7 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, time_t t) {
       right_dragging= true;
     }
     else if (right_dragging && (type == "move"))
-      type2= "right-dragging"; 
+      type2= "right-dragging";
     if (right_dragging && (type == "release-right"))
       type2= "end-right-drag";
 
@@ -69,27 +73,27 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, time_t t) {
       dragging= start_drag= false;
     if (type == "release-right")
       right_dragging= start_right_drag= false;
-    if (mouse_graphics (type2, x, y, t)) return;
+    if (mouse_graphics (type2, x, y, mods, t)) return;
     if (!over_graphics (x, y))
       eval ("(graphics-reset-context 'text-cursor)");
   }
 
   if (type == "press-left") mouse_click (x, y);
   if (dragging && (type == "move")) {
-    if (attached () && dis->check_event (DRAG_EVENT)) return;
+    if (is_attached (this) && check_event (DRAG_EVENT)) return;
     mouse_drag (x, y);
   }
   if (type == "release-left" || type == "release-right") {
     dragging= right_dragging= false;
-    this << emit_mouse_grab (false);
-    if ((t >= last_click) && ((t - last_click) <= 250)) {
+    send_mouse_grab (this, false);
+    if ((t >= last_click) && ((t - last_click) <= 500)) {
       last_click= t;
       if (mouse_extra_click (x, y))
-	last_click= t- 1000;
+	last_click= t- 2000;
     }
     else {
       last_click= t;
-      mouse_select (x, y);
+      mouse_select (x, y, mods);
     }
   }
   if (type == "press-middle") mouse_paste (x, y);
@@ -110,72 +114,57 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, time_t t) {
 
 void
 edit_interface_rep::mouse_click (SI x, SI y) {
-  if (eb->action ("click" , x, y, 0) != "") return;
+  if (eb->action ("click", x, y, 0) != "") return;
   start_x   = x;
   start_y   = y;
   start_drag= dragging= true;
   start_right_drag= right_dragging= false;
-  this << emit_mouse_grab (true);
+  send_mouse_grab (this, true);
 }
 
 bool
 edit_interface_rep::mouse_extra_click (SI x, SI y) {
   go_to (x, y);
-
-  // temporary hack for clickable footnotes
-  path p= path_up (tp);
-  if (rp < p) {
-    if (is_compound (subtree (et, p), "footnote", 1)) {
-      go_to (start (et, p * 0));
-      return true;
-    }
-    tree st= subtree (et, path_up (p));
-    if (is_concat (st) && ((last_item (p) + 1) < N(st)))
-      if (last_item (tp) == right_index (st [last_item (p)]))
-	if (is_compound (subtree (et, path_inc (p)), "footnote", 1)) {
-	  go_to (start (et, path_inc (p) * 0));
-	  return true;
-	}
-    path q= search_upwards ("footnote");
-    if ((!nil (q)) && (tp == start (et, q * 0))) {
-      go_to (end (et, q));
-      return true;
-    }
-  }
-  // end temporary hack
-
-  if (eb->action ("double-click" , x, y, 0) != "") return true;
+  if (eb->action ("double-click", x, y, 0) != "") return true;
   go_to (x, y);
   path p1, p2;
   get_selection (p1, p2);
   if ((p1==p2) || path_less (tp, p1) || path_less (p2, tp)) select (tp, tp);
   select_enlarge ();
+  if (selection_active_any ())
+    selection_set ("mouse", selection_get (), true);
   return false;
 }
 
 void
 edit_interface_rep::mouse_drag (SI x, SI y) {
   if (inside_graphics ()) return;
-  if (eb->action ("drag" , x, y, 0) != "") return;
+  if (eb->action ("drag", x, y, 0) != "") return;
+  go_to (x, y);
   end_x  = x;
   end_y  = y;
   selection_visible ();
-  path p1= tree_path (start_x, start_y, 0);
-  path p2= tree_path (end_x  , end_y  , 0);
+  path sp= find_innermost_scroll (eb, tp);
+  path p1= tree_path (sp, start_x, start_y, 0);
+  path p2= tree_path (sp, end_x  , end_y  , 0);
   if (path_inf (p2, p1)) {
     path temp= p1;
     p1= p2;
     p2= temp;
   }
-  set_selection (p1, p2);
   if ((p1 == p2) && start_drag) return;
+  set_selection (p1, p2);
   start_drag= start_right_drag= false;
   notify_change (THE_SELECTION);
 }
 
 void
-edit_interface_rep::mouse_select (SI x, SI y) {
+edit_interface_rep::mouse_select (SI x, SI y, int mods) {
   if (eb->action ("select" , x, y, 0) != "") return;
+  if (!is_nil (active_ids) && (mods & 256) == 0) {
+    call ("link-follow-ids", object (active_ids));
+    return;
+  }
   tree g;
   bool b0= inside_graphics (false);
   bool b= inside_graphics ();
@@ -192,34 +181,45 @@ edit_interface_rep::mouse_select (SI x, SI y) {
     invalidate_graphical_object ();
     eval ("(graphics-reset-context 'exit)");
   }
+  if (start_drag) {
+    path sp= find_innermost_scroll (eb, tp);
+    path p0= tree_path (sp, x, y, 0);
+    set_selection (p0, p0);
+    notify_change (THE_SELECTION);
+  }
   if (selection_active_any ())
-    selection_set ("primary", selection_get (), true);
+    selection_set ("mouse", selection_get (), true);
 }
 
 void
 edit_interface_rep::mouse_paste (SI x, SI y) { (void) x; (void) y;
-  if (eb->action ("paste" , x, y, 0) != "") return;
-  selection_copy ();
-  selection_paste ();
+  if (eb->action ("paste", x, y, 0) != "") return;
+  go_to (x, y);
+  selection_paste ("mouse");
 }
 
 void
 edit_interface_rep::mouse_adjust (SI x, SI y) {
-  if (eb->action ("adjust" , x, y, 0) != "") return;
+  if (eb->action ("adjust", x, y, 0) != "") return;
   x /= sfactor; y /= sfactor;
   abs_round (x, y);
-  if (popup_win == NULL) {
+  if (is_nil (popup_win)) {
     SI wx, wy;
-    win->get_position (wx, wy);
+    ::get_position (get_window (this), wx, wy);
     widget wid;
     SERVER (menu_widget ("(vertical (link texmacs-popup-menu))", wid));
-    widget popup_wid= popup_widget (wid, center);
-    popup_win= popup_window (popup_wid, wx+ ox+ x, wy+ oy+ y);
-    popup_win->map ();
-    this << emit_mouse_grab (true);
-    popup_wid << set_integer ("grabbed", 1);
-    // popup_wid << set_integer ("freeze", 1);
-    popup_wid << emit_mouse_grab (true);
+    widget popup_wid= popup_widget (wid);
+    popup_win= ::popup_window_widget (popup_wid, "Popup menu");
+#if defined (QTTEXMACS) || defined(AQUATEXMACS)
+    SI ox, oy;
+    get_position (this, ox, oy);
+    set_position (popup_win, wx+     x, wy+     y);
+#else
+    set_position (popup_win, wx+ ox+ x, wy+ oy+ y);
+#endif
+    set_visibility (popup_win, true);
+    send_keyboard_focus (this);
+    send_mouse_grab (popup_wid, true);
   }
 }
 
@@ -229,9 +229,34 @@ edit_interface_rep::mouse_scroll (SI x, SI y, bool up) {
   if (eb->action (action , x, y, 0) != "") return;
   SI dy= 100*PIXEL;
   if (!up) dy= -dy;
-  SERVER (scroll_where (x, y));
-  y += dy;
-  SERVER (scroll_to (x, y));
+  path sp= find_innermost_scroll (eb, tp);
+  if (is_nil (sp)) {
+    SERVER (scroll_where (x, y));
+    y += dy;
+    SERVER (scroll_to (x, y));
+  }
+  else {
+    SI x, y, sx, sy;
+    rectangle outer, inner;
+    find_canvas_info (eb, sp, x, y, sx, sy, outer, inner);
+    SI ty= inner->y2 - inner->y1;
+    SI cy= outer->y2 - outer->y1;
+    if (ty > cy) {
+      tree   old_yt= eb[path_up (sp)]->get_info ("scroll-y");
+      string old_ys= as_string (old_yt);
+      double old_p = 0.0;
+      if (ends (old_ys, "%")) old_p= as_double (old_ys (0, N(old_ys)-1));
+      double new_p= old_p + 100.0 * ((double) dy) / ((double) (ty - cy));
+      new_p= max (min (new_p, 100.0), 0.0);
+      tree new_yt= as_string (new_p) * "%";
+      if (new_yt != old_yt && is_accessible (obtain_ip (old_yt))) {
+	object fun= symbol_object ("tree-set");
+	object cmd= list_object (fun, old_yt, new_yt);
+	exec_delayed (scheme_cmd (cmd));
+	temp_invalid_cursor= true;
+      }
+    }
+  }
 }
 
 /******************************************************************************
@@ -242,7 +267,7 @@ cursor
 edit_interface_rep::get_cursor () {
   if (inside_graphics ()) {
     frame f= find_frame ();
-    if (!nil (f)) {
+    if (!is_nil (f)) {
       point p= f [point (last_x, last_y)];
       p= f (adjust (p));
       SI x= (SI) p[0];
@@ -255,24 +280,67 @@ edit_interface_rep::get_cursor () {
 
 void
 edit_interface_rep::set_pointer (string name) {
-  sv->get_display()->set_pointer(name);
+  send_mouse_pointer (this, name);
 }
 
 void
 edit_interface_rep::set_pointer (
   string curs_name, string mask_name)
 {
-  sv->get_display()->set_pointer(curs_name, mask_name);
+  send_mouse_pointer (this, curs_name, mask_name);
 }
 
 /******************************************************************************
-* event handlers
+* Active loci
 ******************************************************************************/
 
 void
-edit_interface_rep::handle_mouse (mouse_event ev) {
-  string type= ev->type;
-  SI     x   = ev->x*sfactor;
-  SI     y   = ev->y*sfactor;
-  mouse_any (type, x, y, ev->t);
+edit_interface_rep::update_active_loci () {
+  int old_mode= set_access_mode (DRD_ACCESS_SOURCE);
+  path cp= path_up (tree_path (path (), last_x, last_y, 0));
+  set_access_mode (old_mode);
+  tree mt= subtree (et, cp);
+  path p = cp;
+  list<string> ids1, ids2;
+  rectangles rs1, rs2;
+  eb->loci (last_x, last_y, 0, ids1, rs1);
+  while (rp <= p) {
+    ids2 << get_ids (subtree (et, p));
+    p= path_up (p);
+  }
+
+  locus_new_rects= rectangles ();
+  active_ids= list<string> ();
+  if (!is_nil (ids1 * ids2) && !has_changed (THE_FOCUS)) {
+    list<tree> l= as_list_tree (call ("link-active-upwards", object (mt)));
+    while (!is_nil (l)) {
+      tree lt= l->item;
+      path lp= reverse (obtain_ip (lt));
+      selection sel= eb->find_check_selection (lp * start(lt), lp * end(lt));
+      rs2 << outline (sel->rs, pixel);
+      l= l->next;
+    }
+    ids1= as_list_string (call ("link-active-ids", object (ids1)));
+    ids2= as_list_string (call ("link-active-ids", object (ids2)));
+    if (is_nil (ids1)) rs1= rectangles ();
+    // FIXME: we should keep track which id corresponds to which rectangle
+    locus_new_rects= rs1 * rs2;
+    active_ids= ids1 * ids2;
+  }
+  if (locus_new_rects != locus_rects) notify_change (THE_LOCUS);
+}
+
+/******************************************************************************
+* Event handlers
+******************************************************************************/
+
+void
+edit_interface_rep::handle_mouse (string kind, SI x, SI y, int m, time_t t) {
+  start_editing ();
+  x *= sfactor;
+  y *= sfactor;
+  //cout << kind << " (" << x << ", " << y << "; " << m << ")"
+  //<< " at " << t << "\n";
+  mouse_any (kind, x, y, m, t);
+  end_editing ();
 }
