@@ -4,10 +4,9 @@
 * DESCRIPTION: repaint invalid rectangles
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Interface/edit_interface.hpp"
@@ -138,15 +137,14 @@ edit_interface_rep::draw_graphics (renderer ren) {
       eval ("(graphics-reset-context 'graphics-cursor)");
       draw_graphical_object (ren);
       string tm_curs= as_string (eval ("graphics-texmacs-pointer"));
-      if (tm_curs != "none")
+      if (tm_curs != "none") {
 	if (tm_curs == "graphics-cross") {
 	  ren->set_line_style (pixel);
 	  ren->set_color (red);
 	  ren->line (cu->ox, cu->oy-5*pixel, cu->ox, cu->oy+5*pixel);
 	  ren->line (cu->ox-5*pixel, cu->oy, cu->ox+5*pixel, cu->oy);
         }
-	else
-	if (tm_curs == "graphics-cross-arrows") {
+	else if (tm_curs == "graphics-cross-arrows") {
 	  static int s= 6*pixel, a= 2*pixel;
 	  ren->set_line_style (pixel);
 	  ren->set_color (red);
@@ -161,6 +159,7 @@ edit_interface_rep::draw_graphics (renderer ren) {
 	  ren->line (cu->ox+s, cu->oy, cu->ox+s-a, cu->oy+a);
 	  ren->line (cu->ox+s, cu->oy, cu->ox+s-a, cu->oy-a);
         }
+      }
     }
     else eval ("(graphics-reset-context 'text-cursor)");
   }
