@@ -4,14 +4,14 @@
 * DESCRIPTION: treat deletions
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_text.hpp"
 #include "tree_traverse.hpp"
+#include "analyze.hpp"
 
 /******************************************************************************
 * Getting the point where to delete
@@ -25,7 +25,7 @@ edit_text_rep::get_deletion_point (
   p= tp;
   if (forward) {
     //cout << HRULE;
-    if ((rp < p) && (N(p-rp) >= 2) &&
+    if ((rp < p) && (N (p / rp) >= 2) &&
 	is_concat (subtree (et, path_up (p, 2))) &&
 	(last_item (p) == right_index (subtree (et, path_up (p)))) &&
 	(last_item (path_up (p)) < (N (subtree (et, path_up (p, 2))) - 1)))
@@ -84,6 +84,7 @@ edit_text_rep::remove_text (bool forward) {
 	tree u= subtree (et, path_up (p));
 	if (is_func (u, _FLOAT) || is_func (u, WITH) ||
 	    is_func (u, STYLE_WITH) || is_func (u, VAR_STYLE_WITH) ||
+	    is_func (u, LOCUS) ||
 	    is_extension (u))
 	  {
 	    if (is_extension (u) && (N(u) > 1)) {
@@ -108,6 +109,12 @@ edit_text_rep::remove_text (bool forward) {
 	    }
 	    else go_to_border (path_up (p), !forward);
 	  }
+	else if (is_func (u, TABLE) || is_func (u, SUBTABLE) ||
+		 is_func (u, CELL) || is_func (u, ROW) ||
+		 is_func (u, TFORMAT)) {
+	  if (t == tree (DOCUMENT, ""))
+	    back_in_table (u, p, forward);
+	} 
       }
       return;
     }
@@ -133,9 +140,7 @@ edit_text_rep::remove_text (bool forward) {
   if (forward && is_atomic (t) && (last != rix)) {
     language lan= get_env_language ();
     int end= last;
-    if (lan->enc->token_forward (t->label, end))
-      fatal_error ("bad cursor position in string",
-		   "edit_text_rep::remove_text");
+    tm_char_forwards (t->label, end);
     remove (p * last, end-last);
     correct (path_up (p));
     return;
@@ -144,9 +149,7 @@ edit_text_rep::remove_text (bool forward) {
   if ((!forward) && is_atomic (t) && (last != 0)) {
     language lan= get_env_language ();
     int start= last;
-    if (lan->enc->token_backward (t->label, start))
-      fatal_error ("bad cursor position in string",
-		   "edit_text_rep::remove_text");
+    tm_char_backwards (t->label, start);
     remove (p * start, last-start);
     correct (path_up (p));
     return;
@@ -185,6 +188,7 @@ edit_text_rep::remove_text (bool forward) {
     case WITH:
     case STYLE_WITH:
     case VAR_STYLE_WITH:
+    case LOCUS:
       go_to_border (p * (N(t) - 1), forward);
       return;
     case VALUE:
@@ -219,6 +223,7 @@ edit_text_rep::remove_text (bool forward) {
     case WITH:
     case STYLE_WITH:
     case VAR_STYLE_WITH:
+    case LOCUS:
       back_in_with (u, p, forward);
       return;
     default:
