@@ -4,10 +4,9 @@
 * DESCRIPTION: modify tables
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_table.hpp"
@@ -555,11 +554,11 @@ edit_table_rep::table_bound (
 
   int i, j, ii, jj, nr_rows, nr_cols;
   table_get_extents (fp, nr_rows, nr_cols);
-  tree** rs= new tree*[nr_rows];
-  tree** cs= new tree*[nr_rows];
+  tree** rs= tm_new_array<tree*> (nr_rows);
+  tree** cs= tm_new_array<tree*> (nr_rows);
   for (i=0; i<nr_rows; i++) {
-    rs[i]= new tree[nr_cols];
-    cs[i]= new tree[nr_cols];
+    rs[i]= tm_new_array<tree> (nr_cols);
+    cs[i]= tm_new_array<tree> (nr_cols);
   }
   table_get_format (fp, CELL_ROW_SPAN, rs, nr_rows, nr_cols);
   table_get_format (fp, CELL_COL_SPAN, cs, nr_rows, nr_cols);
@@ -584,11 +583,11 @@ edit_table_rep::table_bound (
     }
   
   for (i=0; i<nr_rows; i++) {
-    delete[] rs[i];
-    delete[] cs[i];
+    tm_delete_array (rs[i]);
+    tm_delete_array (cs[i]);
   }
-  delete[] rs;
-  delete[] cs;
+  tm_delete_array (rs);
+  tm_delete_array (cs);
 }
 
 void
@@ -865,7 +864,7 @@ search_decoration (tree T, int& row, int& col) {
       if (C == tree (TMARKER)) return;
     }
   }
-  fatal_error ("decoration not found", "table_undecorate");
+  FAILED ("decoration not found");
 }
 
 static tree
@@ -899,8 +898,8 @@ table_undecorate (tree st, int row, int col) {
 	tree F= table_format_undecorate (st, row, col, dec_row, dec_col);
 	return F * T;
       }
-  fatal_error ("decoration not found", "table_undecorate");
-  return ""; // avoids error message when C++ compiler behaves badly
+  FAILED ("decoration not found");
+  return "";
 }
 
 void
