@@ -4,20 +4,31 @@
 * DESCRIPTION: abstract TeXmacs editors
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef EDITOR_H
 #define EDITOR_H
 #include "typesetter.hpp"
-#include "Widget/attribute_widget.hpp"
+#ifdef AQUATEXMACS
+#include "Cocoa/aqua_simple_widget.h"
+#else
+#ifdef QTTEXMACS
+#include "Qt/qt_simple_widget.hpp"
+#else
+#include "Widkit/simple_wk_widget.hpp"
+#endif
+#endif
 #include "server.hpp"
 #include "scheme.hpp"
 #include "drd_info.hpp"
-#define TEXMACS_COPYRIGHT (string("(c) 1999-2003 by Joris van der Hoeven"))
+#ifdef EXPERIMENTAL
+#include "../Style/Environment/environment.hpp"
+#include "../Style/Memorizer/memorizer.hpp"
+#endif
+#define TEXMACS_COPYRIGHT (string("(c) 1999-2006 by Joris van der Hoeven"))
 
 #define THE_CURSOR 1
 #define THE_FOCUS 2
@@ -26,13 +37,14 @@
 #define THE_SELECTION 16
 #define THE_DECORATIONS 32
 #define THE_EXTENTS 64
-#define THE_AUTOMATIC_SIZE 128
+#define THE_LOCUS 128
 
 class tm_buffer_rep;
 typedef tm_buffer_rep* tm_buffer;
 class server_rep;
+class modification;
 
-class editor_rep: public attribute_widget_rep {
+class editor_rep: public simple_widget_rep {
 public:
   server_rep* sv;   // the underlying texmacs server
 
@@ -43,6 +55,11 @@ protected:
   box         eb;   // box translation of tree
   path        rp;   // path to the root of the document in et
   path        tp;   // path of cursor in tree
+#ifdef EXPERIMENTAL
+  environment ste;  // environment for style rewriting
+  tree        cct;  // clean copy of the document tree
+  memorizer   mem;  // style converted document tree
+#endif
 
   /* exchanging information with the interface */
   virtual SI      get_window_height () = 0;
@@ -53,8 +70,6 @@ protected:
 
   /* exchanging information with the typesetter */
   virtual typesetter           get_typesetter () = 0;
-  virtual tree                 get_style () = 0;
-  virtual void                 set_style (tree t) = 0;
   virtual hashmap<string,tree> get_init () = 0;
   virtual hashmap<string,tree> get_fin () = 0;
   virtual void                 set_init (hashmap<string,tree> H= tree ("?"))=0;
@@ -88,7 +103,7 @@ protected:
   virtual void back_in_general (tree t, path p, bool forward) = 0;
 
   /* other protected subroutines */
-  virtual path tree_path (SI x, SI y, SI delta) = 0;
+  virtual path tree_path (path sp, SI x, SI y, SI delta) = 0;
   virtual void apply_changes () = 0;
   virtual void animate () = 0;
   virtual void correct_concat (path p, int done=0) = 0;
@@ -101,7 +116,7 @@ protected:
 
 public:
   editor_rep ();
-  editor_rep (server_rep* sv, display dis, tm_buffer buf);
+  editor_rep (server_rep* sv, tm_buffer buf);
   inline virtual ~editor_rep () {}
 
   /* public routines from edit_interface */
@@ -124,18 +139,19 @@ public:
   virtual bool in_search_mode () = 0;
   virtual bool in_replace_mode () = 0;
   virtual bool in_spell_mode () = 0;
+  virtual void interrupt_shortcut () = 0;
   virtual bool kbd_get_command (string cmd_s, string& help, command& cmd) = 0;
   virtual void key_press (string key) = 0;
   virtual void emulate_keyboard (string keys, string action= "") = 0;
   virtual bool complete_try () = 0;
   virtual void complete_start (string prefix, array<string> compls) = 0;
   virtual bool complete_keypress (string key) = 0;
-  virtual unsigned int get_kbd_modifiers () = 0;
-  virtual void mouse_any (string s, SI x, SI y, time_t t) = 0;
+  virtual bool session_complete_try (tree t) = 0;
+  virtual void mouse_any (string s, SI x, SI y, int mods, time_t t) = 0;
   virtual void mouse_click (SI x, SI y) = 0;
   virtual bool mouse_extra_click (SI x, SI y) = 0;
   virtual void mouse_drag (SI x, SI y) = 0;
-  virtual void mouse_select (SI x, SI y) = 0;
+  virtual void mouse_select (SI x, SI y, int mods) = 0;
   virtual void mouse_paste (SI x, SI y) = 0;
   virtual void mouse_adjust (SI x, SI y) = 0;
   virtual void mouse_scroll (SI x, SI y, bool up) = 0;
@@ -148,7 +164,7 @@ public:
   /* public routines from edit_cursor */
   virtual path make_cursor_accessible (path p, bool forwards) = 0;
   virtual void show_cursor_if_hidden () = 0;
-  virtual void go_to (SI x, SI y) = 0;
+  virtual void go_to (SI x, SI y, bool absolute= true) = 0;
   virtual void go_left_physical () = 0;
   virtual void go_right_physical () = 0;
   virtual void go_left () = 0;
@@ -181,7 +197,9 @@ public:
   virtual bool   inside_active_graphics (bool b=true) = 0;
   virtual bool   over_graphics (SI x, SI y) = 0;
   virtual tree   get_graphics () = 0;
-  virtual frame  find_frame () = 0;
+  virtual double get_x () = 0;
+  virtual double get_y () = 0;
+  virtual frame  find_frame (bool last= false) = 0;
   virtual grid   find_grid () = 0;
   virtual void   find_limits (point& lim1, point& lim2) = 0;
   virtual bool   find_graphical_region (SI& x1, SI& y1, SI& x2, SI& y2) = 0;
@@ -193,8 +211,8 @@ public:
   virtual tree   get_graphical_object () = 0;
   virtual void   set_graphical_object (tree t) = 0;
   virtual void   invalidate_graphical_object () = 0;
-  virtual void   draw_graphical_object (ps_device dev) = 0;
-  virtual bool   mouse_graphics (string s, SI x, SI y, time_t t) = 0;
+  virtual void   draw_graphical_object (renderer ren) = 0;
+  virtual bool   mouse_graphics (string s, SI x, SI y, int mods, time_t t) = 0;
 
   /* public routines from edit_typeset */
   virtual void     clear_local_info () = 0;
@@ -204,6 +222,9 @@ public:
   virtual bool     is_length (string s) = 0;
   virtual double   divide_lengths (string l1, string l2) = 0;
   virtual void     drd_update () = 0;
+#ifdef EXPERIMENTAL
+  virtual void     environment_update () = 0;
+#endif
   virtual bool     defined_at_cursor (string var_name) = 0;
   virtual bool     defined_at_init (string var_name) = 0;
   virtual bool     defined_in_init (string var_name) = 0;
@@ -217,45 +238,53 @@ public:
   virtual double   get_env_double (string var_name) = 0;
   virtual double   get_init_double (string var_name) = 0;
   virtual language get_env_language () = 0;
-  virtual tree     exec_texmacs (tree t, path p= 0) = 0;
+  virtual tree     exec_texmacs (tree t, path p) = 0;
+  virtual tree     exec_texmacs (tree t) = 0;
   virtual tree     exec_html (tree t, path p) = 0;
   virtual tree     exec_html (tree t) = 0;
   virtual tree     exec_latex (tree t, path p) = 0;
   virtual tree     exec_latex (tree t) = 0;
   virtual tree     texmacs_exec (tree t) = 0;
+  virtual tree     get_style () = 0;
+  virtual void     set_style (tree t) = 0;
   virtual void     init_style () = 0;
   virtual void     init_style (string style) = 0;
   virtual void     init_add_package (string package) = 0;
   virtual void     init_remove_package (string package) = 0;
   virtual void     init_env (string var, tree by) = 0;
   virtual void     init_default (string var) = 0;
+  virtual void     typeset_invalidate (path p) = 0;
   virtual void     typeset_invalidate_all () = 0;
 
   /* public routines from edit_modify */
-  virtual void assign (path p, tree u) = 0;
-  virtual void insert (path p, tree u) = 0;
-  virtual void remove (path p, int nr) = 0;
-  virtual void split (path p) = 0;
-  virtual void join (path p) = 0;
-  virtual void insert_node (path p, tree t) = 0;
-  virtual void remove_node (path p) = 0;
-  virtual void assign_node (path p, tree_label op) = 0;
-  virtual void finished (path p) = 0;
   virtual void notify_assign (path p, tree u) = 0;
   virtual void notify_insert (path p, tree u) = 0;
   virtual void notify_remove (path p, int nr) = 0;
   virtual void notify_split (path p) = 0;
   virtual void notify_join (path p) = 0;
+  virtual void notify_assign_node (path p, tree_label op) = 0;
   virtual void notify_insert_node (path p, tree t) = 0;
   virtual void notify_remove_node (path p) = 0;
-  virtual void notify_assign_node (path p, tree_label op) = 0;
   virtual void post_notify (path p) = 0;
-  virtual void remove_undo_mark () = 0;
+  virtual void clear_undo_history () = 0;
+  virtual double this_author () = 0;
+  virtual void start_editing () = 0;
+  virtual void end_editing () = 0;
+  virtual void start_slave (double a) = 0;
+  virtual void mark_start (double a) = 0;
+  virtual bool mark_cancel (double a) = 0;
+  virtual void mark_end (double a) = 0;
   virtual void add_undo_mark () = 0;
+  virtual void remove_undo_mark () = 0;
+  virtual int  undo_possibilities () = 0;
   virtual void unredoable_undo () = 0;
-  virtual void forget_undo () = 0;
-  virtual void undo () = 0;
-  virtual void redo () = 0;
+  virtual void undo (int i=0) = 0;
+  virtual int  redo_possibilities () = 0;
+  virtual void redo (int i=0) = 0;
+  virtual void require_save () = 0;
+  virtual void notify_save (bool real_save= true) = 0;
+  virtual bool need_save (bool real_save= true) = 0;
+  virtual void show_history () = 0;
   virtual observer position_new (path p) = 0;
   virtual void position_delete (observer o) = 0;
   virtual void position_set (observer o, path p) = 0;
@@ -325,13 +354,12 @@ public:
   virtual void   table_format_center () = 0;
   virtual void   table_row_decoration (bool forward) = 0;
   virtual void   table_column_decoration (bool forward) = 0;
+  virtual void   table_correct_block_content () = 0;
   virtual void   set_cell_mode (string mode) = 0;
   virtual string get_cell_mode () = 0;
   virtual void   cell_set_format (string var, string val) = 0;
   virtual string cell_get_format (string var) = 0;
   virtual void   cell_del_format (string var= "") = 0;
-  virtual void   cell_multi_paragraph (bool flag) = 0;
-  virtual bool   cell_is_multi_paragraph () = 0;
   virtual void   table_test () = 0;
 
   /* public routines from edit_dynamic */
@@ -359,39 +387,6 @@ public:
   virtual void temp_proof_fix () = 0;
 
   /* public routines from edit_process */
-  virtual void process_mutators () = 0;
-  virtual path get_mutator_path () = 0;
-  virtual time_t get_mutator_time () = 0;
-  virtual void invalidate_mutators () = 0;
-  virtual void insert_mutator (tree body, string cmd) = 0;
-
-  virtual void make_session (string lan, string session) = 0;
-  virtual void start_input (string lan, string session, path p) = 0;
-  virtual void process_input () = 0;
-  virtual void start_output () = 0;
-  virtual void session_message (string l, string r) = 0;
-  virtual void session_use_math_input (bool flag) = 0;
-  virtual bool session_is_using_math_input () = 0;
-  virtual int  status_connection () = 0;
-  virtual bool busy_connection () = 0;
-  virtual void interrupt_connection () = 0;
-  virtual void stop_connection () = 0;
-
-  virtual void session_go_up () = 0;
-  virtual void session_go_down () = 0;
-  virtual void session_go_page_up () = 0;
-  virtual void session_go_page_down () = 0;
-  virtual void session_remove (bool forward) = 0;
-  virtual void session_insert_text_field () = 0;
-  virtual void session_insert_input_below () = 0;
-  virtual void session_insert_input_above () = 0;
-  virtual void session_fold_input () = 0;
-  virtual void session_remove_input (bool forward) = 0;
-  virtual void session_remove_all_outputs () = 0;
-  virtual void session_remove_previous_output () = 0;
-  virtual void session_split () = 0;
-  virtual bool session_complete_try () = 0;
-
   virtual void generate_bibliography (string bib, string sty, string fname)= 0;
   virtual void generate_table_of_contents (string toc) = 0;
   virtual void generate_index (string idx) = 0;
@@ -408,6 +403,7 @@ public:
   virtual void select_from_cursor_if_active () = 0;
   virtual void select_from_keyboard (bool flag) = 0;
   virtual void select_from_shift_keyboard () = 0;
+  virtual void select_enlarge_text () = 0;
   virtual void select_enlarge () = 0;
   virtual void select_enlarge_environmental () = 0;
 
@@ -424,6 +420,7 @@ public:
   virtual void selection_get (path& start, path& end) = 0;
   virtual path selection_get_start () = 0;
   virtual path selection_get_end () = 0;
+  virtual path selection_get_path () = 0;
   virtual void selection_set (string key, tree t, bool persistant= false) = 0;
   virtual void selection_set (tree t) = 0;
   virtual void selection_set_start (path p= path()) = 0;
@@ -503,10 +500,12 @@ public:
   friend class tm_data_rep;
   friend class tm_server_rep;
   friend class server_command_rep;
+  friend void  edit_announce (editor_rep* ed, modification mod);
+  friend void  edit_done (editor_rep* ed, modification mod);
 };
 
 class editor {
-  EXTEND_NULL(widget,editor);
+EXTEND_NULL(widget,editor);
 public:
   inline bool operator == (editor w) { return rep == w.rep; }
   inline bool operator != (editor w) { return rep != w.rep; }
