@@ -4,10 +4,9 @@
 * DESCRIPTION: Renderer for printing post-script graphics
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "printer.hpp"
@@ -60,7 +59,7 @@ printer_rep::printer_rep (
   
   prologue   << "%!PS-Adobe-2.0\n"
 	     << "%%Creator: TeXmacs-" TEXMACS_VERSION "\n"
-	     << "%%Title: " << as_string (ps_file_name) << "\n"
+	     << "%%Title: " << as_string (tail (ps_file_name)) << "\n"
 	     << "%%Pages: " << as_string (nr_pages) << "\n"
 	     << "%%PageOrder: Ascend\n";
   if (page_type != "user")
@@ -372,6 +371,80 @@ find_ps_font_name (string name, string s) {
   return name;
 }
 
+
+#define HEX_PER_LINE 30
+
+static SI parse_length (string pfb, int& pos) {
+  QN c4= (QN) pfb[pos++];
+  QN c3= (QN) pfb[pos++];
+  QN c2= (QN) pfb[pos++];
+  QI c1= (QI) pfb[pos++];
+  return (((((((SI) c1)<<8)+ ((SI) c2))<<8)+ ((SI) c3))<<8)+ c4;
+}
+
+static string pfb_to_pfa (url file) {
+  //cout << "pfb_to_pfa :" << file << LF;
+  string pfb, pfa;
+  QN magic, type = 0;
+  SI length;
+  
+  (void) load_string (file, pfb, true);
+  int pos = 0, size = N(pfb);
+  while ((pos < size) && (type != 3)) {
+    parse (pfb, pos, magic);
+    //cout << "magic:" << as_hexadecimal(magic,2) << LF ;
+    if (magic != 128) {
+      FAILED ("Not a pfb file");
+    }
+    parse (pfb, pos, type);
+    //cout << "type:" << as_hexadecimal(type,2) << LF;
+    switch (type) {
+        
+      case 1 :
+        // plain text
+        length = parse_length (pfb, pos);
+        // parse (pfb, pos, length);
+        //cout << "plain text of size " << length << LF;
+        for (int i=0; i <length; i++) {
+          QI ch;
+          parse(pfb, pos, ch);
+          if (ch == '\r') pfa << "\n";
+          else pfa << ch;
+        }
+        break;
+        
+      case 2 :
+        // binary data
+        length = parse_length (pfb, pos);
+        //        parse (pfb, pos, length);
+        //cout << "binary data of size " << length << LF;
+        for (int i=0; i <length; i++) {
+          QI ch;
+          parse(pfb, pos, ch);
+          pfa << as_hexadecimal (ch, 2);
+          if ((i+1) % HEX_PER_LINE == 0) pfa << "\n"; 
+        }
+        break;
+        
+      case 3 :
+        //cout << "end of file"  << LF;
+        // end of file
+        break;
+        
+      default : 
+        FAILED ("Unknown field type while reading PFB file");
+        break;
+        
+    }
+  }
+  return pfa;
+}
+
+#undef HEX_PER_LINE
+
+
+
+
 void
 printer_rep::generate_tex_fonts () {
   hashset<string> done;
@@ -391,10 +464,7 @@ printer_rep::generate_tex_fonts () {
       root= fn_name (0, pos2);
       url u= tt_font_find (root);
       if (suffix (u) == "pfb") {
-	url v= url_temp (".pfa");
-	system ("pfb2pfa", u, v);
-	(void) load_string (v, ttf, true);
-	remove (v);
+        ttf = pfb_to_pfa (u);
       }
     }
 #endif
@@ -609,7 +679,7 @@ printer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
 void
 printer_rep::xpm (url file_name, SI x, SI y) {
   (void) file_name; (void) x; (void) y;
-  fatal_error ("Not yet implemented", "printer_rep::xpm");
+  FAILED ("not yet implemented");
 }
 
 /*
@@ -755,6 +825,6 @@ renderer
 printer (url ps_file_name, int dpi, int nr_pages,
 	 string page_type, bool landscape, double paper_w, double paper_h)
 {
-  return new printer_rep (ps_file_name, dpi, nr_pages,
+  return tm_new<printer_rep> (ps_file_name, dpi, nr_pages,
 			  page_type, landscape, paper_w, paper_h);
 }
