@@ -5,10 +5,9 @@
 ;; DESCRIPTION : Building basic glue
 ;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public licence and comes WITHOUT
-;; ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -21,6 +20,9 @@
   (texmacs-version-release texmacs_version (string string))
   (version-before? version_inf (bool string string))
   (os-win32? os_win32 (bool))
+  (os-mingw? os_mingw (bool))
+  (x-gui? gui_is_x (bool))
+  (qt-gui? gui_is_qt (bool))
   (win32-display win32_display (void string))
   (scheme-dialect scheme_dialect (string))
   (get-texmacs-path get_texmacs_path (string))
@@ -32,6 +34,7 @@
   (var-eval-system var_eval_system (string string))
   (get-locale-language get_locale_language (string))
   (texmacs-time texmacs_time (int))
+  (texmacs-memory mem_used (int))
   (bench-print bench_print (void string))
   (bench-print-all bench_print (void))
   (system-wait system_wait (void string string))
@@ -41,12 +44,16 @@
   (math-symbol-type math_symbol_type (string string))
   (object->command as_command (command object))
   (exec-delayed exec_delayed (void object))
+  (exec-delayed-pause exec_delayed_pause (void object))
   (set-input-language set_input_language (void string))
   (get-input-language get_input_language (string))
   (set-output-language gui_set_output_language (void string))
   (get-output-language get_output_language (string))
   (translate translate (string string string string))
   (color named_color (int string))
+  (new-author new_author (double))
+  (set-author set_author (void double))
+  (get-author get_author (double))
 
   ;; routines for trees
   (tree->stree tree_to_scheme_tree (scheme_tree tree))
@@ -78,6 +85,7 @@
   (tree-load-inclusion load_inclusion (tree url))
   (tree-as-string var_as_string (string tree))
   (tree-extents tree_extents (tree content))
+  (tree-empty? is_empty (bool content))
 
   (tree-assign tree_assign (tree tree content))
   (tree-var-insert tree_insert (tree tree int content))
@@ -99,8 +107,14 @@
   (path-previous previous_valid (path content path))
   (path-next-word next_word (path content path))
   (path-previous-word previous_word (path content path))
+  (path-next-node next_node (path content path))
+  (path-previous-node previous_node (path content path))
   (path-next-tag next_tag (path content path scheme_tree))
   (path-previous-tag previous_tag (path content path scheme_tree))
+  (path-next-tag-same-argument next_tag_same_argument
+			       (path content path scheme_tree))
+  (path-previous-tag-same-argument previous_tag_same_argument
+				   (path content path scheme_tree))
   (path-next-argument next_argument (path content path))
   (path-previous-argument previous_argument (path content path))
 
@@ -129,6 +143,8 @@
   (locase-all locase_all (string string))
   (string-union string_union (string string string))
   (string-minus string_minus (string string string))
+  (string-encode tm_encode (string string))
+  (string-decode tm_decode (string string))
   (escape-generic escape_generic (string string))
   (escape-verbatim escape_verbatim (string string))
   (escape-shell escape_sh (string string))
@@ -143,14 +159,18 @@
   (old-xml-cdata->tm old_xml_cdata_to_tm (string string))
   (xml-unspace xml_unspace (string string bool bool))
 
+  ;; Packrat grammar and parsing tools
+  (define-grammar-rule define_grammar_rule (void tree tree))
+  (grammar-parse grammar_parse (int tree string))
+
   ;; further conversion routines for trees and strings
   (parse-texmacs texmacs_document_to_tree (tree string))
   (serialize-texmacs tree_to_texmacs (string tree))
   (parse-texmacs-snippet texmacs_to_tree (tree string))
   (serialize-texmacs-snippet tree_to_texmacs (string tree))
-  (texmacs->scheme tree_to_scheme (string tree))
-  (scheme->texmacs scheme_document_to_tree (tree string))
-  (scheme-snippet->texmacs scheme_to_tree (tree string))
+  (texmacs->stm tree_to_scheme (string tree))
+  (stm->texmacs scheme_document_to_tree (tree string))
+  (stm-snippet->texmacs scheme_to_tree (tree string))
   (cpp-texmacs->verbatim tree_to_verbatim (string tree bool string))
   (cpp-verbatim-snippet->texmacs verbatim_to_tree (tree string bool string))
   (cpp-verbatim->texmacs verbatim_document_to_tree (tree string bool string))
@@ -209,7 +229,6 @@
   (string-load string_load (string url))
   (system-move move (void url url))
   (system-copy copy (void url url))
-  (system-append append (void url url))
   (system-remove remove (void url))
   (system-mkdir mkdir (void url))
   (system-search-score search_score (int url array_string))
@@ -273,13 +292,13 @@
   (enter-secure-mode enter_secure_mode (void))
 
   ;; connections to extern systems
-  (connection-declared? connection_declared (bool string))
+  (connection-start connection_start (string string string))
   (connection-status connection_status (int string string))
-  (connection-start connection_start (string string string bool))
-  (connection-eval connection_eval (tree string string content))
-  (connection-cmd connection_cmd (tree string string string))
   (connection-write connection_write (void string string content))
-  (connection-read connection_read (tree string string string))
+  (connection-cmd connection_cmd (tree string string string))
+  (connection-eval connection_eval (tree string string content))
+  (connection-interrupt connection_interrupt (void string string))
+  (connection-stop connection_stop (void string string))
 
   ;; widgets
   (widget-hmenu horizontal_menu (widget array_widget))
