@@ -4,10 +4,9 @@
 * DESCRIPTION: Glue for linking TeXmacs commands to guile
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include <tm_configure.hpp>
@@ -18,9 +17,11 @@
 #include "convert.hpp"
 #include "file.hpp"
 #include "sys_utils.hpp"
+#include "tmfs.hpp"
+#include "client_server.hpp"
 #include "analyze.hpp"
 #include "tree_traverse.hpp"
-#include "tm_layout.hpp"
+#include "tm_frame.hpp"
 #include "Concat/concater.hpp"
 #include "converter.hpp"
 #include "timer.hpp"
@@ -28,6 +29,10 @@
 #include "Freetype/free_type.hpp"
 #include "Freetype/tt_file.hpp"
 #include "Bibtex/bibtex.hpp"
+#include "link.hpp"
+#include "dictionary.hpp"
+#include "patch.hpp"
+#include "grammar.hpp"
 #include <string.h>
 #include <libguile.h>
 
@@ -51,10 +56,14 @@ scheme_dialect () {
 #else
 #ifdef GUILE_B
   return "guile-b";
+#else
+#ifdef GUILE_C
+  return "guile-c";
 #else
   return "unknown";
 #endif
 #endif
+#endif
 }
 
 string
@@ -67,6 +76,24 @@ texmacs_version (string which) {
   return TEXMACS_VERSION;
 }
 
+bool
+gui_is_x () {
+#ifdef QTTEXMACS
+  return false;
+#else
+  return true;
+#endif
+}
+
+bool
+gui_is_qt () {
+#ifdef QTTEXMACS
+  return true;
+#else
+  return false;
+#endif
+}
+
 bool
 os_win32 () {
 #ifdef OS_WIN32
@@ -76,12 +103,31 @@ os_win32 () {
 #endif
 }
 
+bool
+os_mingw () {
+#ifdef __MINGW32__
+  return true;
+#else
+  return false;
+#endif
+}
+
 void
 win32_display (string s) {
   cout << s;
   cout.flush ();
 }
 
+#ifdef GUILE_C
+#define SET_SMOB(smob,data,type)   \
+  SCM_NEWSMOB (smob, SCM_UNPACK (type), data);
+#else
+#define SET_SMOB(smob,data,type)   \
+  SCM_NEWCELL (smob);              \
+  SCM_SETCAR (smob, (SCM) (type)); \
+  SCM_SETCDR (smob, (SCM) (data));
+#endif
+
 /******************************************************************************
 * Direct access to scheme objects
 ******************************************************************************/
@@ -110,27 +156,31 @@ bool_to_scm (bool flag) {
   return scm_bool2scm (flag);
 }
 
-bool
+#ifndef GUILE_C
+int
 scm_to_bool (SCM flag) {
   return scm_scm2bool (flag);
 }
+#endif
 
 /******************************************************************************
 * Integers
 ******************************************************************************/
 
 #define SCM_ASSERT_INT(i,arg,rout) \
-  SCM_ASSERT (SCM_INUMP (i), i, arg, rout);
+  SCM_ASSERT (scm_is_int (i), i, arg, rout);
 
 SCM
 int_to_scm (int i) {
   return scm_long2scm ((long) i);
 }
 
+#ifndef GUILE_C
 int
 scm_to_int (SCM i) {
   return (int) scm_scm2long (i);
 }
+#endif
 
 /******************************************************************************
 * Floating point numbers
@@ -144,25 +194,25 @@ double_to_scm (double i) {
   return scm_double2scm (i);
 }
 
+#ifndef GUILE_C
 static double
 scm_to_double (SCM i) {
   return scm_scm2double (i);
 }
+#endif
 
 /******************************************************************************
 * Strings
 ******************************************************************************/
 
-#define SCM_MYSTRINGP(s) (SCM_NIMP (s) && SCM_STRINGP (s))
-
 #define SCM_ASSERT_STRING(s,arg,rout) \
-  SCM_ASSERT (SCM_MYSTRINGP (s), s, arg, rout)
+  SCM_ASSERT (scm_is_string (s), s, arg, rout)
 
 SCM
 string_to_scm (string s) {
   char* _s= as_charp (s);
   SCM r= scm_str2scm (_s, N(s));
-  delete[] _s;
+  tm_delete_array (_s);
   return r;
 }
 
@@ -190,7 +240,7 @@ SCM
 symbol_to_scm (string s) {
   char* _s= as_charp (s);
   SCM r= scm_symbol2scm (_s);
-  delete[] _s;
+  tm_delete_array (_s);
   return r;
 }
 
@@ -244,9 +294,7 @@ scm_is_tree (SCM t) {
 SCM
 tree_to_scm (tree t) {
   SCM tree_smob;
-  SCM_NEWCELL (tree_smob);
-  SCM_SETCDR (tree_smob, (SCM) ((void*) (new tree (t))));
-  SCM_SETCAR (tree_smob, tree_tag);
+  SET_SMOB (tree_smob, (void*) (tm_new<tree> (t)), (SCM) tree_tag);
   return tree_smob;
 }
 
@@ -264,7 +312,7 @@ mark_tree (SCM tree_smob) {
 static scm_sizet
 free_tree (SCM tree_smob) {
   tree *ptr = (tree *) SCM_CDR (tree_smob);
-  delete ptr;
+  tm_delete (ptr);
   return 0;
 }
 
@@ -303,9 +351,10 @@ tree_ref (tree t, int i) {
   return t[i];
 }
 
-void
+tree
 tree_set (tree t, int i, tree u) {
   t[i]= u;
+  return u;
 }
 
 tree
@@ -318,6 +367,134 @@ tree_append (tree t1, tree t2) {
   return t1 * t2;
 }
 
+bool
+tree_active (tree t) {
+  path ip= obtain_ip (t);
+  return is_nil (ip) || last_item (ip) != DETACHED;
+}
+
+tree
+tree_child_insert (tree t, int pos, tree x) {
+  cout << "t= " << t << "\n";
+  cout << "x= " << x << "\n";
+  int i, n= N(t);
+  tree r (t, n+1);
+  for (i=0; i<pos; i++) r[i]= t[i];
+  r[pos]= x;
+  for (i=pos; i<n; i++) r[i+1]= t[i];
+  return r;
+}
+
+/******************************************************************************
+* Document modification routines
+******************************************************************************/
+
+extern tree the_et;
+
+tree
+tree_assign (tree r, tree t) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    assign (reverse (ip), copy (t));
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    assign (r, copy (t));
+    return r;
+  }
+}
+
+tree
+tree_insert (tree r, int pos, tree t) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    insert (reverse (path (pos, ip)), copy (t));
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    insert (r, pos, copy (t));
+    return r;
+  }
+}
+
+tree
+tree_remove (tree r, int pos, int nr) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    remove (reverse (path (pos, ip)), nr);
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    remove (r, pos, nr);
+    return r;
+  }
+}
+
+tree
+tree_split (tree r, int pos, int at) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    split (reverse (path (at, pos, ip)));
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    split (r, pos, at);
+    return r;
+  }
+}
+
+tree
+tree_join (tree r, int pos) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    join (reverse (path (pos, ip)));
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    join (r, pos);
+    return r;
+  }
+}
+
+tree
+tree_assign_node (tree r, tree_label op) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    assign_node (reverse (ip), op);
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    assign_node (r, op);
+    return r;
+  }
+}
+
+tree
+tree_insert_node (tree r, int pos, tree t) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    insert_node (reverse (path (pos, ip)), copy (t));
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    insert_node (r, pos, copy (t));
+    return r;
+  }
+}
+
+tree
+tree_remove_node (tree r, int pos) {
+  path ip= copy (obtain_ip (r));
+  if (ip_attached (ip)) {
+    remove_node (reverse (path (pos, ip)));
+    return subtree (the_et, reverse (ip));
+  }
+  else {
+    remove_node (r, pos);
+    return r;
+  }
+}
+
 /******************************************************************************
 * Scheme trees
 ******************************************************************************/
@@ -356,10 +533,10 @@ scm_to_scheme_tree (SCM p) {
     }
     return t;
   }
-  if (gh_symbol_p (p)) return scm_to_symbol (p);
+  if (scm_is_symbol (p)) return scm_to_symbol (p);
   if (scm_is_string (p)) return scm_quote (scm_to_string (p));
   //if (scm_is_string (p)) return "\"" * scm_to_string (p) * "\"";
-  if (SCM_INUMP (p)) return as_string (scm_to_int (p));
+  if (scm_is_int (p)) return as_string ((int) scm_to_int (p));
   if (scm_is_bool (p)) return (scm_to_bool (p)? string ("#t"): string ("#f"));
   if (scm_is_tree (p)) return tree_to_scheme_tree (scm_to_tree (p));
   return "?";
@@ -369,15 +546,28 @@ scm_to_scheme_tree (SCM p) {
 * Content
 ******************************************************************************/
 
+bool
+scm_is_content (SCM p) {
+  if (scm_is_string (p) || scm_is_tree (p)) return true;
+  else if (!scm_is_pair (p) || !scm_is_symbol (SCM_CAR (p))) return false;
+  else {
+    for (p= SCM_CDR (p); !scm_is_null (p); p= SCM_CDR (p))
+      if (!scm_is_content (SCM_CAR (p))) return false;
+    return true;
+  }
+}
+
 #define content tree
-#define SCM_ASSERT_CONTENT(p,arg,rout)
+#define SCM_ASSERT_CONTENT(p,arg,rout) \
+  SCM_ASSERT (scm_is_content (p), p, arg, rout)
 #define content_to_scm tree_to_scm
 
 tree
 scm_to_content (SCM p) {
+  if (scm_is_string (p)) return scm_to_string (p);
   if (scm_is_tree (p)) return scm_to_tree (p);
-  if (scm_is_list (p)) {
-    if (scm_is_null (p) || (!gh_symbol_p (SCM_CAR (p)))) return "?";
+  if (scm_is_pair (p)) {
+    if (!scm_is_symbol (SCM_CAR (p))) return "?";
     tree t (make_tree_label (scm_to_symbol (SCM_CAR (p))));
     p= SCM_CDR (p);
     while (!scm_is_null (p)) {
@@ -386,13 +576,15 @@ scm_to_content (SCM p) {
     }
     return t;
   }
-  if (gh_symbol_p (p)) return scm_to_symbol (p);
-  if (scm_is_string (p)) return scm_to_string (p);
-  if (SCM_INUMP (p)) return as_string (scm_to_int (p));
-  if (scm_is_bool (p)) return (scm_to_bool (p)? string ("#t"): string ("#f"));
   return "?";
 }
 
+static SCM
+contentP (SCM t) {
+  bool b= scm_is_content (t);
+  return bool_to_scm (b);
+}
+
 /******************************************************************************
 * Paths
 ******************************************************************************/
@@ -400,7 +592,7 @@ scm_to_content (SCM p) {
 bool
 scm_is_path (SCM p) {
   if (scm_is_null (p)) return true;
-  else return SCM_INUMP (SCM_CAR (p)) && scm_is_path (SCM_CDR (p));
+  else return scm_is_int (SCM_CAR (p)) && scm_is_path (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_PATH(p,arg,rout) \
@@ -408,14 +600,14 @@ scm_is_path (SCM p) {
 
 SCM
 path_to_scm (path p) {
-  if (nil (p)) return SCM_NULL;
+  if (is_nil (p)) return SCM_NULL;
   else return scm_cons (int_to_scm (p->item), path_to_scm (p->next));
 }
 
 path
 scm_to_path (SCM p) {
   if (scm_is_null (p)) return path ();
-  else return path (scm_to_int (SCM_CAR (p)), scm_to_path (SCM_CDR (p)));
+  else return path ((int) scm_to_int (SCM_CAR (p)), scm_to_path (SCM_CDR (p)));
 }
 
 /******************************************************************************
@@ -432,9 +624,7 @@ static long observer_tag;
 /*static*/ SCM
 observer_to_scm (observer o) {
   SCM observer_smob;
-  SCM_NEWCELL (observer_smob);
-  SCM_SETCDR (observer_smob, (SCM) ((void*) (new observer (o))));
-  SCM_SETCAR (observer_smob, observer_tag);
+  SET_SMOB (observer_smob, (void*) (tm_new<observer> (o)), (SCM) observer_tag);
   return observer_smob;
 }
 
@@ -452,7 +642,7 @@ mark_observer (SCM observer_smob) {
 static scm_sizet
 free_observer (SCM observer_smob) {
   observer *ptr = (observer *) SCM_CDR (observer_smob);
-  delete ptr;
+  tm_delete (ptr);
   return 0;
 }
 
@@ -469,55 +659,10 @@ cmp_observer (SCM o1, SCM o2) {
   return scm_bool2scm (scm_to_observer (o1) == scm_to_observer (o2));
 }
 
-/******************************************************************************
-* Displays
-******************************************************************************/
-
-static long display_tag;
-
-#define scm_is_display(dis) \
-  (SCM_NIMP (dis) && (((long) SCM_CAR (dis)) == display_tag))
-#define SCM_ASSERT_DISPLAY(dis,arg,rout) \
-  SCM_ASSERT (scm_is_display (dis), dis, arg, rout)
-
-/*static*/ SCM
-display_to_scm (display dis) {
-  SCM display_smob;
-  SCM_NEWCELL (display_smob);
-  SCM_SETCDR (display_smob, (SCM) ((void*) (new display (dis))));
-  SCM_SETCAR (display_smob, display_tag);
-  return display_smob;
-}
-
-static display
-scm_to_display (SCM display_smob) {
-  return *((display*) SCM_CDR (display_smob));
-}
-
 static SCM
-mark_display (SCM display_smob) {
-  (void) display_smob;
-  return SCM_BOOL_F;
-}
-
-static scm_sizet
-free_display (SCM display_smob) {
-  display *ptr = (display *) SCM_CDR (display_smob);
-  delete ptr;
-  return 0;
-}
-
-static int
-print_display (SCM display_smob, SCM port, scm_print_state *pstate) {
-  (void) display_smob; (void) pstate;
-  string s= "<display>";
-  scm_display (string_to_scm (s), port);
-  return 1;
-}
-
-static SCM
-cmp_display (SCM dis1, SCM dis2) {
-  return scm_bool2scm (scm_to_display (dis1) == scm_to_display (dis2));
+observerP (SCM t) {
+  bool b= scm_is_observer (t);
+  return bool_to_scm (b);
 }
 
 /******************************************************************************
@@ -539,9 +684,7 @@ scm_is_widget (SCM wid) {
 static SCM
 widget_to_scm (widget wid) {
   SCM widget_smob;
-  SCM_NEWCELL (widget_smob);
-  SCM_SETCDR (widget_smob, (SCM) ((void*) (new widget (wid))));
-  SCM_SETCAR (widget_smob, widget_tag);
+  SET_SMOB (widget_smob, (void*) (tm_new<widget> (wid)), (SCM) widget_tag);
   return widget_smob;
 }
 
@@ -559,7 +702,7 @@ mark_widget (SCM widget_smob) {
 static scm_sizet
 free_widget (SCM widget_smob) {
   widget *ptr = (widget *) SCM_CDR (widget_smob);
-  delete ptr;
+  tm_delete (ptr);
   return 0;
 }
 
@@ -580,53 +723,58 @@ cmp_widget (SCM wid1, SCM wid2) {
 * Widget factory
 ******************************************************************************/
 
-static long make_widget_tag;
+typedef promise<widget> promise_widget;
 
-#define SCM_ASSERT_MAKE_WIDGET(mw,arg,rout) \
-  SCM_ASSERT (scm_is_make_widget (mw), mw, arg, rout)
+static long promise_widget_tag;
 
-#define scm_is_make_widget(mw) \
-  (SCM_NIMP (mw) && (((long) SCM_CAR (mw)) == make_widget_tag))
+#define SCM_ASSERT_PROMISE_WIDGET(pw,arg,rout) \
+  SCM_ASSERT (scm_is_promise_widget (pw), pw, arg, rout)
+
+#define scm_is_promise_widget(pw) \
+  (SCM_NIMP (pw) && (((long) SCM_CAR (pw)) == promise_widget_tag))
 
 static SCM
-make_widget_to_scm (make_widget mw) {
-  SCM make_widget_smob;
-  SCM_NEWCELL (make_widget_smob);
-  SCM_SETCDR (make_widget_smob, (SCM) ((void*) (new make_widget (mw))));
-  SCM_SETCAR (make_widget_smob, make_widget_tag);
-  return make_widget_smob;
+promise_widget_to_scm (promise_widget pw) {
+  SCM promise_widget_smob;
+  SET_SMOB (promise_widget_smob,
+	    (void*) (tm_new<promise_widget> (pw)),
+	    (SCM) promise_widget_tag);
+  return promise_widget_smob;
 }
 
-static make_widget
-scm_to_make_widget (SCM make_widget_smob) {
-  return *((make_widget*) SCM_CDR (make_widget_smob));
+static promise_widget
+scm_to_promise_widget (SCM promise_widget_smob) {
+  return *((promise_widget*) SCM_CDR (promise_widget_smob));
 }
 
 static SCM
-mark_make_widget (SCM make_widget_smob) {
-  (void) make_widget_smob;
+mark_promise_widget (SCM promise_widget_smob) {
+  (void) promise_widget_smob;
   return SCM_BOOL_F;
 }
 
 static scm_sizet
-free_make_widget (SCM make_widget_smob) {
-  make_widget *ptr = (make_widget *) SCM_CDR (make_widget_smob);
-  delete ptr;
+free_promise_widget (SCM promise_widget_smob) {
+  promise_widget *ptr = (promise_widget *) SCM_CDR (promise_widget_smob);
+  tm_delete (ptr);
   // should be replaced by total size of the widget factory
   return 0;
 }
 
 static int
-print_make_widget (SCM make_widget_smob, SCM port, scm_print_state *pstate) {
-  (void) make_widget_smob; (void) pstate;
-  string s= "<make-widget>";
+print_promise_widget (SCM promise_widget_smob, SCM port,
+		      scm_print_state *pstate)
+{
+  (void) promise_widget_smob; (void) pstate;
+  string s= "<promise-widget>";
   scm_display (string_to_scm (s), port);
   return 1;
 }
 
 static SCM
-cmp_make_widget (SCM mw1, SCM mw2) {
-  return scm_bool2scm (scm_to_make_widget (mw1) == scm_to_make_widget (mw2));
+cmp_promise_widget (SCM pw1, SCM pw2) {
+  return scm_bool2scm (scm_to_promise_widget (pw1) ==
+		       scm_to_promise_widget (pw2));
 }
 
 /******************************************************************************
@@ -643,9 +791,7 @@ static long command_tag;
 static SCM
 command_to_scm (command cmd) {
   SCM command_smob;
-  SCM_NEWCELL (command_smob);
-  SCM_SETCDR (command_smob, (SCM) ((void*) (new command (cmd))));
-  SCM_SETCAR (command_smob, command_tag);
+  SET_SMOB (command_smob, (void*) (tm_new<command> (cmd)), (SCM) command_tag);
   return command_smob;
 }
 
@@ -663,7 +809,7 @@ mark_command (SCM command_smob) {
 static scm_sizet
 free_command (SCM command_smob) {
   command *ptr = (command *) SCM_CDR (command_smob);
-  delete ptr;
+  tm_delete (ptr);
   return 0;
 }
 
@@ -689,7 +835,7 @@ static long url_tag;
 #define SCM_URLP(u) \
   (SCM_NIMP (u) && (((long) SCM_CAR (u)) == url_tag))
 #define SCM_ASSERT_URL(u,arg,rout) \
-  SCM_ASSERT (scm_is_url (u) || SCM_MYSTRINGP (u), u, arg, rout)
+  SCM_ASSERT (scm_is_url (u) || scm_is_string (u), u, arg, rout)
 
 bool
 scm_is_url (SCM u) {
@@ -699,16 +845,18 @@ scm_is_url (SCM u) {
 SCM
 url_to_scm (url u) {
   SCM url_smob;
-  SCM_NEWCELL (url_smob);
-  SCM_SETCDR (url_smob, (SCM) ((void*) (new url (u))));
-  SCM_SETCAR (url_smob, url_tag);
+  SET_SMOB (url_smob, (void*) (tm_new<url> (u)), (SCM) url_tag);
   return url_smob;
 }
 
 url
 scm_to_url (SCM url_smob) {
   if (scm_is_string (url_smob))
+#ifdef __MINGW32__
+    return url_system(scm_to_string (url_smob));
+#else
     return scm_to_string (url_smob);
+#endif
   return *((url*) SCM_CDR (url_smob));
 }
 
@@ -721,7 +869,7 @@ mark_url (SCM url_smob) {
 static scm_sizet
 free_url (SCM url_smob) {
   url *ptr = (url *) SCM_CDR (url_smob);
-  delete ptr;
+  tm_delete (ptr);
   return 0;
 }
 
@@ -748,9 +896,61 @@ urlP (SCM t) {
 url url_concat (url u1, url u2) { return u1 * u2; }
 url url_or (url u1, url u2) { return u1 | u2; }
 void string_save (string s, url u) { (void) save_string (u, s); }
-string string_load (url u) { string s; (void) load_string (u, s); return s; }
+string string_load (url u) {
+  string s; (void) load_string (u, s, false); return s; }
 url url_ref (url u, int i) { return u[i]; }
 
+/******************************************************************************
+* Table types
+******************************************************************************/
+
+typedef hashmap<string,string> table_string_string;
+
+static bool
+scm_is_table_string_string (SCM p) {
+  if (scm_is_null (p)) return true;
+  else if (!scm_is_pair (p)) return false;
+  else {
+    SCM f= SCM_CAR (p);
+    return scm_is_pair (f) &&
+           scm_is_string (SCM_CAR (f)) &&
+           scm_is_string (SCM_CDR (f)) &&
+           scm_is_table_string_string (SCM_CDR (p));
+  }
+}
+
+#define SCM_ASSERT_TABLE_STRING_STRING(p,arg,rout) \
+  SCM_ASSERT (scm_is_table_string_string (p), p, arg, rout)
+
+SCM
+table_string_string_to_scm (hashmap<string,string> t) {
+  SCM p= SCM_NULL;
+  iterator<string> it= iterate (t);
+  while (it->busy ()) {
+    string s= it->next ();
+    SCM n= scm_cons (string_to_scm (s), string_to_scm (t[s]));
+    p= scm_cons (n, p);
+  }
+  return p;
+}
+
+hashmap<string,string>
+scm_to_table_string_string (SCM p) {
+  hashmap<string,string> t;
+  while (!scm_is_null (p)) {
+    SCM n= SCM_CAR (p);
+    t (scm_to_string (SCM_CAR (n)))= scm_to_string (SCM_CDR (n));
+    p= SCM_CDR (p);
+  }
+  return t;
+}
+
+#define scm_is_solution scm_is_table_string_string
+#define SCM_ASSERT_SOLUTION(p,arg,rout) \
+  SCM_ASSERT (scm_is_solution(p), p, arg, rout)
+#define solution_to_scm table_string_string_to_scm
+#define scm_to_solution scm_to_table_string_string
+
 /******************************************************************************
 * Several array types
 ******************************************************************************/
@@ -763,7 +963,9 @@ typedef array<widget> array_widget;
 static bool
 scm_is_array_int (SCM p) {
   if (scm_is_null (p)) return true;
-  else return SCM_INUMP (SCM_CAR (p)) && scm_is_array_int (SCM_CDR (p));
+  else return scm_is_pair (p) &&
+	      scm_is_int (SCM_CAR (p)) &&
+	      scm_is_array_int (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_INT(p,arg,rout) \
@@ -781,7 +983,7 @@ array_int_to_scm (array<int> a) {
 scm_to_array_int (SCM p) {
   array<int> a;
   while (!scm_is_null (p)) {
-    a << scm_to_int (SCM_CAR (p));
+    a << ((int) scm_to_int (SCM_CAR (p)));
     p= SCM_CDR (p);
   }
   return a;
@@ -790,7 +992,9 @@ scm_to_array_int (SCM p) {
 static bool
 scm_is_array_string (SCM p) {
   if (scm_is_null (p)) return true;
-  else return SCM_MYSTRINGP (SCM_CAR (p)) && scm_is_array_string (SCM_CDR (p));
+  else return scm_is_pair (p) && 
+	      scm_is_string (SCM_CAR (p)) &&
+	      scm_is_array_string (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_STRING(p,arg,rout) \
@@ -814,10 +1018,17 @@ scm_to_array_string (SCM p) {
   return a;
 }
 
+#define scm_is_property scm_is_array_string
+#define SCM_ASSERT_PROPERTY(p,arg,rout) SCM_ASSERT_ARRAY_STRING (p,arg,rout)
+#define property_to_scm array_string_to_scm
+#define scm_to_property scm_to_array_string
+
 static bool
 scm_is_array_tree (SCM p) {
   if (scm_is_null (p)) return true;
-  else return SCM_TREEP (SCM_CAR (p)) && scm_is_array_tree (SCM_CDR (p));
+  else return scm_is_pair (p) && 
+	      SCM_TREEP (SCM_CAR (p)) &&
+	      scm_is_array_tree (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_TREE(p,arg,rout) \
@@ -844,7 +1055,9 @@ scm_to_array_tree (SCM p) {
 static bool
 scm_is_array_widget (SCM p) {
   if (scm_is_null (p)) return true;
-  else return scm_is_widget (SCM_CAR (p)) && scm_is_array_widget (SCM_CDR (p));
+  else return scm_is_pair (p) &&
+	      scm_is_widget (SCM_CAR (p)) &&
+	      scm_is_array_widget (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_WIDGET(p,arg,rout) \
@@ -868,6 +1081,139 @@ scm_to_array_widget (SCM p) {
   return a;
 }
 
+static bool
+scm_is_properties (SCM p) {
+  if (scm_is_null (p)) return true;
+  else return scm_is_pair (p) &&
+	      scm_is_property (SCM_CAR (p)) &&
+	      scm_is_properties (SCM_CDR (p));
+}
+
+#define SCM_ASSERT_PROPERTIES(p,arg,rout) \
+  SCM_ASSERT (scm_is_properties (p), p, arg, rout)
+
+SCM
+properties_to_scm (array<property> a) {
+  int i, n= N(a);
+  SCM p= SCM_NULL;
+  for (i=n-1; i>=0; i--) p= scm_cons (property_to_scm (a[i]), p);
+  return p;
+}
+
+array<property>
+scm_to_properties (SCM p) {
+  array<property> a;
+  while (!scm_is_null (p)) {
+    a << scm_to_property (SCM_CAR (p));
+    p= SCM_CDR (p);
+  }
+  return a;
+}
+
+static bool
+scm_is_solutions (SCM p) {
+  if (scm_is_null (p)) return true;
+  else return scm_is_pair (p) &&
+	      scm_is_solution (SCM_CAR (p)) &&
+	      scm_is_solutions (SCM_CDR (p));
+}
+
+#define SCM_ASSERT_SOLUTIONS(p,arg,rout) \
+  SCM_ASSERT (scm_is_solutions (p), p, arg, rout)
+
+SCM
+solutions_to_scm (array<solution> a) {
+  int i, n= N(a);
+  SCM p= SCM_NULL;
+  for (i=n-1; i>=0; i--) p= scm_cons (solution_to_scm (a[i]), p);
+  return p;
+}
+
+array<solution>
+scm_to_solutions (SCM p) {
+  array<solution> a;
+  while (!scm_is_null (p)) {
+    a << scm_to_solution (SCM_CAR (p));
+    p= SCM_CDR (p);
+  }
+  return a;
+}
+
+/******************************************************************************
+* List types
+******************************************************************************/
+
+typedef list<string> list_string;
+
+bool
+scm_is_list_string (SCM p) {
+  if (scm_is_null (p)) return true;
+  else return scm_is_pair (p) &&
+	      scm_is_string (SCM_CAR (p)) &&
+	      scm_is_list_string (SCM_CDR (p));
+}
+
+#define SCM_ASSERT_LIST_STRING(p,arg,rout) \
+  SCM_ASSERT (scm_is_list_string (p), p, arg, rout)
+
+SCM
+list_string_to_scm (list_string l) {
+  if (is_nil (l)) return SCM_NULL;
+  return scm_cons (string_to_scm (l->item),
+		   list_string_to_scm (l->next));
+}
+
+list_string
+scm_to_list_string (SCM p) {
+  if (scm_is_null (p)) return list_string ();
+  return list_string (scm_to_string (SCM_CAR (p)),
+		      scm_to_list_string (SCM_CDR (p)));
+}
+
+typedef list<tree> list_tree;
+
+bool
+scm_is_list_tree (SCM p) {
+  if (scm_is_null (p)) return true;
+  else return scm_is_pair (p) &&
+	      scm_is_tree (SCM_CAR (p)) &&
+	      scm_is_list_tree (SCM_CDR (p));
+}
+
+#define SCM_ASSERT_LIST_TREE(p,arg,rout) \
+  SCM_ASSERT (scm_is_list_tree (p), p, arg, rout)
+
+SCM
+list_tree_to_scm (list_tree l) {
+  if (is_nil (l)) return SCM_NULL;
+  return scm_cons (tree_to_scm (l->item),
+		   list_tree_to_scm (l->next));
+}
+
+list_tree
+scm_to_list_tree (SCM p) {
+  if (scm_is_null (p)) return list_tree ();
+  return list_tree (scm_to_tree (SCM_CAR (p)),
+		    scm_to_list_tree (SCM_CDR (p)));
+}
+
+/******************************************************************************
+* Other wrapper types
+******************************************************************************/
+
+#define SCM_ASSERT_COLLECTION(p,arg,rout) \
+  SCM_ASSERT (scm_is_array_string (p), p, arg, rout)
+
+SCM
+collection_to_scm (collection ss) {
+  return array_string_to_scm (as_strings (ss));
+}
+
+collection
+scm_to_collection (SCM p) {
+  return as_collection (scm_to_array_string (p));
+}
+
 /******************************************************************************
 * Initialization
 ******************************************************************************/
@@ -876,43 +1222,41 @@ scm_to_array_widget (SCM p) {
 
 void
 initialize_glue () {
-  tree_tag= scm_make_smob_type ("tree", 0);
+  tree_tag= scm_make_smob_type (const_cast<char*> ("tree"), 0);
   scm_set_smob_mark (tree_tag, mark_tree);
   scm_set_smob_free (tree_tag, free_tree);
   scm_set_smob_print (tree_tag, print_tree);
   scm_set_smob_equalp (tree_tag, cmp_tree);
-  observer_tag= scm_make_smob_type ("observer", 0);
+  observer_tag= scm_make_smob_type (const_cast<char*> ("observer"), 0);
   scm_set_smob_mark (observer_tag, mark_observer);
   scm_set_smob_free (observer_tag, free_observer);
   scm_set_smob_print (observer_tag, print_observer);
   scm_set_smob_equalp (observer_tag, cmp_observer);
-  display_tag= scm_make_smob_type ("display", 0);
-  scm_set_smob_mark (display_tag, mark_display);
-  scm_set_smob_free (display_tag, free_display);
-  scm_set_smob_print (display_tag, print_display);
-  scm_set_smob_equalp (display_tag, cmp_display);
-  widget_tag= scm_make_smob_type ("widget", 0);
+  widget_tag= scm_make_smob_type (const_cast<char*> ("widget"), 0);
   scm_set_smob_mark (widget_tag, mark_widget);
   scm_set_smob_free (widget_tag, free_widget);
   scm_set_smob_print (widget_tag, print_widget);
   scm_set_smob_equalp (widget_tag, cmp_widget);
-  make_widget_tag= scm_make_smob_type ("make-widget", 0);
-  scm_set_smob_mark (make_widget_tag, mark_make_widget);
-  scm_set_smob_free (make_widget_tag, free_make_widget);
-  scm_set_smob_print (make_widget_tag, print_make_widget);
-  scm_set_smob_equalp (make_widget_tag, cmp_make_widget);
-  command_tag= scm_make_smob_type ("command", 0);
+  promise_widget_tag=
+    scm_make_smob_type (const_cast<char*> ("promise-widget"), 0);
+  scm_set_smob_mark (promise_widget_tag, mark_promise_widget);
+  scm_set_smob_free (promise_widget_tag, free_promise_widget);
+  scm_set_smob_print (promise_widget_tag, print_promise_widget);
+  scm_set_smob_equalp (promise_widget_tag, cmp_promise_widget);
+  command_tag= scm_make_smob_type (const_cast<char*> ("command"), 0);
   scm_set_smob_mark (command_tag, mark_command);
   scm_set_smob_free (command_tag, free_command);
   scm_set_smob_print (command_tag, print_command);
   scm_set_smob_equalp (command_tag, cmp_command);
-  url_tag= scm_make_smob_type ("url", 0);
+  url_tag= scm_make_smob_type (const_cast<char*> ("url"), 0);
   scm_set_smob_mark (url_tag, mark_url);
   scm_set_smob_free (url_tag, free_url);
   scm_set_smob_print (url_tag, print_url);
   scm_set_smob_equalp (url_tag, cmp_url);
-  gh_new_procedure ("tree?", (FN) treeP, 1, 0, 0);
-  gh_new_procedure ("url?", (FN) urlP, 1, 0, 0);
+  scm_new_procedure ("tree?", (FN) treeP, 1, 0, 0);
+  scm_new_procedure ("tm?", (FN) contentP, 1, 0, 0);
+  scm_new_procedure ("observer?", (FN) observerP, 1, 0, 0);
+  scm_new_procedure ("url?", (FN) urlP, 1, 0, 0);
   initialize_glue_basic ();
   initialize_glue_editor ();
   initialize_glue_server ();
@@ -928,16 +1272,13 @@ scm_smobfuns observer_smob_funcs = {
   mark_observer, free_observer, print_observer, cmp_observer
 };
 
-scm_smobfuns display_smob_funcs = {
-  mark_display, free_display, print_display, cmp_display
-};
-
 scm_smobfuns widget_smob_funcs = {
   mark_widget, free_widget, print_widget, cmp_widget
 };
 
-scm_smobfuns make_widget_smob_funcs = {
-  mark_make_widget, free_make_widget, print_make_widget, cmp_make_widget
+scm_smobfuns promise_widget_smob_funcs = {
+  mark_promise_widget, free_promise_widget,
+  print_promise_widget, cmp_promise_widget
 };
 
 scm_smobfuns command_smob_funcs = {
@@ -952,13 +1293,14 @@ void
 initialize_glue () {
   tree_tag= scm_newsmob (&tree_smob_funcs);
   observer_tag= scm_newsmob (&observer_smob_funcs);
-  display_tag= scm_newsmob (&display_smob_funcs);
   widget_tag= scm_newsmob (&widget_smob_funcs);
-  make_widget_tag= scm_newsmob (&make_widget_smob_funcs);
+  promise_widget_tag= scm_newsmob (&promise_widget_smob_funcs);
   command_tag= scm_newsmob (&command_smob_funcs);
   url_tag= scm_newsmob (&url_smob_funcs);
-  gh_new_procedure ("tree?", (FN) treeP, 1, 0, 0);
-  gh_new_procedure ("url?", (FN) urlP, 1, 0, 0);
+  scm_new_procedure ("tree?", (FN) treeP, 1, 0, 0);
+  scm_new_procedure ("tm?", (FN) contentP, 1, 0, 0);
+  scm_new_procedure ("observer?", (FN) observerP, 1, 0, 0);
+  scm_new_procedure ("url?", (FN) urlP, 1, 0, 0);
   initialize_glue_basic ();
   initialize_glue_editor ();
   initialize_glue_server ();
