@@ -4,14 +4,18 @@
 * DESCRIPTION: Execution of scheme commands via guile
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
-#include "guile.hpp"
+#ifdef __MINGW32__
+//FIXME: if this include is not here we have compilation problems on mingw32 
+//       (probably name clashes with Windows headers)
+#include "tree.hpp"
+#endif
 #include "Glue/glue.hpp"
+#include "guile.hpp"
 #include "Scheme/evaluate.hpp"
 #include "file.hpp"
 #include "timer.hpp"
@@ -108,7 +112,7 @@ eval_scheme_file (string file) {
   if (DEBUG_STD) cout << "TeXmacs] Evaluating " << file << "...\n";
   char* _file= as_charp (file);
   SCM result= TeXmacs_eval_file (_file);
-  delete[] _file;
+  tm_delete_array (_file);
   //int extra= tm->watch (); cumul += extra;
   //cout << extra << "\t" << cumul << "\t" << file << "\n";
   return result;
@@ -137,7 +141,7 @@ eval_scheme (string s) {
   // cout << "Eval] " << s << "\n";
   char* _s= as_charp (s);
   SCM result= TeXmacs_eval_string (_s);
-  delete[] _s;
+  tm_delete_array (_s);
   return result;
 }
 
@@ -204,6 +208,12 @@ call_scheme (SCM fun, SCM a1, SCM a2, SCM a3) {
   return TeXmacs_call_scm (&args);
 }
 
+SCM
+call_scheme (SCM fun, SCM a1, SCM a2, SCM a3, SCM a4) {
+  SCM a[]= { fun, a1, a2, a3, a4 }; arg_list args= { 4, a };
+  return TeXmacs_call_scm (&args);
+}
+
 SCM
 call_scheme (SCM fun, array<SCM> a) {
   const int n= N(a);
