@@ -5,16 +5,55 @@
 *              should be move to this file
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef GUILE_HH
 #define GUILE_HH
 #include "tm_configure.hpp"
+#ifdef __MINGW32__
+// we redefine some symbols to avoid name clashes with Windows headers (included by Guile)
+#define PATTERN WIN_PATTERN
+#define STRING WIN_STRING
+#define GROUP WIN_GROUP
+#ifdef IN
+#define MY_IN IN
+#undef IN
+#endif
+#ifdef OUT
+#define MY_OUT OUT
+#undef OUT
+#endif
+#ifdef MENU_EVENT
+#define MY_MENU_EVENT MENU_EVENT
+#undef MENU_EVENT
+#endif
 #include <guile/gh.h>
+#undef STRING
+#undef ERROR
+#undef PATTERN
+#undef GROUP
+#undef IN
+#undef OUT
+#undef MENU_EVENT
+#ifdef MY_MENU_EVENT
+#define MENU_EVENT MY_MENU_EVENT
+#undef MY_MENU_EVENT
+#endif
+#ifdef MY_IN
+#define IN MY_IN
+#undef MY_IN
+#endif
+#ifdef MY_OUT
+#define OUT MY_OUT
+#undef MY_OUT
+#endif
+#else
+#include <guile/gh.h>
+#endif
+
 
 #define SCM_NULL gh_list (SCM_UNDEFINED)
 
