@@ -4,25 +4,16 @@
 * DESCRIPTION: see basic.cpp
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef BASIC_H
 #define BASIC_H
-#include "tm_configure.hpp"
-#include <stdlib.h>
+#include "fast_alloc.hpp"
 #include <math.h>
 
-// g++ >= 3.2 requires
-#include <iostream>
-using std::ostream;
-using std::cout;
-using std::cerr;
-// instead of include <iostream.h>
-
 #ifdef HAVE_INTPTR_T
 #ifdef OS_SUN
 #include <inttypes.h>
@@ -39,7 +30,7 @@ typedef long intptr_t;
 #define LESSGTR <>
 #endif
 
-#define DEBUG(x)
+#define TM_DEBUG(x)
 typedef int SI;
 typedef unsigned int SN;
 typedef short HI;
@@ -53,32 +44,17 @@ typedef long long int DI;
 #endif
 typedef void* pointer;
 
-#ifndef NO_FAST_ALLOC
-#ifdef OS_IRIX
-void* operator new (register size_t s) throw(std::bad_alloc);
-void  operator delete (register void* ptr) throw();
-void* operator new[] (register size_t s) throw(std::bad_alloc);
-void  operator delete[] (register void* ptr) throw();
-#else
-void* operator new (register size_t s);
-void  operator delete (register void* ptr);
-void* operator new[] (register size_t s);
-void  operator delete[] (register void* ptr);
-#endif
-#endif // not defined NO_FAST_ALLOC
-int   mem_used ();
-void  mem_info ();
-
 #if (defined OS_WIN32 || defined __SUNPRO_CC)
-#define STACK_NEW_ARRAY(name,T,size) T* name= new T[size]
-#define STACK_DELETE_ARRAY(name) delete[] name
+#define STACK_NEW_ARRAY(name,T,size) T* name= tm_new_array<T> (size)
+#define STACK_DELETE_ARRAY(name) tm_delete_array (name)
 #else
 #define STACK_NEW_ARRAY(name,T,size) T name[size]
 #define STACK_DELETE_ARRAY(name)
 #endif
 
 enum { DEBUG_FLAG_AUTO, DEBUG_FLAG_VERBOSE, DEBUG_FLAG_EVENTS,
-       DEBUG_FLAG_STD, DEBUG_FLAG_IO, DEBUG_FLAG_BENCH };
+       DEBUG_FLAG_STD, DEBUG_FLAG_IO, DEBUG_FLAG_BENCH,
+       DEBUG_FLAG_HISTORY, DEBUG_FLAG_QT };
 bool debug (int which, bool write_flag= false);
 int  debug_off ();
 void debug_on (int status);
@@ -88,6 +64,16 @@ void debug_on (int status);
 #define DEBUG_STD (debug (DEBUG_FLAG_STD))
 #define DEBUG_IO (debug (DEBUG_FLAG_IO))
 #define DEBUG_BENCH (debug (DEBUG_FLAG_BENCH))
+#define DEBUG_HISTORY (debug (DEBUG_FLAG_HISTORY))
+#define DEBUG_QT (debug (DEBUG_FLAG_QT))
+
+#ifdef DEBUG_ASSERT
+#include <assert.h>
+#define ASSERT(cond,msg) assert (cond); // msg
+#else
+#define ASSERT(cond,msg)
+#endif
+#define FAILED(msg) ASSERT(false,msg)
 
 inline SI min (SI i, SI j) { if (i<j) return i; else return j; }
 inline SI max (SI i, SI j) { if (i>j) return i; else return j; }
@@ -105,15 +91,15 @@ ostream& operator << (ostream& out, display_control ctrl);
 extern int concrete_count;
 struct concrete_struct {
   int ref_count;
-  inline concrete_struct (): ref_count (1) { DEBUG(concrete_count++); }
-  virtual inline ~concrete_struct () { DEBUG(concrete_count--); }
+  inline concrete_struct (): ref_count (1) { TM_DEBUG(concrete_count++); }
+  virtual inline ~concrete_struct () { TM_DEBUG(concrete_count--); }
 };
 
 extern int abstract_count;
 struct abstract_struct {
   int ref_count;
-  inline abstract_struct (): ref_count (0) { DEBUG(abstract_count++); }
-  virtual inline ~abstract_struct () { DEBUG(abstract_count--); }
+  inline abstract_struct (): ref_count (0) { TM_DEBUG(abstract_count++); }
+  virtual inline ~abstract_struct () { TM_DEBUG(abstract_count--); }
 };
 
 template<typename T>
@@ -131,9 +117,9 @@ template<typename T> T   type_helper<T>::init= T ();
 ******************************************************************************/
 
 #define INC_COUNT(R)      { (R)->ref_count++; }
-#define DEC_COUNT(R)      { if(0==--((R)->ref_count)) delete (R); }
+#define DEC_COUNT(R)      { if(0==--((R)->ref_count)) tm_delete (R); }
 #define INC_COUNT_NULL(R) { if ((R)!=NULL) (R)->ref_count++; }
-#define DEC_COUNT_NULL(R) { if ((R)!=NULL && 0==--((R)->ref_count)) delete (R); }
+#define DEC_COUNT_NULL(R) { if ((R)!=NULL && 0==--((R)->ref_count)) tm_delete (R); }
 
 // concrete
 #define CONCRETE(PTR)               \
