@@ -4,10 +4,9 @@
 * DESCRIPTION: Aqua dialogues widgets classes
 * COPYRIGHT  : (C) 2007  Massimiliano Gubinelli
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "mac_cocoa.h" 
@@ -23,37 +22,43 @@
 #include "url.hpp"
 #include "analyze.hpp"
 
+#define TYPE_CHECK(b) ASSERT (b, "type mismatch")
+#define NOT_IMPLEMENTED \
+  { if (DEBUG_EVENTS) cout << "STILL NOT IMPLEMENTED\n";  }
+
 #pragma mark aqua_chooser_widget_rep
 
 class aqua_chooser_widget_rep: public aqua_widget_rep {
 protected:	
-	command cmd;
-	string type;
-	string mgn;
-	string win_title;
-	coord2 position;
-	coord2 size;
-	
-	string file;
+  command cmd;
+  string type;
+  string mgn;
+  string win_title;
+  string directory;
+  coord2 position;
+  coord2 size;
+  string file;
 	
 public:
   aqua_chooser_widget_rep (command, string, string);
   ~aqua_chooser_widget_rep ();
 	
-	virtual void send (slot s, blackbox val);
+  virtual void send (slot s, blackbox val);
   virtual blackbox query (slot s, int type_id);
   virtual widget read (slot s, blackbox index);
   virtual void write (slot s, blackbox index, widget w);
   virtual void notify (slot s, blackbox new_val);
-	//  virtual void connect (slot s, widget w2, slot s2);
-	//  virtual void deconnect (slot s, widget w2, slot s2);
-	virtual widget plain_window_widget (string s);
+  //  virtual void connect (slot s, widget w2, slot s2);
+  //  virtual void deconnect (slot s, widget w2, slot s2);
+  virtual widget plain_window_widget (string s);
 
-	void perform_dialog();
+  void perform_dialog();
 };
 
 aqua_chooser_widget_rep::aqua_chooser_widget_rep (command _cmd, string _type, string _mgn) 
-: aqua_widget_rep(), cmd(_cmd), type(_type), mgn(_mgn), size(coord2(100,100)), position(coord2(0,0)), file("")
+: aqua_widget_rep(), cmd(_cmd), type(_type), 
+  mgn(_mgn), position (coord2 (0, 0)), 
+  size (coord2 (100, 100)), file ("")
 {
 }
 
@@ -64,52 +69,59 @@ aqua_chooser_widget_rep::~aqua_chooser_widget_rep()  {  }
 void
 aqua_chooser_widget_rep::send (slot s, blackbox val) {
   switch (s) {
-		case SLOT_VISIBILITY:
-		{	
-			check_type<bool> (val, "SLOT_VISIBILITY");
-			bool flag = open_box<bool> (val);
-		}	
-			break;
-		case SLOT_SIZE:
-		{
-			if (type_box (val) != type_helper<coord2>::id)
-				fatal_error ("type mismatch", "SLOT_SIZE");
-			size = open_box<coord2> (val);
-		}
-			break;
-		case SLOT_POSITION:
-		{
-			if (type_box (val) != type_helper<coord2>::id)
-				fatal_error ("type mismatch", "SLOT_POSITION");
-			position = open_box<coord2> (val);
-		}
-			break;
-		case SLOT_KEYBOARD_FOCUS:
-		{
-			if (type_box (val) != type_helper<bool>::id)
-				fatal_error ("type mismatch", "SLOT_KEYBOARD_FOCUS");
-			perform_dialog();
-		}
-			break;
-			
-		case SLOT_STRING_INPUT:
-	//		send_string (THIS, "input", val);
-			break;
-		case SLOT_INPUT_TYPE:
-		//	send_string (THIS, "type", val);
-			break;
-		case SLOT_INPUT_PROPOSAL:
-			//send_string (THIS, "default", val);
-			break;
-		case SLOT_FILE:
-			//send_string (THIS, "file", val);
-			break;
-		case SLOT_DIRECTORY:
-			//send_string (THIS, "directory", val);
-			break;
-
-		default:
-			aqua_widget_rep::send(s,val);
+  case SLOT_VISIBILITY:
+    {	
+      check_type<bool> (val, "SLOT_VISIBILITY");
+      bool flag = open_box<bool> (val);
+      (void) flag;
+      NOT_IMPLEMENTED
+	}	
+    break;
+  case SLOT_SIZE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      size = open_box<coord2> (val);
+    }
+    break;
+  case SLOT_POSITION:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      position = open_box<coord2> (val);
+    }
+    break;
+  case SLOT_KEYBOARD_FOCUS:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<bool>::id);
+      perform_dialog();
+    }
+    break;
+    
+  case SLOT_STRING_INPUT:
+    //		send_string (THIS, "input", val);
+    NOT_IMPLEMENTED 
+      break;
+  case SLOT_INPUT_TYPE:
+    TYPE_CHECK (type_box (val) == type_helper<string>::id);
+    type = open_box<string> (val);        
+    //	send_string (THIS, "type", val);
+    break;
+#if 0
+  case SLOT_INPUT_PROPOSAL:
+    //send_string (THIS, "default", val);
+    break;
+#endif
+  case SLOT_FILE:
+    //send_string (THIS, "file", val);
+    NOT_IMPLEMENTED
+      break;
+  case SLOT_DIRECTORY:
+    TYPE_CHECK (type_box (val) == type_helper<string>::id);
+    directory = open_box<string> (val);
+    directory = as_string (url_pwd () * url_system (directory));
+    break;
+    
+  default:
+    aqua_widget_rep::send(s,val);
   }
 }
 
@@ -117,30 +129,27 @@ aqua_chooser_widget_rep::send (slot s, blackbox val) {
 blackbox
 aqua_chooser_widget_rep::query (slot s, int type_id) {
   switch (s) {
-		case SLOT_POSITION:  
-		{
-			typedef pair<SI,SI> coord2;
-			if (type_id != type_helper<coord2>::id)
-				fatal_error ("type mismatch (SLOT_POSITION)", "aqua_chooser_widget_rep::query");
-			return close_box<coord2> (position);
-		}
-		case SLOT_SIZE:
-		{
-			typedef pair<SI,SI> coord2;
-			if (type_id != type_helper<coord2>::id)
-				fatal_error ("type mismatch (SLOT_SIZE)", "aqua_chooser_widget_rep::query");
-			return close_box<coord2> (size);
-		}
+  case SLOT_POSITION:  
+    {
+      typedef pair<SI,SI> coord2;
+      TYPE_CHECK (type_id == type_helper<coord2>::id);
+      return close_box<coord2> (position);
+    }
+  case SLOT_SIZE:
+    {
+      typedef pair<SI,SI> coord2;
+      TYPE_CHECK (type_id == type_helper<coord2>::id);
+      return close_box<coord2> (size);
+    }
 			
-		case SLOT_STRING_INPUT:
-		{
-			if (type_id != type_helper<string>::id)
-				fatal_error ("type mismatch (SLOT_STRING_INPUT)", "aqua_chooser_widget_rep::query");
-			return close_box<string> (file);
-		}
-
-		default:
-			return aqua_widget_rep::query(s,type_id);
+  case SLOT_STRING_INPUT:
+    {
+      TYPE_CHECK (type_id == type_helper<string>::id);
+      return close_box<string> (file);
+    }
+    
+  default:
+    return aqua_widget_rep::query(s,type_id);
   }
 }
 
@@ -148,7 +157,7 @@ aqua_chooser_widget_rep::query (slot s, int type_id) {
 void
 aqua_chooser_widget_rep::notify (slot s, blackbox new_val) {
   switch (s) {
-			default: ;
+  default: ;
   }
   widget_rep::notify (s, new_val);
 }
@@ -156,36 +165,36 @@ aqua_chooser_widget_rep::notify (slot s, blackbox new_val) {
 widget
 aqua_chooser_widget_rep::read (slot s, blackbox index) {
   switch (s) {
-		case SLOT_WINDOW:
-			check_type_void (index, "SLOT_WINDOW");
-			return this;
-		case SLOT_FORM_FIELD:
-			check_type<int> (index, "SLOT_FORM_FIELD");
-			return this;
-		case SLOT_FILE:
-			check_type_void (index, "SLOT_FILE");
-			return this;
-		case SLOT_DIRECTORY:
-			check_type_void (index, "SLOT_DIRECTORY");
-			return this;
-		default:
-			return aqua_widget_rep::read(s,index);
+  case SLOT_WINDOW:
+    check_type_void (index, "SLOT_WINDOW");
+    return this;
+  case SLOT_FORM_FIELD:
+    check_type<int> (index, "SLOT_FORM_FIELD");
+    return this;
+  case SLOT_FILE:
+    check_type_void (index, "SLOT_FILE");
+    return this;
+  case SLOT_DIRECTORY:
+    check_type_void (index, "SLOT_DIRECTORY");
+    return this;
+  default:
+    return aqua_widget_rep::read(s,index);
   }
 }
 
 void
 aqua_chooser_widget_rep::write (slot s, blackbox index, widget w) {
   switch (s) {
-		default:
-			aqua_widget_rep::write(s,index,w);
+  default:
+    aqua_widget_rep::write(s,index,w);
   }
 }
 
 
 widget aqua_chooser_widget_rep::plain_window_widget (string s)
 {
-	win_title = s;
-	return this;
+  win_title = s;
+  return this;
 }
 
 
@@ -195,33 +204,47 @@ widget file_chooser_widget (command cmd, string type, string mgn)
 // the widget includes a previsualizer and a default magnification
 // for importation can be specified
 {
-	return new aqua_chooser_widget_rep (cmd,type,mgn);
+  return tm_new <aqua_chooser_widget_rep> (cmd,type,mgn);
 }
 
+
+@interface TMSavePanel : NSSavePanel
+{
+}
+- (BOOL)_overwriteExistingFileCheck:(NSString *)filename;
+@end
+
+@implementation TMSavePanel
+- (BOOL)_overwriteExistingFileCheck:(NSString *)filename
+{
+  return YES;
+}
+@end
+
 #if 0
 void aqua_chooser_widget_rep::perform_dialog()
 {
-	int result;
-	NSArray *fileTypes = [NSArray arrayWithObject:@"tm"];
-	NSOpenPanel *oPanel = [NSOpenPanel openPanel];
-    [oPanel setTitle:to_nsstring(win_title)];
-	[oPanel setAllowsMultipleSelection:YES];
-	result = [oPanel runModalForDirectory:NSHomeDirectory()
-																	 file:nil types:fileTypes];
-	if (result == NSOKButton) {
-		NSArray *filesToOpen = [oPanel filenames];
-		int i, count = [filesToOpen count];
-		for (i=0; i<count; i++) {
-			NSString *aFile = [filesToOpen objectAtIndex:i];
-//			id currentDoc = [[ToDoDoc alloc] initWithFile:aFile];
-		}
-		if (count > 0) {
-			file = from_nsstring([filesToOpen objectAtIndex:0]);
-			url u= url_system (scm_unquote (file));
+  int result;
+  NSArray *fileTypes = [NSArray arrayWithObject:@"tm"];
+  NSOpenPanel *oPanel = [NSOpenPanel openPanel];
+  [oPanel setTitle:to_nsstring(win_title)];
+  [oPanel setAllowsMultipleSelection:YES];
+  result = [oPanel runModalForDirectory:NSHomeDirectory()
+		   file:nil types:fileTypes];
+  if (result == NSOKButton) {
+    NSArray *filesToOpen = [oPanel filenames];
+    int i, count = [filesToOpen count];
+    for (i=0; i<count; i++) {
+      NSString *aFile = [filesToOpen objectAtIndex:i];
+      //			id currentDoc = [[ToDoDoc alloc] initWithFile:aFile];
+    }
+    if (count > 0) {
+      file = from_nsstring([filesToOpen objectAtIndex:0]);
+      url u= url_system (scm_unquote (file));
       file = "(url-system " * scm_quote (as_string (u)) * ")";
-
-		}
-	}
+      
+    }
+  }
   cmd();	
 }
 #else
@@ -229,21 +252,35 @@ void aqua_chooser_widget_rep::perform_dialog()
 {
   int result;
   NSArray *fileTypes = [NSArray arrayWithObject:@"tm"];
-  NSSavePanel *oPanel = [NSSavePanel savePanel];
+  NSSavePanel *oPanel = [TMSavePanel savePanel];
   [oPanel setTitle:to_nsstring(win_title)];
-//  [oPanel setMessage:@"Choose a file."];
+  //  [oPanel setMessage:@"Choose a file."];
   [oPanel setNameFieldLabel:@"File:"];
   [oPanel setPrompt:@"Choose"];
   [oPanel setAllowedFileTypes:fileTypes];
- // [oPanel setAllowsMultipleSelection:YES];
-  result = [oPanel runModalForDirectory:NSHomeDirectory()
-                                   file:nil ];
+  // [oPanel setAllowsMultipleSelection:YES];
+  NSPoint pos = to_nspoint(position);
+  NSRect r = NSMakeRect(0,0,0,0);
+  r.size = [oPanel frame].size;
+  NSOffsetRect(r, pos.x - r.size.width/2, pos.y - r.size.height/2);
+  [oPanel setFrameOrigin:r.origin];
+  
+  result = [oPanel runModalForDirectory:to_nsstring(directory)
+		   file:nil ];
   if (result == NSOKButton) {
-      file = from_nsstring([oPanel filename]);
-      url u= url_system (scm_unquote (file));
+    file = from_nsstring([oPanel filename]);
+    url u= url_system (scm_unquote (file));
+    if (type == "image")
+      file = "(list (url-system " *
+	scm_quote (as_string (u)) *
+	") \"100\" \"100\" \"0\" \"0\" \"10\" \"10\")";
+    //FIXME: fake image dimensions
+    else
       file = "(url-system " * scm_quote (as_string (u)) * ")";
+  } else {
+    file = "#f";
   }
-  cmd();	
+  cmd ();	
 }
 
 #endif
@@ -266,11 +303,11 @@ public:
   virtual widget read (slot s, blackbox index);
   virtual void write (slot s, blackbox index, widget w);
   virtual void notify (slot s, blackbox new_val);
-	//  virtual void connect (slot s, widget w2, slot s2);
-	//  virtual void deconnect (slot s, widget w2, slot s2);
-	virtual widget plain_window_widget (string s);
-
-	void perform_dialog();
+  //  virtual void connect (slot s, widget w2, slot s2);
+  //  virtual void deconnect (slot s, widget w2, slot s2);
+  virtual widget plain_window_widget (string s);
+  
+  void perform_dialog();
 };
 
 class aqua_field_widget_rep : public widget_rep {
@@ -278,12 +315,12 @@ class aqua_field_widget_rep : public widget_rep {
   string input;
   string type;
   array<string> proposals;
-   aqua_input_widget_rep *parent;
-public:
-  aqua_field_widget_rep(aqua_input_widget_rep *_parent) : widget_rep(), prompt(""), input(""),  proposals(), parent(_parent) {};
+  aqua_input_widget_rep *parent;
+ public:
+ aqua_field_widget_rep(aqua_input_widget_rep *_parent) : widget_rep(), prompt(""), input(""),  proposals(), parent(_parent) {};
   virtual void send (slot s, blackbox val);
   virtual blackbox query (slot s, int type_id);
-
+  
   friend class aqua_input_widget_rep;
 };
 
@@ -291,51 +328,47 @@ public:
 void
 aqua_field_widget_rep::send (slot s, blackbox val) {
   switch (s) {
-		case SLOT_STRING_INPUT:
-	  {
-		  if (type_box (val) != type_helper<string>::id)
-			  fatal_error ("type mismatch", "SLOT_STRING_INPUT");
-		  input =  open_box<string> (val);
-	  }
-	//		send_string (THIS, "input", val);
-			break;
-		case SLOT_INPUT_TYPE:
-	  {
-		  if (type_box (val) != type_helper<string>::id)
-			  fatal_error ("type mismatch", "SLOT_INPUT_TYPE");
-		  type =  open_box<string> (val);
-	  }
-		  break;
-		case SLOT_INPUT_PROPOSAL:
-	  {
-		  if (type_box (val) != type_helper<string>::id)
-			  fatal_error ("type mismatch", "SLOT_INPUT_PROPOSAL");
-		  proposals <<  open_box<string> (val);
-	  }
-			//send_string (THIS, "default", val);
-			break;
-		case SLOT_KEYBOARD_FOCUS:
-      {
-	     parent->send(s,val);
-	  }
-	      break;
-		default:
-			widget_rep::send(s,val);
+  case SLOT_STRING_INPUT:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<string>::id);
+      input =  open_box<string> (val);
+    }
+    //		send_string (THIS, "input", val);
+    break;
+  case SLOT_INPUT_TYPE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<string>::id);
+      type =  open_box<string> (val);
+    }
+    break;
+  case SLOT_INPUT_PROPOSAL:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<string>::id);
+      proposals <<  open_box<string> (val);
+    }
+    //send_string (THIS, "default", val);
+    break;
+  case SLOT_KEYBOARD_FOCUS:
+    {
+      parent->send(s,val);
+    }
+    break;
+  default:
+    widget_rep::send(s,val);
   }
 }
 
 blackbox
 aqua_field_widget_rep::query (slot s, int type_id) {
   switch (s) {
-		case SLOT_STRING_INPUT:
-		{
-			if (type_id != type_helper<string>::id)
-				fatal_error ("type mismatch (SLOT_STRING_INPUT)", "aqua_field_widget_rep::query");
-			return close_box<string> (input);
-		}
-
-		default:
-			return widget_rep::query(s,type_id);
+  case SLOT_STRING_INPUT:
+    {
+      TYPE_CHECK (type_id == type_helper<string>::id);
+      return close_box<string> (input);
+    }
+    
+  default:
+    return widget_rep::query(s,type_id);
   }
 }
 
@@ -349,11 +382,11 @@ ABSTRACT_NULL_CODE(aqua_field_widget);
 
 
 aqua_input_widget_rep::aqua_input_widget_rep (command _cmd, array<string> _prompts) 
-: aqua_widget_rep(), cmd(_cmd),  size(coord2(100,100)), position(coord2(0,0)), win_title(""), fields(N(_prompts))
+: aqua_widget_rep(), cmd(_cmd), fields(N(_prompts)), size(coord2(100,100)), position(coord2(0,0)), win_title("") 
 {
   for(int i=0; i < N(_prompts); i++) {
-   fields[i] = new aqua_field_widget_rep(this);
-   fields[i]->prompt = _prompts[i];
+    fields[i] = tm_new <aqua_field_widget_rep> (this);
+    fields[i]->prompt = _prompts[i];
   }
 }
 
@@ -364,37 +397,36 @@ aqua_input_widget_rep::~aqua_input_widget_rep()  {  }
 void
 aqua_input_widget_rep::send (slot s, blackbox val) {
   switch (s) {
-		case SLOT_VISIBILITY:
-		{	
-			check_type<bool> (val, "SLOT_VISIBILITY");
-			bool flag = open_box<bool> (val);
-		}	
-			break;
-		case SLOT_SIZE:
-		{
-			if (type_box (val) != type_helper<coord2>::id)
-				fatal_error ("type mismatch", "SLOT_SIZE");
-			size = open_box<coord2> (val);
-		}
-			break;
-		case SLOT_POSITION:
-		{
-			if (type_box (val) != type_helper<coord2>::id)
-				fatal_error ("type mismatch", "SLOT_POSITION");
-			position = open_box<coord2> (val);
-		}
-			break;
-		case SLOT_KEYBOARD_FOCUS:
-		{
-			if (type_box (val) != type_helper<bool>::id)
-				fatal_error ("type mismatch", "SLOT_KEYBOARD_FOCUS");
-			perform_dialog();
-		}
-			break;
-			
-
-		default:
-			aqua_widget_rep::send(s,val);
+  case SLOT_VISIBILITY:
+    {	
+      check_type<bool> (val, "SLOT_VISIBILITY");
+      bool flag = open_box<bool> (val);
+      (void) flag;
+      NOT_IMPLEMENTED 
+	}	
+    break;
+  case SLOT_SIZE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      size = open_box<coord2> (val);
+    }
+    break;
+  case SLOT_POSITION:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      position = open_box<coord2> (val);
+    }
+    break;
+  case SLOT_KEYBOARD_FOCUS:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<bool>::id);
+      perform_dialog();
+    }
+    break;
+    
+    
+  default:
+    aqua_widget_rep::send(s,val);
   }
 }
 
@@ -402,26 +434,24 @@ aqua_input_widget_rep::send (slot s, blackbox val) {
 blackbox
 aqua_input_widget_rep::query (slot s, int type_id) {
   switch (s) {
-		case SLOT_POSITION:  
-		{
-			typedef pair<SI,SI> coord2;
-			if (type_id != type_helper<coord2>::id)
-				fatal_error ("type mismatch (SLOT_POSITION)", "aqua_input_widget_rep::query");
-			return close_box<coord2> (position);
-		}
-		case SLOT_SIZE:
-		{
-			typedef pair<SI,SI> coord2;
-			if (type_id != type_helper<coord2>::id)
-				fatal_error ("type mismatch (SLOT_SIZE)", "aqua_input_widget_rep::query");
-			return close_box<coord2> (size);
-		}
-      case SLOT_STRING_INPUT:
-        return fields[0]->query(s,type_id);
-			
-
-		default:
-			return aqua_widget_rep::query(s,type_id);
+  case SLOT_POSITION:  
+    {
+      typedef pair<SI,SI> coord2;
+      TYPE_CHECK (type_id == type_helper<coord2>::id);
+      return close_box<coord2> (position);
+    }
+  case SLOT_SIZE:
+    {
+      typedef pair<SI,SI> coord2;
+      TYPE_CHECK (type_id == type_helper<coord2>::id);
+      return close_box<coord2> (size);
+    }
+  case SLOT_STRING_INPUT:
+    return fields[0]->query(s,type_id);
+    
+    
+  default:
+    return aqua_widget_rep::query(s,type_id);
   }
 }
 
@@ -429,7 +459,7 @@ aqua_input_widget_rep::query (slot s, int type_id) {
 void
 aqua_input_widget_rep::notify (slot s, blackbox new_val) {
   switch (s) {
-			default: ;
+  default: ;
   }
   widget_rep::notify (s, new_val);
 }
@@ -437,30 +467,30 @@ aqua_input_widget_rep::notify (slot s, blackbox new_val) {
 widget
 aqua_input_widget_rep::read (slot s, blackbox index) {
   switch (s) {
-		case SLOT_WINDOW:
-			check_type_void (index, "SLOT_WINDOW");
-			return this;
-		case SLOT_FORM_FIELD:
-			check_type<int> (index, "SLOT_FORM_FIELD");
-			return (widget_rep*)(fields[open_box<int>(index)].rep);
-		default:
-			return aqua_widget_rep::read(s,index);
+  case SLOT_WINDOW:
+    check_type_void (index, "SLOT_WINDOW");
+    return this;
+  case SLOT_FORM_FIELD:
+    check_type<int> (index, "SLOT_FORM_FIELD");
+    return (widget_rep*)(fields[open_box<int>(index)].rep);
+  default:
+    return aqua_widget_rep::read(s,index);
   }
 }
 
 void
 aqua_input_widget_rep::write (slot s, blackbox index, widget w) {
   switch (s) {
-		default:
-			aqua_widget_rep::write(s,index,w);
+  default:
+    aqua_widget_rep::write(s,index,w);
   }
 }
 
 
 widget aqua_input_widget_rep::plain_window_widget (string s)
 {
-	win_title = s;
-	return this;
+  win_title = s;
+  return this;
 }
 
 
@@ -477,18 +507,59 @@ widget aqua_input_widget_rep::plain_window_widget (string s)
 @implementation TMInputHelper
 - (id) init
 {
-	self = [super init];
-	if (self != nil) {
-		[NSBundle loadNibNamed:@"InputPanel" owner:self];
-      [dialog setReleasedWhenClosed:NO];
-      wid = NULL;
-	}
-	return self;
+  self = [super init];
+  if (self != nil) {
+    NSRect panelRect = NSMakeRect(0, 0, 480, 360);
+    dialog = [[NSWindow alloc] initWithContentRect:panelRect
+                                        styleMask:NSTitledWindowMask | NSClosableWindowMask | NSResizableWindowMask
+                                          backing:NSBackingStoreBuffered defer:YES];
+    form = [[[NSMatrix alloc] initWithFrame: NSMakeRect(20, 60, 440, 280) ] autorelease];
+    NSButton* cancelButton = [[[NSButton alloc] initWithFrame: NSMakeRect(274, 12, 96, 32) ] autorelease];
+    NSButton* okButton = [[[NSButton alloc] initWithFrame: NSMakeRect(370, 12, 96, 32) ] autorelease];
+    [cancelButton setTitle:@"Cancel"];
+    [okButton setTitle:@"Ok"];
+
+    [okButton setButtonType:   NSMomentaryPushInButton];
+    [cancelButton setButtonType:   NSMomentaryPushInButton];
+
+    [okButton setBezelStyle: NSRoundedBezelStyle];
+    [cancelButton setBezelStyle: NSRoundedBezelStyle];
+    
+    [cancelButton setTag: 1];
+    [okButton setTag: 0];
+    [okButton setTarget: self];
+    [okButton setAction: @selector(doForm:)];
+    [cancelButton setTarget: self];
+    [cancelButton setAction: @selector(doForm:)];
+
+    [okButton setKeyEquivalent:@"\r"];
+    [cancelButton setKeyEquivalent:@"\E"];
+ 
+    [form   setAutoresizingMask: NSViewHeightSizable | NSViewWidthSizable];
+    [okButton setAutoresizingMask: NSViewMinXMargin | NSViewMaxYMargin];
+    [cancelButton setAutoresizingMask: NSViewMinXMargin | NSViewMaxYMargin];
+    
+    [[dialog contentView] addSubview: form];
+    [[dialog contentView] addSubview: cancelButton];
+    [[dialog contentView] addSubview: okButton];
+    
+    [dialog makeFirstResponder: form];
+    [form setNextKeyView: cancelButton];
+    [cancelButton setNextKeyView: okButton];
+    [okButton setNextKeyView: form];
+    
+    [form retain];   
+
+    [dialog setReleasedWhenClosed:NO];
+    wid = NULL;
+  }
+  return self;
 }
 - (void) dealloc
 {
-	[dialog release];
-	[super dealloc];
+  [dialog release];
+  [form release];
+  [super dealloc];
 }
 
 - (IBAction) doForm:(id)sender
@@ -496,7 +567,7 @@ widget aqua_input_widget_rep::plain_window_widget (string s)
   if ([sender tag] == 0)
   {
     [NSApp stopModalWithCode:0]; // OK button
-   }
+  }
   else
   {
     [NSApp stopModalWithCode:1]; // Cancel button
@@ -570,13 +641,16 @@ void aqua_input_widget_rep::perform_dialog()
       NSString *ans = [(NSComboBoxCell*)[form cellAtRow:i column:1] stringValue];
       fields[i]->input = scm_quote(from_nsstring(ans));
     }
-    cmd();
   }
   else  { // Cancel button
+    for(int i=0; i<N(fields); i++) {
+      fields[i]->input = "#f";
+    }
   }
  
   
   [ih release];
+  cmd();
 }
 
 
@@ -586,7 +660,7 @@ widget inputs_list_widget (command call_back, array<string> prompts)
 // a dialogue widget with Ok and Cancel buttons and a series of textual
 // input widgets with specified prompts
 {
-	return new aqua_input_widget_rep (call_back,prompts);
+	return tm_new <aqua_input_widget_rep> (call_back,prompts);
 
 }
 
@@ -596,7 +670,7 @@ widget input_text_widget (command call_back, string type, array<string> def)
 // a textual input widget for input of a given type and a list of suggested
 // default inputs (the first one should be displayed, if there is one)
 {
-  return new aqua_input_text_widget_rep(call_back, type, def);
+  return tm_new <aqua_input_text_widget_rep> (call_back, type, def);
 }
 
 
