@@ -4,10 +4,9 @@
 * DESCRIPTION: interface with ghostscript
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Ghostscript/ghostscript.hpp"
@@ -73,6 +72,14 @@ ghostscript_run (Display* dpy, Window gs_win, Pixmap pm,
   int y1= by1 + as_int (cy1 * (by2 - by1));
   int x2= bx1 + as_int (cx2 * (bx2 - bx1));
   int y2= by1 + as_int (cy2 * (by2 - by1));
+  // FIXME: this dirty hack is needed as a correction for the rendering
+  // of patterns with a non-entire width or height. Without the hack,
+  // white lines may appear at the borders of the patterns.
+  // However the hack also causes normal images to be rendered
+  // slightly incorrectly
+  if (x1+1 < x2) x1++;
+  if (y1+1 < y2) y1++;
+  // End dirty hack
 
   if (ghostscript_bugged ()) {
     int scr  = DefaultScreen (dpy);
@@ -100,7 +107,7 @@ ghostscript_run (Display* dpy, Window gs_win, Pixmap pm,
   unsigned char* _data= (unsigned char*) as_charp (data);
   int _n= N(data);
   XChangeProperty (dpy, gs_win, gh, st, 8, PropModeReplace, _data, _n);
-  delete[] _data;
+  tm_delete_array (_data);
   XSync(dpy, false);
 #endif
 
