@@ -4,10 +4,9 @@
 * DESCRIPTION: TeX text fonts
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "font.hpp"
@@ -713,33 +712,33 @@ font
 tex_font (string family, int size, int dpi, int dsize) {
   string name= "tex:" * family * as_string (size) * "@" * as_string(dpi);
   return make (font, name,
-    new tex_font_rep (name, TEX_ANY, family, size, dpi, dsize));
+    tm_new<tex_font_rep> (name, TEX_ANY, family, size, dpi, dsize));
 }
 
 font
 tex_cm_font (string family, int size, int dpi, int dsize) {
   string name= "cm:" * family * as_string (size) * "@" * as_string(dpi);
   return make (font, name,
-    new tex_font_rep (name, TEX_CM, family, size, dpi, dsize));
+    tm_new<tex_font_rep> (name, TEX_CM, family, size, dpi, dsize));
 }
 
 font
 tex_ec_font (string family, int size, int dpi, int dsize) {
   string name= "ec:" * family * as_string (size) * "@" * as_string(dpi);
   return make (font, name,
-    new tex_font_rep (name, TEX_EC, family, size, dpi, dsize));
+    tm_new<tex_font_rep> (name, TEX_EC, family, size, dpi, dsize));
 }
 
 font
 tex_la_font (string family, int size, int dpi, int dsize) {
   string name= "la:" * family * as_string (size) * "@" * as_string(dpi);
   return make (font, name,
-    new tex_font_rep (name, TEX_LA, family, size, dpi, dsize));
+    tm_new<tex_font_rep> (name, TEX_LA, family, size, dpi, dsize));
 }
 
 font
 tex_adobe_font (string family, int size, int dpi, int dsize) {
   string name= "adobe:" * family * as_string (size) * "@" * as_string(dpi);
   return make (font, name,
-    new tex_font_rep (name, TEX_ADOBE, family, size, dpi, dsize));
+    tm_new<tex_font_rep> (name, TEX_ADOBE, family, size, dpi, dsize));
 }
