@@ -4,10 +4,9 @@
 * DESCRIPTION: file handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "file.hpp"
@@ -33,6 +32,10 @@
 #endif
 #include <sys/types.h>
 
+#ifdef MACOSX_EXTENSIONS
+#include "MacOS/mac_images.h"
+#endif
+
 /******************************************************************************
 * New style loading and saving
 ******************************************************************************/
@@ -42,10 +45,11 @@ load_string (url u, string& s, bool fatal) {
   // cout << "Load " << u << LF;
   url r= u;
   if (!is_rooted_name (r)) r= resolve (r);
+  // cout << "Resolved " << r << LF;
   bool err= !is_rooted_name (r);
   if (!err) {
     string name= concretize (r);
-
+    // cout << "Concrete :" << name << LF;
     // File contents in cache?
     bool file_flag= do_cache_file (name);
     bool doc_flag= do_cache_doc (name);
@@ -59,10 +63,15 @@ load_string (url u, string& s, bool fatal) {
 
     bench_start ("load file");
     char* _name= as_charp (name);
+    // cout << "OPEN :" << _name << LF;
 #ifdef OS_WIN32
     FILE* fin= _fopen (_name, "rb");
 #else
-    FILE* fin= fopen (_name, "r");
+#ifdef __MINGW32__
+    FILE* fin= fopen (_name, "rb");
+#else
+	FILE* fin= fopen (_name, "r");
+#endif
 #endif
     if (fin == NULL) err= true;
     int size= 0;
@@ -80,7 +89,7 @@ load_string (url u, string& s, bool fatal) {
       if (read < size) s->resize (read);
       fclose (fin);
     }
-    delete[] _name;
+    tm_delete_array (_name);
     bench_cumul ("load file");
 
     // Cache file contents
@@ -89,8 +98,10 @@ load_string (url u, string& s, bool fatal) {
 	cache_set (cache_type, name, s);
     // End caching
   }
-  if (err && fatal)
-    fatal_error (as_string (u) * " not readable", "load_string");
+  if (err && fatal) {
+    cerr << "File name= " << as_string (u) << "\n";
+    FAILED ("file not readable");
+  }
   return err;
 }
 
@@ -98,8 +109,10 @@ bool
 save_string (url u, string s, bool fatal) {
   if (is_rooted_tmfs (u)) {
     bool err= save_to_server (u, s);
-    if (err && fatal)
-      fatal_error (as_string (u) * " not writeable", "save_string");
+    if (err && fatal) {
+      cerr << "File name= " << as_string (u) << "\n";
+      FAILED ("file not writeable");
+    }
     return err;
   }
 
@@ -122,7 +135,7 @@ save_string (url u, string s, bool fatal) {
 	fputc (s[i], fout);
       fclose (fout);
     }
-    delete[] _name;
+    tm_delete_array (_name);
 
     // Cache file contents
     bool file_flag= do_cache_file (name);
@@ -135,8 +148,10 @@ save_string (url u, string s, bool fatal) {
     // End caching
   }
 
-  if (err && fatal)
-    fatal_error (as_string (u) * " not writeable", "save_string");
+  if (err && fatal) {
+    cerr << "File name= " << as_string (u) << "\n";
+    FAILED ("file not writeable");
+  }
   return err;
 }
 
@@ -174,7 +189,7 @@ get_attributes (url name, struct stat* buf,
   (void) link_flag;
   // FIXME: configure should test whether lstat works
   // flag= (link_flag? lstat (temp, buf): stat (temp, buf));
-  delete[] temp;
+  tm_delete_array (temp);
   bench_cumul ("stat");
 
   // Cache stat results
@@ -257,19 +272,35 @@ is_of_type (url name, string filter) {
       if (!S_ISDIR (buf.st_mode)) return false;
       break;
     case 'l':
+#ifdef __MINGW32__
+      return false;
+#else
       if (!S_ISLNK (buf.st_mode)) return false;
+#endif
       break;
     case 'r':
+#ifndef __MINGW32__
       if ((buf.st_mode & (S_IRUSR | S_IRGRP | S_IROTH)) == 0) return false;
+#else
+      if ((buf.st_mode & 292) == 0) return false;
+#endif
       break;
     case 'w':
+#ifndef __MINGW32__
       if ((buf.st_mode & (S_IWUSR | S_IWGRP | S_IWOTH)) == 0) return false;
+#else
+      if ((buf.st_mode & 146) == 0) return false;
+#endif
       break;
     case 'x':
 #ifdef OS_WIN32
       if (suffix(name) == "bat") break;
 #endif
+#ifndef __MINGW32__
       if ((buf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) == 0) return false;
+#else
+      if ((buf.st_mode & 73) == 0) return false;
+#endif
       break;
     }
   return true;
@@ -301,13 +332,16 @@ is_newer (url which, url than) {
 
 url
 url_temp (string suffix) {
+#ifdef __MINGW32__
+  int rnd= raw_time ();
+#else
   static bool initialized= false;
   if (!initialized) {
-    srandom ((int) texmacs_time ());
+    srandom ((int) raw_time ());
     initialized= true;
   }
-
   int rnd= random ();
+#endif
   string name= "tmp_" * as_string (rnd) * suffix;
   url u ("$TEXMACS_HOME_PATH/system/tmp", name);
   if (exists (u)) return url_temp (suffix);
@@ -351,7 +385,7 @@ cache_dir_set (string dir, array<string> a) {
 
 array<string>
 read_directory (url u, bool& error_flag) {
-  // cout << "Directory " << u << LF;
+ //  cout << "Directory " << u << LF;
   u= resolve (u, "dr");
   if (is_none (u)) return array<string> ();
   string name= concretize (u);
@@ -365,7 +399,7 @@ read_directory (url u, bool& error_flag) {
   DIR* dp;
   char* temp= as_charp (name);
   dp= opendir (temp);
-  delete[] temp;
+  tm_delete_array (temp);
   error_flag= (dp==NULL);
   if (error_flag) return array<string> ();
 
@@ -466,9 +500,76 @@ search_score (url u, array<string> a) {
 }
 
 /******************************************************************************
-* Miscellaneous
+* Commands on files
 ******************************************************************************/
 
+void
+move (url u1, url u2) {
+  char *_u1, *_u2;
+  _u1 = as_charp (concretize (u1));
+  _u2 = as_charp (concretize (u2));
+  (void) rename (_u1, _u2);
+  tm_delete_array (_u1);
+  tm_delete_array (_u2);
+}
+
+void
+copy (url u1, url u2) {
+  string s;
+  if (!load_string (u1, s, false))
+    (void) save_string (u2, s, false);
+}
+
+void
+remove (url u) {
+  u= expand (complete (u));
+  if (is_none (u));
+  else if (is_or (u)) {
+    remove (u[1]);
+    remove (u[2]);
+  }
+  else {
+    char *_u= as_charp (concretize (u));
+    (void) ::remove (_u);
+    tm_delete_array (_u);
+  }
+}
+
+void
+mkdir (url u) {
+#if defined (HAVE_SYS_TYPES_H) && defined (HAVE_SYS_STAT_H)
+  if (exists (u)) return;
+  char *_u= as_charp (concretize (u));
+#if defined(__MINGW__) || defined(__MINGW32__)
+  (void) ::mkdir (_u);
+#else
+  (void) ::mkdir (_u, S_IRWXU + S_IRGRP + S_IROTH);
+#endif
+  tm_delete_array (_u);
+#else
+#if defined(__MINGW__) || defined(__MINGW32__)
+  system ("mkdir", u);
+#else
+  system ("mkdir -p", u);
+#endif
+#endif
+}
+
+void
+change_mode (url u, int mode) {
+#if defined (HAVE_SYS_TYPES_H) && defined (HAVE_SYS_STAT_H)
+  char *_u= as_charp (concretize (u));
+  (void) ::chmod (_u, mode);
+  tm_delete_array (_u);
+#else
+  string m0= as_string ((mode >> 9) & 7);
+  string m1= as_string ((mode >> 6) & 7);
+  string m2= as_string ((mode >> 3) & 7);
+  string m3= as_string (mode & 7);
+  system ("chmod -f " * m0 * m1 * m2 * m3, u);
+#endif
+}
+
 void
 ps2pdf (url u1, url u2) {
 #ifdef OS_WIN32
@@ -476,9 +577,65 @@ ps2pdf (url u1, url u2) {
   _u1 = as_charp (concretize (u1));
   _u2 = as_charp (concretize (u2));
   XPs2Pdf (_u1, _u2);
-  delete [] _u1;
-  delete [] _u2;
+  tm_delete_array (_u1);
+  tm_delete_array (_u2);
+#else
+#ifdef MACOSX_EXTENSIONS
+  mac_ps_to_pdf (u1, u2);
 #else
   system ("ps2pdf", u1, u2);
 #endif
+#endif
 }
+
+
+/******************************************************************************
+ * Tab-completion for file names
+ ******************************************************************************/
+
+#ifdef OS_WIN32
+#define URL_CONCATER  '\\'
+#else
+#define URL_CONCATER  '/'
+#endif
+
+static void
+file_completions_file (array<string>& a, url search, url u) {
+  if (is_or (u)) {
+    file_completions_file (a, search, u[1]);
+    file_completions_file (a, search, u[2]);
+  }
+  else {
+    url v= delta (search * url ("dummy"), u);
+    if (is_none (v)) return;
+    string s= as_string (v);
+    if (is_directory (u)) s= s * string (URL_CONCATER);
+    a << s;
+  }
+}
+
+static void
+file_completions_dir (array<string>& a, url search, url dir) {
+  if (is_or (search)) {
+    file_completions_dir (a, search[1], dir);
+    file_completions_dir (a, search[2], dir);
+  }
+  else if (is_or (dir)) {
+    file_completions_dir (a, search, dir[1]);
+    file_completions_dir (a, search, dir[2]);
+  }
+  else {
+    url u= search * dir * url_wildcard ("*");
+    u= complete (u, "r");
+    u= expand (u);
+    file_completions_file (a, search, u);
+  }
+}
+
+array<string>
+file_completions (url search, url dir) {
+  array<string> a;
+  file_completions_dir (a, search, dir);
+  return a;
+}
+
