@@ -4,10 +4,9 @@
 * DESCRIPTION: image file handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "file.hpp"
@@ -19,6 +18,11 @@
 #include "scheme.hpp"
 #include "../Plugins/Imlib2/imlib2.hpp"
 
+#ifdef QTTEXMACS
+bool qt_supports_image (url u);
+void qt_image_size (url image, int& w, int& h);
+#endif
+
 #ifdef OS_WIN32
 #include <x11/xlib.h>
 #endif
@@ -32,8 +36,8 @@ hashmap<tree,string> ps_bbox ("");
 tree
 xpm_load (url u) {
   string s;
-  load_string ("$TEXMACS_PIXMAPS_PATH" * u, s);
-  if (s == "") load_string ("$TEXMACS_PATH/misc/pixmaps/TeXmacs.xpm", s);
+  load_string ("$TEXMACS_PIXMAP_PATH" * u, s, false);
+  if (s == "") load_string ("$TEXMACS_PATH/misc/pixmaps/TeXmacs.xpm", s, true);
 
   int i, j;
   tree t (TUPLE);
@@ -64,7 +68,10 @@ xpm_size (url u, int& w, int& h) {
   ok= read_int (s, i, w);
   skip_spaces (s, i);
   ok= read_int (s, i, h) && ok;
-  if (!ok) fatal_error ("Invalid xpm (" * file_name * ")", "xpm_size");
+  if (!ok) {
+    cerr << "File name= " << file_name << "\n";
+    FAILED ("invalid xpm");
+  }
 }
 
 array<string>
@@ -80,8 +87,7 @@ xpm_colors (tree t) {
   ok= read_int (s, i, c) && ok;
   skip_spaces (s, i);
   ok= read_int (s, i, b) && ok;
-  if ((!ok) || (N(t)<(c+1)) || (c<=0))
-    fatal_error ("Invalid xpm tree", "x_drawable_rep::xpm_colors");
+  ASSERT (ok && N(t)>c && c>0, "invalid xpm tree");
 
   for (k=0; k<c; k++) {
     string s   = as_string (t[k+1]);
@@ -120,8 +126,7 @@ xpm_hotspot (tree t) {
   ok= read_int (s, i, c) && ok;
   skip_spaces (s, i);
   ok= read_int (s, i, b) && ok;
-  if ((!ok) || (N(t)<(c+1)) || (c<=0))
-    fatal_error ("Invalid xpm tree", "x_drawable_rep::xpm_hotspot");
+  ASSERT (ok && N(t)>c && c>0, "invalid xpm tree");
 
   skip_spaces (s, i);
   ok= read_int (s, i, x) && ok;
@@ -149,7 +154,7 @@ ps_load (url image) {
 #endif
 
   string s, suf= suffix (name);
-  if (suf == "ps" || suf == "eps") load_string (name, s);
+  if (suf == "ps" || suf == "eps") load_string (name, s, false);
   else s= as_string (call ("image->postscript", object (name)));
 
 #ifdef OS_WIN32
@@ -157,7 +162,7 @@ ps_load (url image) {
     char *data;
     char *path= as_charp (as_string (name));
     data= XLoadImageAsPS (path);
-    delete[] path;
+    tm_delete_array (path);
     if (!data) s= "";
     else {
       s= string (data);
@@ -166,7 +171,7 @@ ps_load (url image) {
   }
 #endif
 
-  if (s == "") load_string ("$TEXMACS_PATH/misc/pixmaps/unknown.ps", s);
+  if (s == "") load_string ("$TEXMACS_PATH/misc/pixmaps/unknown.ps", s, true);
   return s;
 }
 
@@ -214,6 +219,12 @@ ps_bounding_box (url image, int& x1, int& y1, int& x2, int& y2) {
 
 void
 image_size (url image, int& w, int& h) {
+#ifdef QTTEXMACS
+  if (qt_supports_image (image)) {
+    qt_image_size (image, w, h);
+    return;
+  }
+#endif
   if (imlib2_supports (image))
     imlib2_image_size (image, w, h);
   else {
