@@ -5,10 +5,9 @@
 * COPYRIGHT  : (C) 2003  Joris van der Hoeven
 * THANKS     : Beej's Guide to Network Programming has been helpful
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "socket_link.hpp"
@@ -19,6 +18,7 @@
 #include "Scheme/object.hpp"
 #include <stdio.h>
 #include <string.h>
+#ifndef __MINGW32__
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/wait.h>
@@ -26,6 +26,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#endif
 #include <errno.h>
 #ifdef OS_WIN32
 #include <sys/misc.h>
@@ -33,6 +34,13 @@
 
 hashset<pointer> socket_link_set;
 
+
+static void
+socket_callback (void *obj, void* info) {
+  socket_link_rep* con= (socket_link_rep*) obj;
+  if (con->alive) con->feed (LINK_OUT);
+}
+
 /******************************************************************************
 * Constructors and destructors for socket_links
 ******************************************************************************/
@@ -55,7 +63,7 @@ socket_link_rep::~socket_link_rep () {
 
 tm_link
 make_socket_link (string host, int port, int type, int fd) {
-  return new socket_link_rep (host, port, type, fd);
+  return tm_new<socket_link_rep> (host, port, type, fd);
 }
 
 tm_link
@@ -74,6 +82,7 @@ find_socket_link (int fd) {
 
 string
 socket_link_rep::start () {
+#ifndef __MINGW32__
   if (alive) return "busy";
   if (DEBUG_AUTO)
     cout << "TeXmacs] Connecting to '" << host << ":" << port << "'\n";
@@ -85,7 +94,7 @@ socket_link_rep::start () {
 #else
   struct hostent *hp = gethostbyname (_host);
 #endif
-  delete[] _host;
+  tm_delete_array (_host);
   if (hp == NULL) return "Error: no connection for '" * host * "'";
 
   // creating socket
@@ -122,8 +131,12 @@ socket_link_rep::start () {
   if (fcntl (io, F_SETFL, flags) < 0)
 #endif
     return "Error: non working connection to '" * where * "'";
-  alive= true;
+  alive = true;
+  sn = socket_notifier (io, &socket_callback, this, NULL);  
   return "ok";
+#else
+  return "Error: sockets not implemented";
+#endif
 }
 
 static string
@@ -143,6 +156,7 @@ debug_io_string (string s) {
 
 static int
 send_all (int s, char *buf, int *len) {
+#ifndef __MINGW32__
   int total= 0;          // how many bytes we've sent
   int bytes_left= *len;  // how many we have left to send
   int n= 0;
@@ -156,6 +170,9 @@ send_all (int s, char *buf, int *len) {
 
   *len= total;
   return n==-1? -1: 0;
+#else
+  return 0;
+#endif
 } 
 
 
@@ -172,6 +189,7 @@ socket_link_rep::write (string s, int channel) {
 
 void
 socket_link_rep::feed (int channel) {
+#ifndef __MINGW32__
   if ((!alive) || (channel != LINK_OUT)) return;
   char tempout[1024];
   int r= recv (io, tempout, 1024, 0);
@@ -184,6 +202,9 @@ socket_link_rep::feed (int channel) {
     if (DEBUG_IO) cout << debug_io_string (string (tempout, r));
     outbuf << string (tempout, r);
   }
+
+  if (!is_nil(feed_cmd)) (feed_cmd)->apply();
+#endif
 }
 
 string&
@@ -205,6 +226,7 @@ socket_link_rep::read (int channel) {
 
 void
 socket_link_rep::listen (int msecs) {
+#ifndef __MINGW32__
   if (!alive) return;
   fd_set rfds;
   FD_ZERO (&rfds);
@@ -214,6 +236,7 @@ socket_link_rep::listen (int msecs) {
   tv.tv_usec = 1000 * (msecs % 1000);
   int nr= select (io+1, &rfds, NULL, NULL, &tv);
   if (nr != 0 && FD_ISSET (io, &rfds)) feed (LINK_OUT);
+#endif
 }
 
 void
@@ -222,13 +245,16 @@ socket_link_rep::interrupt () {
 
 void
 socket_link_rep::stop () {
+#ifndef __MINGW32__
   if (!alive) return;
   if (type == SOCKET_SERVER) call ("server-remove", object (io));
   else if (type == SOCKET_CLIENT) call ("client-remove");
   close (io);
   io= -1;
   alive= false;
+  sn = socket_notifier ();
   wait (NULL);
+#endif
 }
 
 /******************************************************************************
@@ -237,6 +263,7 @@ socket_link_rep::stop () {
 
 void
 listen_to_sockets () {
+#ifndef __MINGW32__
   while (true) {
     fd_set rfds;
     FD_ZERO (&rfds);
@@ -263,4 +290,5 @@ listen_to_sockets () {
       if (con->alive && FD_ISSET (con->io, &rfds)) con->feed (LINK_OUT);
     }
   }
+#endif
 }
