@@ -4,10 +4,9 @@
 * DESCRIPTION: file handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "sys_utils.hpp"
@@ -23,17 +22,48 @@ int script_status = 1;
 * System functions
 ******************************************************************************/
 
+#if defined(__MINGW__) || defined(__MINGW32__) || defined(OS_WIN32)
+namespace {
+  namespace win32 {
+    #include <windows.h>
+
+    int
+    system (string s) {
+      if (starts(s, "convert ")) return 1;
+      STARTUPINFOW si;
+      PROCESS_INFORMATION pi;
+      
+      char* cs = as_charp(s);
+      WCHAR* wcs = tm_new_array<WCHAR>(N(s));
+      mbstowcs(wcs, cs, N(s)); 
+      ZeroMemory(&si, (sizeof (si)));
+      si.cb = (sizeof (si));
+      ZeroMemory( &pi, (sizeof (pi)));
+      int ret = CreateProcess(L"cmd.exe", wcs, 0, 0, FALSE, CREATE_NO_WINDOW, 0, 0, &si, &pi);
+      if (ret) {
+        WaitForSingleObject(pi.hProcess, INFINITE);
+        CloseHandle(pi.hProcess);
+        CloseHandle(pi.hThread);
+      }
+      tm_delete_array(wcs);
+      tm_delete_array(cs);
+      return ret;
+    }
+  }
+}
+#endif
+
 int
 system (string s) {
-  // cout << "System: " << s << "\n";
-  char* _s= as_charp (s);
-#ifdef OS_WIN32
-  int r= _system (_s);
+  if (DEBUG_STD) cerr << "TeXmacs] System: " << s << "\n";
+#if defined(__MINGW__) || defined(__MINGW32__) || defined(OS_WIN32)
+  return win32::system(s);
 #else
-  int r= system (_s);
+  char* cs = as_charp(s);
+  int ret = system(cs);
+  tm_delete_array(cs);
+  return ret;
 #endif
-  delete[] _s;
-  return r;
 }
 
 string
@@ -47,11 +77,7 @@ eval_system (string s) {
 #endif
   string result;
   bool flag= load_string (temp, result, false);
-#ifdef OS_WIN32
-  system ("rm \"" * temp_s * "\"");
-#else
-  system ("rm " * temp_s);
-#endif
+  remove (temp);
   if (flag) {
 #ifdef OS_WIN32
     cerr << "TeXmacs] failed: " << s * " > " * temp_s << "\n";
@@ -72,7 +98,7 @@ string
 get_env (string var) {
   char* _var= as_charp (var);
   char* _ret= getenv (_var);
-  delete[] _var;
+  tm_delete_array (_var);
   if (_ret==NULL) {
     if (var == "PWD") return get_env ("HOME");
     return "";
@@ -84,13 +110,16 @@ get_env (string var) {
 
 void
 set_env (string var, string with) {
-#ifdef STD_SETENV
+#if defined(STD_SETENV) && !defined(__MINGW32__)
   char* _var = as_charp (var);
   char* _with= as_charp (with);
   setenv (_var, _with, 1);
+  tm_delete_array(_var);
+  tm_delete_array(_with);
 #else
   char* _varw= as_charp (var * "=" * with);
   (void) putenv (_varw);
+  // do not delete _varw !!!
+  // -> known memory leak, but solution more complex than it is worth
 #endif
-  // do not delete _var and _with !!!
 }
