@@ -4,25 +4,31 @@
 * DESCRIPTION: main program
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
+#include <fstream>
+
 #include "boot.hpp"
 #include "file.hpp"
 #include "server.hpp"
 #include "timer.hpp"
 #include "data_cache.hpp"
-#ifdef EXPERIMENTAL
-#include "../../Style/Memorizer/clean_copy.hpp"
-#include "../../Style/Environment/environment.hpp"
-#endif
 #ifdef AQUATEXMACS
 void mac_fix_paths ();
 #endif
 
+#ifdef QTTEXMACS
+#include "Qt/qt_utilities.hpp"
+#include <QApplication>
+#endif
+
+#if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
+#include "MacOS/mac_app.h"
+#endif
+
 extern bool   char_clip;
 extern bool   reverse_colors;
 
@@ -36,6 +42,8 @@ extern int geometry_x, geometry_y;
 extern tree the_et;
 extern bool texmacs_started;
 
+std::ofstream log_ofstream;
+
 /******************************************************************************
 * For testing
 ******************************************************************************/
@@ -49,6 +57,56 @@ test_routines () {
 }
 #endif
 
+/******************************************************************************
+* Texmacs paths
+******************************************************************************/
+
+void
+TeXmacs_init_paths (int& argc, char** argv) {
+#ifdef QTTEXMACS
+  url exedir = url_system (qt_application_directory ());
+#else
+  url exedir = url_system(argv[0]) * ".." ;
+  if (! is_rooted(exedir)) {
+    exedir = url_pwd() * exedir ;
+  }
+#endif
+
+#if defined(AQUATEXMACS) ||(defined(QTTEXMACS) && defined(Q_WS_MAC)) || (defined(X11TEXMACS) && defined (MACOSX_EXTENSIONS))
+  // Mac bundle environment initialization
+  // We set some environment variables when the executable
+  // is in a .app bundle on MacOSX
+  if (get_env ("TEXMACS_PATH") == "")
+    set_env ("TEXMACS_PATH", as_string(exedir * "../Resources/share/TeXmacs"));
+  //cout << get_env("PATH") * ":" * as_string(url("$PWD") * argv[0]
+  // * "../../Resources/share/TeXmacs/bin") << LF;
+  set_env ("PATH", get_env("PATH") * ":" *
+	           as_string (exedir * "../Resources/share/TeXmacs/bin"));
+  //system("set");
+#endif
+
+#ifdef __MINGW32__
+  // Win bundle environment initialization
+  // TEXMACS_PATH is set by assuming that the executable is in TeXmacs/bin/
+  // HOME is set to USERPROFILE
+  // PWD is set to HOME
+  // if PWD is lacking, then the path resolution machinery may not work
+  
+  if (get_env ("TEXMACS_PATH") == "")
+    set_env ("TEXMACS_PATH", as_string (exedir * ".."));
+  if (get_env ("HOME") == "")
+    set_env ("HOME", get_env("USERPROFILE"));
+  // HACK
+  // In WINE the variable PWD is already in the outer Unix environment 
+  // so we need to override it to have a correct behaviour
+  if ((get_env ("PWD") == "") || (get_env ("PWD")[0] == '/'))  {
+    set_env ("PWD", as_string (exedir));
+    // set_env ("PWD", get_env("HOME"));
+  }
+  // system("set");
+#endif
+}
+
 /******************************************************************************
 * Real main program for encaptulation of guile
 ******************************************************************************/
@@ -72,15 +130,19 @@ TeXmacs_main (int argc, char** argv) {
       else if (s == "-debug-events") debug (DEBUG_FLAG_EVENTS, true);
       else if (s == "-debug-io") debug (DEBUG_FLAG_IO, true);
       else if (s == "-debug-bench") debug (DEBUG_FLAG_BENCH, true);
+      else if (s == "-debug-history") debug (DEBUG_FLAG_HISTORY, true);
+      else if (s == "-debug-qt") debug (DEBUG_FLAG_QT, true);
       else if (s == "-debug-all") {
 	debug (DEBUG_FLAG_EVENTS, true);
 	debug (DEBUG_FLAG_STD, true);
 	debug (DEBUG_FLAG_IO, true);
+	debug (DEBUG_FLAG_HISTORY, true);
 	debug (DEBUG_FLAG_BENCH, true);
+	debug (DEBUG_FLAG_QT, true);
       }
       else if ((s == "-fn") || (s == "-font")) {
 	i++;
-	if (i<argc) the_default_font= argv[i];	
+	if (i<argc) the_default_font= argv[i];
       }
       else if ((s == "-g") || (s == "-geometry")) {
 	i++;
@@ -100,7 +162,7 @@ TeXmacs_main (int argc, char** argv) {
 	    geometry_w= max (as_int (g (0, j1)), 320);
 	    geometry_h= max (as_int (g (j1+1, j2)), 200);
 	  }
-	  if (j3<N(g)) { 
+	  if (j3<N(g)) {
 	    if (g[j2] == '-') geometry_x= as_int (g (j2, j3)) - 1;
 	    else geometry_x= as_int (g (j2+1, j3));
 	    if (g[j3] == '-') geometry_y= as_int (g (j3, N(g))) - 1;
@@ -124,11 +186,11 @@ TeXmacs_main (int argc, char** argv) {
 	exit (0);
       }
       else if ((s == "-p") || (s == "-path")) {
-	system ("echo $TEXMACS_PATH");
+	cout << get_env ("TEXMACS_PATH") << "\n";
 	exit (0);
       }
       else if ((s == "-bp") || (s == "-binpath")) {
-	system ("echo $TEXMACS_BIN_PATH");
+	cout << get_env ("TEXMACS_BIN_PATH") << "\n";
 	exit (0);
       }
       else if ((s == "-q") || (s == "-quit"))
@@ -137,16 +199,20 @@ TeXmacs_main (int argc, char** argv) {
 	reverse_colors= true;
       else if ((s == "-c") || (s == "-convert")) {
 	i+=2;
-	if (i<argc)
+	if (i<argc) {
+	  url in  ("$PWD", argv[i-1]);
+	  url out ("$PWD", argv[ i ]);
 	  my_init_cmds= my_init_cmds * " " *
-	    "(texmacs-load-buffer " * scm_quote (argv[i-1]) *
+	    "(texmacs-load-buffer " * scm_quote (as_string (in)) *
 	    " \"generic\" 0 #f) " *
-	    "(export-buffer " * scm_quote (argv[i]) * ")";
+	    "(export-buffer " * scm_quote (as_string (out)) * ")";
+	}
       }
       else if ((s == "-x") || (s == "-execute")) {
 	i++;
 	if (i<argc) my_init_cmds= (my_init_cmds * " ") * argv[i];
       }
+      else if (s == "-log-file") i++;
       else if ((s == "-Oc") || (s == "-no-char-clipping")) char_clip= false;
       else if ((s == "+Oc") || (s == "-char-clipping")) char_clip= true;
       else if ((s == "-S") || (s == "-setup") ||
@@ -186,12 +252,19 @@ TeXmacs_main (int argc, char** argv) {
   init_plugins ();
   bench_cumul ("initialize plugins");
   if (DEBUG_STD) cout << "TeXmacs] Opening display...\n";
+  
+#if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
+  init_mac_application ();
+#endif
+    
   gui_open (argc, argv);
   set_default_font (the_default_font);
   if (DEBUG_STD) cout << "TeXmacs] Starting server...\n";
+  { // opening scope for server sv
   server sv;
 
   for (i=1; i<argc; i++) {
+    if (argv[i] == NULL) break;
     string s= argv[i];
     if ((N(s)>=2) && (s(0,2)=="--")) s= s (1, N(s));
     if ((s[0] != '-') && (s[0] != '+')) {
@@ -203,7 +276,8 @@ TeXmacs_main (int argc, char** argv) {
 	(s == "-fn") || (s == "-font") ||
 	(s == "-i") || (s == "-initialize") ||
 	(s == "-g") || (s == "-geometry") ||
-	(s == "-x") || (s == "-execute")) i++;
+	(s == "-x") || (s == "-execute") ||
+	(s == "-log-file")) i++;
   }
   if (install_status == 1) {
     if (DEBUG_STD) cout << "TeXmacs] Loading welcome message...\n";
@@ -229,8 +303,16 @@ TeXmacs_main (int argc, char** argv) {
   texmacs_started= true;
   gui_start_loop ();
 
+  if (DEBUG_STD) cout << "TeXmacs] Stopping server...\n";
+  } // ending scope for server sv
+
   if (DEBUG_STD) cout << "TeXmacs] Closing display...\n";
   gui_close ();
+  
+#if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
+  finalize_mac_application ();
+#endif
+  
   if (DEBUG_STD) cout << "TeXmacs] Good bye...\n";
 }
 
@@ -241,7 +323,11 @@ TeXmacs_main (int argc, char** argv) {
 void
 immediate_options (int argc, char** argv) {
   if (get_env ("TEXMACS_HOME_PATH") == "")
-    set_env ("TEXMACS_HOME_PATH", get_env ("HOME") * "/.TeXmacs");
+#ifdef __MINGW32__
+    set_env ("TEXMACS_HOME_PATH", get_env ("APPDATA") * "/TeXmacs");
+#else
+  set_env ("TEXMACS_HOME_PATH", get_env ("HOME") * "/.TeXmacs");
+#endif
   if (get_env ("TEXMACS_HOME_PATH") == "") return;
   for (int i=1; i<argc; i++) {
     string s= argv[i];
@@ -269,21 +355,34 @@ immediate_options (int argc, char** argv) {
       remove (url ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm"));
       remove (url ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm"));
     }
+#if defined(__MINGW__) || defined(__MINGW32__) || defined(OS_WIN32)
+    else if (s == "-log-file" && i+1 < argc) {
+      i++;
+      char* log_file = argv[i];
+      log_ofstream.open (log_file, std::ios_base::out | std::ios_base::trunc);
+      if (log_ofstream.rdstate() & std::ofstream::failbit)
+	cerr << "TeXmacs] Error: could not open " << log_file << "\n";
+      else {
+	cout.rdbuf (log_ofstream.rdbuf ());
+	cerr.rdbuf (log_ofstream.rdbuf ());
+      }
+    }
+#endif
   }
 }
 
 int
 main (int argc, char** argv) {
-#ifdef AQUATEXMACS
-  mac_fix_paths ();
+  immediate_options (argc, argv);
+  set_env ("LC_NUMERIC", "POSIX");
+#ifdef QTTEXMACS
+  // initialize the Qt application infrastructure
+  new QApplication (argc, argv);
 #endif
+  TeXmacs_init_paths (argc, argv);
   //cout << "Bench  ] Started TeXmacs\n";
   the_et     = tuple ();
   the_et->obs= ip_observer (path ());
-#ifdef EXPERIMENTAL
-  global_notify_assign (path (), tuple ());
-#endif
-  immediate_options (argc, argv);
   cache_initialize ();
   bench_start ("initialize texmacs");
   init_texmacs ();
