@@ -4,10 +4,9 @@
 * DESCRIPTION: Main TeXmacs windows
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "tm_window.hpp"
@@ -29,15 +28,13 @@ tm_window_rep::tm_window_rep (widget wid2, tree geom):
   win (texmacs_window_widget (wid2, geom)),
   wid (wid2), id (create_window_id ()),
   serial (tm_window_serial++),
-  texmacs_menu (NULL), texmacs_icon_menu (NULL),
+  menu_current (object ()), menu_cache (widget ()),
   text_ptr (NULL)
 {
   sfactor= get_server () -> get_default_shrinking_factor ();
 }
 
 tm_window_rep::~tm_window_rep () {
-  if (texmacs_menu != NULL) delete[] texmacs_menu;
-  if (texmacs_icon_menu != NULL) delete[] texmacs_icon_menu;
   destroy_window_id (id);
 }
 
@@ -86,27 +83,39 @@ tm_window_rep::unmap () {
 * Menus
 ******************************************************************************/
 
+bool
+tm_window_rep::get_menu_widget (int which, string menu, widget& w) {
+  object xmenu= call ("menu-expand", eval ("'" * menu));
+  //cout << "xmenu= " << xmenu << "\n";
+  if (menu_cache->contains (xmenu)) {
+    if (menu_current[which] == xmenu) return false;
+    menu_current (which)= xmenu;
+    //cout << "Cached " << menu << "\n";
+    w= menu_cache [xmenu];
+    return true;
+  }
+  //cout << "Compute " << menu << "\n";
+  object umenu= eval ("'" * menu);
+  w= make_menu_widget (umenu);
+  menu_cache (xmenu)= w;
+  return true;
+}
+
 void
 tm_window_rep::menu_main (string menu) {
-  if (texmacs_menu == NULL) texmacs_menu= new object[1];
-  object xmenu= call ("menu-expand", eval ("'" * menu));
-  if (xmenu == texmacs_menu[0]) return;
-  texmacs_menu[0]= xmenu;
-  widget w= make_menu_widget (xmenu);
-  ::set_main_menu (wid, w);
+  widget w;
+  if (get_menu_widget (-1, menu, w))
+    ::set_main_menu (wid, w);
 }
 
 void
 tm_window_rep::menu_icons (int which, string menu) {
-  if ((which<0) || (which>2)) return;
-  if (texmacs_icon_menu == NULL) texmacs_icon_menu= new object[3];
-  object xmenu= call ("menu-expand", eval ("'" * menu));
-  if (xmenu == texmacs_icon_menu[which]) return;
-  texmacs_icon_menu[which]= xmenu;
-  widget w= make_menu_widget (xmenu);
-  if      (which == 0) set_main_icons (wid, w);
-  else if (which == 1) set_context_icons (wid, w);
-  else if (which == 2) set_user_icons (wid, w);
+  widget w;
+  if (get_menu_widget (which, menu, w)) {
+    if      (which == 0) set_main_icons (wid, w);
+    else if (which == 1) set_context_icons (wid, w);
+    else if (which == 2) set_user_icons (wid, w);
+  }
 }
 
 void
@@ -233,7 +242,7 @@ tm_window_rep::interactive (string name, string type, array<string> def,
   text_ptr = &s;
   call_back= cmd;
   widget tw = text_widget (name, black, false, "english");
-  widget inp= input_text_widget (new ia_command_rep (this), type, def);
+  widget inp= input_text_widget (tm_new<ia_command_rep> (this), type, def);
   set_interactive_prompt (wid, tw);
   set_interactive_input (wid, inp);
   set_interactive_mode (true);
