@@ -4,10 +4,9 @@
 * DESCRIPTION: TeXmacs main data structures (buffers, views and windows)
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef TM_WINDOW_H
@@ -26,10 +25,10 @@ public:
   int                sfactor;       // the shrinking factor
 
 protected:
-  object*  texmacs_menu;       // accelerate menu rendering
-  object*  texmacs_icon_menu;  // accelerate icon bar rendering
-  string*  text_ptr;           // where the interactive string is returned
-  command  call_back;          // called when typing finished
+  hashmap<int,object>    menu_current;
+  hashmap<object,widget> menu_cache;
+  string*  text_ptr;  // where the interactive string is returned
+  command  call_back; // called when typing finished
 
 public:
   tm_window_rep (widget wid2, tree geom);
@@ -42,6 +41,7 @@ public:
   inline scheme_tree get_property (scheme_tree what) {
     return props [what]; }
 
+  bool get_menu_widget (int which, string menu, widget& w);
   void menu_main (string menu);
   void menu_icons (int which, string menu);
   void set_header_flag (bool flag);
