@@ -4,10 +4,9 @@
 * DESCRIPTION: Boxes stacked one above another
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Boxes/composite.hpp"
@@ -362,5 +361,5 @@ stack_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
 
 box
 stack_box (path ip, array<box> bs, array<SI> spc) {
-  return new stack_box_rep (ip, bs, spc);
+  return tm_new<stack_box_rep> (ip, bs, spc);
 }
