@@ -4,10 +4,9 @@
 * DESCRIPTION: Bridge between logical and physically typesetted document
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "bridge.hpp"
@@ -23,8 +22,12 @@ bridge bridge_argument (typesetter, tree, path);
 bridge bridge_default (typesetter, tree, path);
 bridge bridge_compound (typesetter, tree, path);
 bridge bridge_mark (typesetter, tree, path);
+bridge bridge_expand_as (typesetter, tree, path);
 bridge bridge_eval (typesetter, tree, path);
 bridge bridge_auto (typesetter, tree, path, tree, bool);
+bridge bridge_locus (typesetter, tree, path);
+bridge bridge_ornament (typesetter, tree, path);
+bridge bridge_canvas (typesetter, tree, path);
 
 bridge nil_bridge;
 
@@ -83,6 +86,8 @@ make_bridge (typesetter ttt, tree st, path ip) {
     return bridge_argument (ttt, st, ip);
   case MARK:
     return bridge_mark (ttt, st, ip);
+  case EXPAND_AS:
+    return bridge_expand_as (ttt, st, ip);
   case EVAL:
   case QUASI:
     return bridge_eval (ttt, st, ip);
@@ -101,6 +106,15 @@ make_bridge (typesetter ttt, tree st, path ip) {
     return bridge_auto (ttt, st, ip, var_inactive_m, true);
   case REWRITE_INACTIVE:
     return bridge_rewrite (ttt, st, ip);
+  case LOCUS:
+    return bridge_locus (ttt, st, ip);
+  case HLINK:
+  case ACTION:
+    return bridge_compound (ttt, st, ip);
+  case CANVAS:
+    return bridge_canvas (ttt, st, ip);
+  case ORNAMENT:
+    return bridge_ornament (ttt, st, ip);
   default:
     if (L(st) < START_EXTENSIONS) return bridge_default (ttt, st, ip);
     else return bridge_compound (ttt, st, ip);
@@ -140,8 +154,7 @@ bridge_rep::notify_insert (path p, tree u) {
   int  l= last_item (p);
   tree t= subtree (st, q);
   if (is_atomic (t)) {
-    if (is_compound (u))
-      fatal_error ("two atoms expected", "bridge_rep::notify_insert");
+    ASSERT (is_atomic (u), "two atoms expected");
     t= t->label (0, l) * u->label * t->label (l, N(t->label));
   }
   else t= (t (0, l) * u) * t (l, N(t));
@@ -202,6 +215,11 @@ bridge_rep::notify_join (path p) {
 * Getting environment variables and typesetting
 ******************************************************************************/
 
+void
+bridge_rep::my_clean_links () {
+  link_env= link_repository (true);
+}
+
 void
 bridge_rep::my_exec_until (path p) {
   env->exec_until (st, p);
@@ -269,12 +287,16 @@ bridge_rep::typeset (int desired_status) {
     // cout << "Typesetting " << st << ", " << desired_status << "\n";
     // cout << "  recomputing\n";
     hashmap<string,tree> prev_back (UNINIT);
+    my_clean_links ();
+    link_repository old_link_env= env->link_env;
+    env->link_env= link_env;
     ttt->local_start (l, sb);
     env->local_start (prev_back);
     my_typeset (desired_status);
     env->local_update (ttt->old_patch, changes);
     env->local_end (prev_back);
     ttt->local_end (l, sb);
+    env->link_env= old_link_env;
     status= desired_status;
     // cout << "  old_patch     = " << ttt->old_patch << "\n";
     // cout << "  changes       = " << changes << "\n";
