@@ -4,10 +4,9 @@
 * DESCRIPTION: Typeset active markup
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "concater.hpp"
@@ -123,8 +122,8 @@ build_locus (edit_env env, tree t, list<string>& ids, string& col) {
   string var= (visited? VISITED_COLOR: LOCUS_COLOR);
   string current_col= env->get_string (COLOR);
   string locus_col= env->get_string (var);
-  if (on_paper && preserve) col= locus_col;
-  else if (locus_col == "preserve") col= current_col;
+  if (locus_col == "preserve") col= current_col;
+  else if (on_paper && preserve) col= locus_col;
   else if (locus_col == "global") col= get_locus_rendering (var);
   else col= locus_col;
 
@@ -154,7 +153,8 @@ concater_rep::typeset_locus (tree t, path ip) {
 void
 concater_rep::typeset_set_binding (tree t, path ip) {
   tree keys= env->exec (t);
-  if (L(keys) == TUPLE) {
+  if (L(keys) == HIDDEN) {
+    keys= keys[1];
     flag ("set binding", ip, blue);
     if (N(keys) > 0) {
       path sip= ip;
@@ -227,6 +227,24 @@ concater_rep::typeset_flag (tree t, path ip) {
 * Typesetting postscript images
 ******************************************************************************/
 
+bool
+is_percentage (tree t, string s) {
+  return
+    is_atomic (t) &&
+    ends (t->label, s) &&
+    is_double (t->label (0, N (t->label) - 1));
+}
+
+bool
+is_percentage (tree t) {
+  return is_percentage (t, "%");
+}
+
+double
+as_percentage (tree t) {
+  return as_double (t->label (0, N (t->label) - 1)) / 100.0;
+}
+
 bool
 is_magnification (string s) {
   double result;
