@@ -6,10 +6,9 @@
 *                - handle brackets and scripts
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef CONCATER_H
@@ -42,7 +41,10 @@ class concater_rep {
 
   // textual markup
   void typeset_substring (string s, path ip, int pos);
-  void typeset_string (string s, path ip);
+  void typeset_colored_substring (string s, path ip, int pos, string col);
+  void typeset_text_string (string s, path ip);
+  void typeset_math_string (string s, path ip);
+  void typeset_prog_string (tree t, path ip);
   void typeset_document (tree t, path ip);
   void typeset_paragraph (tree t, path ip);
   void typeset_surround (tree t, path ip);
@@ -51,6 +53,7 @@ class concater_rep {
   void typeset_space (tree t, path ip);
   void typeset_move (tree t, path ip);
   void typeset_resize (tree t, path ip);
+  void typeset_clipped (tree t, path ip);
   void typeset_float (tree t, path ip);
   void typeset_repeat (tree t, path ip);
   void typeset_formatting (tree t, path ip, string var);
@@ -97,6 +100,7 @@ class concater_rep {
   void typeset_argument (tree t, path ip);
   void typeset_eval_args (tree t, path ip);
   void typeset_mark (tree t, path ip);
+  void typeset_expand_as (tree t, path ip);
   void typeset_dynamic (tree t, path ip);
   void typeset_executable (tree t, path ip);
   void typeset_rewrite (tree t, path ip);
@@ -105,14 +109,10 @@ class concater_rep {
   void typeset_if (tree t, path ip);
   void typeset_var_if (tree t, path ip);
   void typeset_case (tree t, path ip);
-  void typeset_label (tree t, path ip);
-  void typeset_reference (tree t, path ip, int type);
+  void typeset_locus (tree t, path ip);
+  void typeset_set_binding (tree t, path ip);
   void typeset_write (tree t, path ip);
   void typeset_specific (tree t, path ip);
-  void typeset_hyperlink (tree t, path ip);
-  void typeset_action (tree t, path ip);
-  void typeset_tag (tree t, path ip);
-  void typeset_meaning (tree t, path ip);
   void typeset_flag (tree t, path ip);
 
   // animations
@@ -139,6 +139,10 @@ class concater_rep {
   void typeset_fill (tree t, path ip);
   void typeset_postscript (tree t, path ip);
 
+  // markup for graphical user interface
+  void typeset_canvas (tree t, path ip);
+  void typeset_ornament (tree t, path ip);
+
   // postprocessing of brackets and scripts
   int  prec (int i);
   int  succ (int i);
@@ -163,6 +167,7 @@ public:
 
   friend box              typeset_as_concat (edit_env env, tree t, path ip);
   friend box              typeset_as_box (edit_env env, tree t, path ip);
+  friend box              typeset_as_atomic (edit_env env, tree t, path ip);
   friend array<line_item> typeset_concat (edit_env env, tree t, path ip);
   friend array<line_item> typeset_concat_range (edit_env env, tree t, path ip,
 						int i1, int i2);
@@ -171,4 +176,10 @@ public:
 
 typedef concater_rep* concater;
 
+box typeset_as_concat (edit_env env, tree t, path ip);
+array<line_item> typeset_concat (edit_env env, tree t, path ip);
+array<line_item> typeset_concat_range (edit_env env, tree t, path ip,
+				       int i1, int i2);
+array<line_item> typeset_marker (edit_env env, path ip);
+
 #endif // defined CONCATER_H
