@@ -4,10 +4,9 @@
 * DESCRIPTION: mathematical curves
 * COPYRIGHT  : (C) 2003  Joris van der Hoeven and Henri Lesourd
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "path.hpp"
@@ -241,7 +240,7 @@ segment_rep::get_control_points (
 
 curve
 segment (point p1, point p2) {
-  return new segment_rep (p1, p2);
+  return tm_new<segment_rep> (p1, p2);
 }
 
 /******************************************************************************
@@ -295,7 +294,7 @@ poly_segment_rep::get_control_points (
 
 curve
 poly_segment (array<point> a, array<path> cip) {
-  return new poly_segment_rep (a, cip);
+  return tm_new<poly_segment_rep> (a, cip);
 }
 
 /******************************************************************************
@@ -471,8 +470,8 @@ spline_rep::S (
   else if (u<U[i+1]) return p1[i](u);
   else if (u<U[i+2]) return p2[i](u);
   else if (u<U[i+3]) return p3[i](u);
-  else fatal_error ("We should **never** go here");
-  return 0.0; // NOT REACHED
+  else FAILED ("we should **never** go here");
+  return 0.0;
 }
 
 point
@@ -628,10 +627,7 @@ spline_rep::get_control_points (
     for (i=0; i<=(close ? n-2 : n); i++)
       p[i]= evaluate (u[i]);
   }
-  else {
-    fatal_error ("Not yet implemented",
-		 "noninterpolated(spline_rep)::get_control_points");
-  }
+  else FAILED ("not yet implemented");
   abs = u;
   pts = p;
   rcip= cip;
@@ -640,7 +636,7 @@ spline_rep::get_control_points (
 
 curve
 spline (array<point> a, array<path> cip, bool close, bool interpol) {
-  return new spline_rep (a, cip, close, interpol);
+  return tm_new<spline_rep> (a, cip, close, interpol);
 }
 
 /******************************************************************************
@@ -757,7 +753,7 @@ arc_rep::curvature (double t1, double t2) {
 
 curve
 arc (array<point> a, array<path> cip, bool close) {
-  return new arc_rep (a, cip, close);
+  return tm_new<arc_rep> (a, cip, close);
 }
 
 int
@@ -807,7 +803,7 @@ struct compound_curve_rep: public curve_rep {
 curve
 operator * (curve c1, curve c2) {
   // FIXME: we might want to test whether c1(1.0) is approx equal to c2(0.0)
-  return new compound_curve_rep (c1, c2);
+  return tm_new<compound_curve_rep> (c1, c2);
 }
 
 /******************************************************************************
@@ -852,7 +848,7 @@ inverted_curve_rep::get_control_points (
 
 curve
 invert (curve c) {
-  return new inverted_curve_rep (c);
+  return tm_new<inverted_curve_rep> (c);
 }
 
 /******************************************************************************
@@ -873,9 +869,8 @@ struct transformed_curve_rep: public curve_rep {
   }
   point grad (double t, bool& error);
   double curvature (double t1, double t2) {
-    fatal_error ("Not yet implemented",
-	         "transformed_curve_rep::curvature");
-    return 0.0; // NOT REACHED
+    FAILED ("not yet implemented");
+    return 0.0;
   }
   int get_control_points (
     array<double>&abs, array<point>& pts, array<path>& cip);
@@ -889,8 +884,7 @@ transformed_curve_rep::rectify_cumul (array<point>& a, double eps) {
     int i, k= N(b);
     for (i=0; i<k; i++) a << f(b[i]);
   }
-  else fatal_error ("Not yet implemented",
-		    "transformed_curve_rep::rectify_cumul");
+  else FAILED ("not yet implemented");
 }
 
 point
@@ -916,10 +910,10 @@ transformed_curve_rep::get_control_points (
 
 curve
 frame::operator () (curve c) {
-  return new transformed_curve_rep (*this, c);
+  return tm_new<transformed_curve_rep> (*this, c);
 }
 
 curve
 frame::operator [] (curve c) {
-  return new transformed_curve_rep (invert (*this), c);
+  return tm_new<transformed_curve_rep> (invert (*this), c);
 }
