@@ -4,10 +4,9 @@
 * DESCRIPTION: Line breaking facility for paragraphs
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Boxes/construct.hpp"
@@ -27,7 +26,7 @@ struct lb_info_rep: concrete_struct {
 
 struct lb_info {
   CONCRETE(lb_info);
-  lb_info () { rep= new lb_info_rep (); }
+  lb_info () { rep= tm_new<lb_info_rep> (); }
   operator tree () {
     return tuple ((tree) rep->prev,
 		  as_string (rep->pen),
@@ -257,8 +256,8 @@ void
 line_breaker_rep::test_better (path new_pos, path old_pos,
 			       int pen, int pen_spc)
 {
-  // cout << "Test " << new_pos << ", " << old_pos << ", "
-  //      << pen << ", " << pen_spc << "\n";
+  //cout << "Test " << new_pos << ", " << old_pos << ", "
+  //<< pen << ", " << pen_spc << "\n";
   if (!best->contains (new_pos)) best (new_pos)= lb_info ();
   lb_info cur= best [new_pos];
   if ((pen < cur->pen) ||
@@ -266,6 +265,7 @@ line_breaker_rep::test_better (path new_pos, path old_pos,
     cur->prev   = old_pos;
     cur->pen    = pen;
     cur->pen_spc= min (pen_spc, 1000000000);
+    //cout << "  Better\n";
   }
 }
 
@@ -417,6 +417,8 @@ line_breaker_rep::compute_breaks () {
     for (i=start; i<end; i++)
       process (path (i));
 
+  test_better (path (end), path (start), HYPH_INVALID, 999999999);
+
   array<path> ap (0);
   get_breaks (ap, path (end));
 
@@ -448,8 +450,8 @@ line_breaks (array<line_item> a, int start, int end,
   int tol= 5;         // extra tolerance of 5tmpt avoid rounding errors when
   line_width += tol;  // the widths of the boxes sum up to precisely 1par
   line_breaker_rep* H=
-    new line_breaker_rep (a, start, end, line_width, first_spc, last_spc);
+    tm_new<line_breaker_rep> (a, start, end, line_width, first_spc, last_spc);
   array<path> ap= ragged? H->compute_ragged_breaks (): H->compute_breaks ();
-  delete H;
+  tm_delete (H);
   return ap;
 }
