@@ -4,16 +4,16 @@
 * DESCRIPTION: edit environment for typesetting
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef ENV_H
 #define ENV_H
 #include "vars.hpp"
 #include "drd_info.hpp"
+#include "gui.hpp"
 #include "font.hpp"
 #include "language.hpp"
 #include "path.hpp"
@@ -21,6 +21,7 @@
 #include "boxes.hpp"
 #include "url.hpp"
 #include "Graphics/frame.hpp"
+#include "link.hpp"
 
 #define DECORATION (-1)
 
@@ -93,12 +94,6 @@
 #define INACTIVE_BLOCK_ONCE      4
 #define INACTIVE_BLOCK_ERROR     5
 
-#define MODIFY_NONE              0
-#define MODIFY_COMPACT           1
-#define MODIFY_LONG              2
-#define MODIFY_PERMANENT_COMPACT 3
-#define MODIFY_PERMANENT_LONG    4
-
 /******************************************************************************
 * Other enumerated values
 ******************************************************************************/
@@ -115,7 +110,6 @@
 class edit_env;
 class edit_env_rep: public concrete_struct {
 public:
-  display                      dis;
   drd_info&                    drd;
 private:
   hashmap<string,tree>         env;
@@ -129,12 +123,14 @@ public:
   hashmap<string,int>&         var_type;
   url                          base_file_name;
   url                          cur_file_name;
+  bool                         secure;
   hashmap<string,tree>&        local_ref;
   hashmap<string,tree>&        global_ref;
   hashmap<string,tree>&        local_aux;
   hashmap<string,tree>&        global_aux;
   bool                         complete;    // typeset complete document ?
   bool                         read_only;   // write-protected ?
+  link_repository              link_env;
 
   int          dpi;
   double       inch;
@@ -212,6 +208,7 @@ private:
   tree exec_get_arity (tree t);
   tree exec_eval_args (tree t);
   bool exec_until_mark (tree t, path p, string var, int level);
+  bool exec_until_quasi (tree t, path p, string var, int level);
   tree exec_quasiquoted (tree t);
   tree exec_copy (tree t);
   tree exec_if (tree t);
@@ -222,6 +219,7 @@ private:
   bool exec_until_while (tree t, path p, string var, int level);
   tree exec_for_each (tree t);
   tree exec_use_package (tree t);
+  tree exec_use_module (tree t);
 
   tree exec_or (tree t);
   tree exec_xor (tree t);
@@ -231,6 +229,13 @@ private:
   tree exec_times_over (tree t);
   tree exec_divide (tree t);
   tree exec_modulo (tree t);
+  tree exec_math_sqrt (tree t);
+  tree exec_exp (tree t);
+  tree exec_log (tree t);
+  tree exec_pow (tree t);
+  tree exec_cos (tree t);
+  tree exec_sin (tree t);
+  tree exec_tan (tree t);
   tree exec_merge (tree t);
   tree exec_length (tree t);
   tree exec_range (tree t);
@@ -267,6 +272,8 @@ private:
   tree exec_fn_length ();
   tree exec_fns_length ();
   tree exec_bls_length ();
+  tree exec_fnbot_length ();
+  tree exec_fntop_length ();
   tree exec_spc_length ();
   tree exec_xspc_length ();
   tree exec_par_length ();
@@ -280,6 +287,13 @@ private:
   tree exec_min_length ();
   tree exec_h_length ();
 
+  tree exec_hard_id (tree t);
+  tree exec_script (tree t);
+  tree exec_set_binding (tree t);
+  tree exec_get_binding (tree t);
+
+  tree exec_pattern (tree t);
+
   tree exec_point (tree t);
   tree exec_box_info (tree t);
   tree exec_frame_direct (tree t);
@@ -303,8 +317,7 @@ private:
   tree rewrite_inactive (tree t, tree var);
 
 public:
-  edit_env_rep (display dis,
-		drd_info& drd,
+  edit_env_rep (drd_info& drd,
 		url base_file_name,
 		hashmap<string,tree>& local_ref,
 		hashmap<string,tree>& global_ref,
@@ -317,7 +330,7 @@ public:
   void   exec_until (tree t, path p);
   bool   exec_until (tree t, path p, string var, int level);
   string exec_string (tree t);        /* should be inline */
-  tree   expand (tree t);
+  tree   expand (tree t, bool search_accessible= false);
   bool   depends (tree t, string s, int level);
   tree   rewrite (tree t);
 
@@ -411,6 +424,9 @@ public:
   inline space get_vspace (string var) {
     tree t= env [var];
     return as_vspace (t); }
+  inline color get_color (string var) {
+    tree t= env [var];
+    return named_color (as_string (t)); }
 
   friend class edit_env;
   friend ostream& operator << (ostream& out, edit_env env);
@@ -420,8 +436,7 @@ class edit_env {
   CONCRETE_NULL(edit_env);
   inline edit_env (edit_env_rep* rep2):
     rep(rep2) { INC_COUNT_NULL (this->rep); }
-  edit_env (display dis,
-	    drd_info& drd,
+  edit_env (drd_info& drd,
 	    url base_file_name,
 	    hashmap<string,tree>& local_ref,
 	    hashmap<string,tree>& global_ref,
@@ -430,8 +445,15 @@ class edit_env {
 };
 CONCRETE_NULL_CODE(edit_env);
 
+ostream& operator << (ostream& out, edit_env env);
 tree texmacs_exec (edit_env env, tree cmd);
 void extract_format (tree fm, tree* r, int n);
 tree load_inclusion (url u); // implemented in tm_file.cpp
+tree tree_extents (tree t);
+bool is_percentage (tree t, string s);
+bool is_percentage (tree t);
+double as_percentage (tree t);
+bool is_magnification (string s);
+double get_magnification (string s);
 
 #endif // defined ENV_H
